Package: magick
Type: Package
Title: Advanced Graphics and Image-Processing in R
Version: 2.9.1
Authors@R: person("Jeroen", "Ooms", role = c("aut", "cre"), email = "jeroenooms@gmail.com",
    comment = c(ORCID = "0000-0002-4035-0289"))
Description: Bindings to 'ImageMagick': the most comprehensive open-source image
    processing library available. Supports many common formats (png, jpeg, tiff,
    pdf, etc) and manipulations (rotate, scale, crop, trim, flip, blur, etc).
    All operations are vectorized via the Magick++ STL meaning they operate either
    on a single frame or a series of frames for working with layers, collages,
    or animation. In RStudio images are automatically previewed when printed to
    the console, resulting in an interactive editing environment. Also includes
    a graphics device for creating drawing onto images using pixel coordinates.
License: MIT + file LICENSE
URL: https://docs.ropensci.org/magick/
        https://ropensci.r-universe.dev/magick
BugReports: https://github.com/ropensci/magick/issues
SystemRequirements: ImageMagick++: ImageMagick-c++-devel (rpm) or
        libmagick++-dev (deb)
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Imports: Rcpp (>= 0.12.12), magrittr, curl
LinkingTo: Rcpp
Suggests: av, spelling, jsonlite, methods, knitr, rmarkdown, rsvg,
        webp, pdftools, ggplot2, gapminder, IRdisplay, tesseract,
        gifski
Encoding: UTF-8
RoxygenNote: 7.3.3
Language: en-US
NeedsCompilation: yes
Packaged: 2026-02-28 14:23:36 UTC; jeroen
Author: Jeroen Ooms [aut, cre] (ORCID: <https://orcid.org/0000-0002-4035-0289>)
Maintainer: Jeroen Ooms <jeroenooms@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-28 15:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:03:37 UTC; unix
Archs: magick.so.dSYM

Package: magick
Type: Package
Title: Advanced Graphics and Image-Processing in R
Version: 2.7.5
Authors@R: person("Jeroen", "Ooms", role = c("aut", "cre"), email = "jeroen@berkeley.edu",
    comment = c(ORCID = "0000-0002-4035-0289"))
Description: Bindings to 'ImageMagick': the most comprehensive open-source image
    processing library available. Supports many common formats (png, jpeg, tiff,
    pdf, etc) and manipulations (rotate, scale, crop, trim, flip, blur, etc).
    All operations are vectorized via the Magick++ STL meaning they operate either
    on a single frame or a series of frames for working with layers, collages,
    or animation. In RStudio images are automatically previewed when printed to
    the console, resulting in an interactive editing environment. The latest 
    version of the package includes a native graphics device for creating 
    in-memory graphics or drawing onto images using pixel coordinates.
License: MIT + file LICENSE
URL: https://docs.ropensci.org/magick/ (website)
        https://github.com/ropensci/magick (devel)
BugReports: https://github.com/ropensci/magick/issues
SystemRequirements: ImageMagick++: ImageMagick-c++-devel (rpm) or
        libmagick++-dev (deb)
VignetteBuilder: knitr
Imports: Rcpp (>= 0.12.12), magrittr, curl
LinkingTo: Rcpp
Suggests: av (>= 0.3), spelling, jsonlite, methods, knitr, rmarkdown,
        rsvg, webp, pdftools, ggplot2, gapminder, IRdisplay, tesseract
        (>= 2.0), gifski
Encoding: UTF-8
RoxygenNote: 7.2.3
Language: en-US
NeedsCompilation: yes
Packaged: 2023-08-07 11:37:02 UTC; jeroen
Author: Jeroen Ooms [aut, cre] (<https://orcid.org/0000-0002-4035-0289>)
Maintainer: Jeroen Ooms <jeroen@berkeley.edu>
Repository: CRAN
Date/Publication: 2023-08-07 12:50:05 UTC

Package: magick
Type: Package
Title: Advanced Graphics and Image-Processing in R
Version: 2.9.1
Authors@R: person("Jeroen", "Ooms", role = c("aut", "cre"), email = "jeroenooms@gmail.com",
    comment = c(ORCID = "0000-0002-4035-0289"))
Description: Bindings to 'ImageMagick': the most comprehensive open-source image
    processing library available. Supports many common formats (png, jpeg, tiff,
    pdf, etc) and manipulations (rotate, scale, crop, trim, flip, blur, etc).
    All operations are vectorized via the Magick++ STL meaning they operate either
    on a single frame or a series of frames for working with layers, collages,
    or animation. In RStudio images are automatically previewed when printed to
    the console, resulting in an interactive editing environment. Also includes
    a graphics device for creating drawing onto images using pixel coordinates.
License: MIT + file LICENSE
URL: https://docs.ropensci.org/magick/
        https://ropensci.r-universe.dev/magick
BugReports: https://github.com/ropensci/magick/issues
SystemRequirements: ImageMagick++: ImageMagick-c++-devel (rpm) or
        libmagick++-dev (deb)
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Imports: Rcpp (>= 0.12.12), magrittr, curl
LinkingTo: Rcpp
Suggests: av, spelling, jsonlite, methods, knitr, rmarkdown, rsvg,
        webp, pdftools, ggplot2, gapminder, IRdisplay, tesseract,
        gifski
Encoding: UTF-8
RoxygenNote: 7.3.3
Language: en-US
NeedsCompilation: yes
Packaged: 2026-02-28 14:23:36 UTC; jeroen
Author: Jeroen Ooms [aut, cre] (ORCID: <https://orcid.org/0000-0002-4035-0289>)
Maintainer: Jeroen Ooms <jeroenooms@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-28 15:40:02 UTC

# Generated by roxygen2: do not edit by hand

S3method("[","magick-image")
S3method("[<-","magick-image")
S3method("[[","magick-image")
S3method("[[<-","magick-image")
S3method(as.double,bitmap)
S3method(as.integer,"magick-image")
S3method(as.integer,bitmap)
S3method(as.list,"magick-image")
S3method(as.raster,"magick-image")
S3method(c,"magick-image")
S3method(length,"magick-image")
S3method(plot,"magick-image")
S3method(print,"magick-image")
S3method(print,bitmap)
S3method(rep,"magick-image")
S3method(rev,"magick-image")
export("%>%")
export(as_EBImage)
export(autoviewer_disable)
export(autoviewer_enable)
export(channel_types)
export(coder_info)
export(colorspace_types)
export(compose_types)
export(compress_types)
export(decoration_types)
export(demo_image)
export(dispose_types)
export(distort_types)
export(dump_option_info)
export(filter_types)
export(geometry_area)
export(geometry_point)
export(geometry_size_percent)
export(geometry_size_pixels)
export(granite)
export(gravity_types)
export(image_animate)
export(image_annotate)
export(image_append)
export(image_apply)
export(image_attributes)
export(image_average)
export(image_background)
export(image_blank)
export(image_blur)
export(image_border)
export(image_browse)
export(image_canny)
export(image_capture)
export(image_channel)
export(image_charcoal)
export(image_chop)
export(image_coalesce)
export(image_colorize)
export(image_combine)
export(image_comment)
export(image_compare)
export(image_compare_dist)
export(image_composite)
export(image_connect)
export(image_contrast)
export(image_convert)
export(image_convolve)
export(image_crop)
export(image_data)
export(image_deskew)
export(image_deskew_angle)
export(image_despeckle)
export(image_destroy)
export(image_device)
export(image_display)
export(image_distort)
export(image_draw)
export(image_edge)
export(image_emboss)
export(image_enhance)
export(image_equalize)
export(image_extent)
export(image_fft)
export(image_fill)
export(image_flatten)
export(image_flip)
export(image_flop)
export(image_frame)
export(image_fuzzycmeans)
export(image_fx)
export(image_fx_sequence)
export(image_get_artifact)
export(image_ggplot)
export(image_graph)
export(image_hough_draw)
export(image_hough_txt)
export(image_implode)
export(image_info)
export(image_join)
export(image_lat)
export(image_level)
export(image_map)
export(image_median)
export(image_modulate)
export(image_montage)
export(image_morph)
export(image_morphology)
export(image_mosaic)
export(image_motion_blur)
export(image_negate)
export(image_noise)
export(image_normalize)
export(image_ocr)
export(image_ocr_data)
export(image_oilpaint)
export(image_ordered_dither)
export(image_orient)
export(image_page)
export(image_quantize)
export(image_raster)
export(image_read)
export(image_read_pdf)
export(image_read_svg)
export(image_read_video)
export(image_reducenoise)
export(image_repage)
export(image_resize)
export(image_rotate)
export(image_sample)
export(image_scale)
export(image_separate)
export(image_set_defines)
export(image_shade)
export(image_shadow)
export(image_shadow_mask)
export(image_shear)
export(image_split)
export(image_strip)
export(image_threshold)
export(image_transparent)
export(image_trim)
export(image_types)
export(image_virtual_pixel)
export(image_write)
export(image_write_gif)
export(image_write_video)
export(kernel_types)
export(logo)
export(magick_config)
export(magick_fonts)
export(magick_options)
export(magick_set_seed)
export(metric_types)
export(morphology_types)
export(noise_types)
export(option_types)
export(orientation_types)
export(rose)
export(style_types)
export(virtual_pixel_methods)
export(wizard)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,as.raster)
importFrom(graphics,plot)
importFrom(magrittr,"%>%")
useDynLib(magick)

#' Morphology
#'
#' Apply a morphology method. This is a very flexible function which
#' can be used to apply any morphology method with custom parameters.
#' See [imagemagick website](https://legacy.imagemagick.org/Usage/morphology/)
#' for examples.
#'
#' @export
#' @inheritParams effects
#' @param method a string with a valid method from [morphology_types()]
#' @param opts a named list or character vector with custom attributes
#' @rdname morphology
#' @name morphology
#' @family image
#' @examples #example from IM website:
#' if(magick_config()$version > "6.8.8"){
#' pixel <- image_blank(1, 1, 'white') %>% image_border('black', '5x5')
#'
#' # See the effect of Dilate method
#' pixel %>% image_scale('800%')
#' pixel %>% image_morphology('Dilate', "Diamond") %>% image_scale('800%')
#'
#' # These produce the same output:
#' pixel %>% image_morphology('Dilate', "Diamond", iter = 3) %>% image_scale('800%')
#' pixel %>% image_morphology('Dilate', "Diamond:3") %>% image_scale('800%')
#'
#' # Plus example
#' pixel %>% image_morphology('Dilate', "Plus", iterations = 2) %>% image_scale('800%')
#'
#' # Rose examples
#' rose %>% image_morphology('ErodeI', 'Octagon', iter = 3)
#' rose %>% image_morphology('DilateI', 'Octagon', iter = 3)
#' rose %>% image_morphology('OpenI', 'Octagon', iter = 3)
#' rose %>% image_morphology('CloseI', 'Octagon', iter = 3)
#'
#' # Edge detection
#' man <- demo_image('man.gif')
#' man %>% image_morphology('EdgeIn', 'Octagon')
#' man %>% image_morphology('EdgeOut', 'Octagon')
#' man %>% image_morphology('Edge', 'Octagon')
#'
#' # Octagonal Convex Hull
#'  man %>%
#'    image_morphology('Close', 'Diamond') %>%
#'    image_morphology('Thicken', 'ConvexHull', iterations = 1)
#'
#' # Thinning down to a Skeleton
#' man %>% image_morphology('Thinning', 'Skeleton', iterations = 1)
#'
#' # Specify custom kernel matrix usingn a string:
#' img <- demo_image("test_mag.gif")
#' i <- image_convolve(img, kernel = '4x5:
#'        0 -1  0  0
#'       -1 +1 -1  0
#'       -1 +1 -1  0
#'       -1 +1 +1 -1
#'        0 -1 -1  0 ', bias = "50%")
#' }
image_morphology <- function(image, method = "convolve", kernel = "Gaussian",
                             iterations = 1, opts = list()){
  assert_image(image)
  method <- as.character(method)
  kernel <- as.character(kernel)
  opt_names <- as.character(names(opts))
  opt_values <- as.character(unname(opts))
  magick_image_morphology(image, method, kernel, iterations, opt_names, opt_values)
}

#' @export
#' @rdname morphology
#' @param kernel either a square matrix or a string. The string can either be a
#' parameterized [kerneltype][kernel_types] such as: `"DoG:0,0,2"` or `"Diamond"`
#' or it can contain a custom matrix (see examples)
#' @param iterations number of iterations
#' @param scaling string with kernel scaling. The special flag `"!"` automatically scales to full
#' dynamic range, for example: \code{"50\%!"}
#' @param bias output bias string, for example \code{"50\%"}
image_convolve <- function(image, kernel = 'Gaussian', iterations = 1, scaling = NULL, bias = NULL){
  assert_image(image)
  iterations <- as.integer(iterations)
  scaling <- as.character(scaling)
  bias <- as.character(bias)
  if(is.character(kernel)){
    magick_image_convolve_kernel(image, kernel, iterations, scaling, bias)
  } else if(is.matrix(kernel)) {
    if(nrow(kernel) != ncol(kernel))
      stop("Kernel matrix must be squared")
    kernel <- t(kernel) #See issue #126
    if(iterations < 1)
      stop("Convolve with matrix kernel does not support negative iterations")
    magick_image_convolve_matrix(image, kernel, iterations, scaling, bias)
  } else {
    stop("Argument 'kernel' must either be a string or matrix")
  }
}

#' Magick Options
#'
#' List option types and values supported in your version of ImageMagick. For
#' descriptions see
#' [ImageMagick Enumerations](https://www.imagemagick.org/Magick++/Enumerations.html).
#'
#' @rdname options
#' @family image
#' @name options
#' @export
#' @references ImageMagick Manual: [Enumerations](https://www.imagemagick.org/Magick++/Enumerations.html)
magick_options <- function(){
  types <- option_types()
  types <- types[types != "Undefined"]
  types <- types[types != "Command"]
  Filter(length, sapply(types, list_options))
}

#' @export
#' @rdname options
option_types <- function(){
  list_options("List")
}

#' @export
#' @rdname options
filter_types <- function(){
  list_options('filter')
}

#' @export
#' @rdname options
metric_types <- function(){
  list_options('metric')
}

#' @export
#' @rdname options
dispose_types <- function(){
  list_options('dispose')
}

#' @export
#' @rdname options
compose_types <- function(){
  list_options('compose')
}

#' @export
#' @rdname options
colorspace_types <- function(){
  list_options('colorspace')
}

#' @export
#' @rdname options
channel_types <- function(){
  list_options('channel')
}

#' @export
#' @rdname options
image_types <- function(){
  list_options('type')
}

#' @export
#' @rdname options
kernel_types <- function(){
  list_options('kernel')
}

#' @export
#' @rdname options
noise_types <- function(){
  list_options('noise')
}

#' @export
#' @rdname options
gravity_types <- function(){
  list_options('gravity')
}

#' @export
#' @rdname options
orientation_types <- function(){
  list_options('orientation')
}

#' @export
#' @rdname options
morphology_types <- function(){
  list_options('morphology')
}

#' @export
#' @rdname options
style_types <- function(){
  list_options('style')
}

#' @export
#' @rdname options
decoration_types <- function(){
  list_options('decoration')
}

#' @export
#' @rdname options
compress_types <- function(){
  list_options('compress')
}

#' @export
#' @rdname options
distort_types <- function(){
  list_options('distort')
}

#' @export
"[.magick-image" <- function(x, i){
  assert_image(x)
  stopifnot(is.numeric(i))
  i <- seq_along(x)[i]  # normalize to positive subscripts
  magick_image_subset(x, i)
}

#' @export
"[<-.magick-image" <- function(x, i, value){
  assert_image(x)
  assert_image(value)
  stopifnot(is.numeric(i))
  i <- seq_along(x)[i]
  magick_image_replace(x, i, value)
}

#TODO: return 3 ch 'rgb' or 1 ch greyscale bitmap depending on colorspace
#' @export
"[[.magick-image" <- function(x, i){
  assert_image(x)
  image_data(x[i])
}

#' @export
"[[<-.magick-image" <- function(x, i, value){
  stop("[[ assignment not implemented. Try single bracket.")
}

#' @export
"print.bitmap" <- function(x, ...){
  dims <- dim(x)
  cat(sprintf("%d channel %dx%d bitmap array:", dims[1], dims[2], dims[3]))
  utils::str(x)
}

#' @export
"as.integer.bitmap" <- function(x, transpose = TRUE, ...){
  if(transpose){
    x <- aperm(x)
  }
  structure(as.vector(x, mode = 'integer'), dim = dim(x))
}

#' @export
"as.double.bitmap" <- function(x, transpose = TRUE, ...){
  if(transpose){
    x <- aperm(x)
  }
  structure(as.vector(x, mode = 'double') / 255, dim = dim(x))
}

#' @export
"as.integer.magick-image" <- function(x, ...){
  magick_image_write_integer(x)
}

#' @export
"c.magick-image" <- function(...){
  image_join(...)
}

#' @export
"length.magick-image" <- function(x){
  if(magick_image_dead(x))
    return(NULL)
  assert_image(x)
  magick_image_length(x)
}

#' @export
"rev.magick-image" <- function(x){
  assert_image(x)
  magick_image_rev(x)
}

#' @export
"as.list.magick-image" <- function(x, ...){
  assert_image(x)
  len <- length(x)
  lapply(seq_len(len), magick_image_subset, image = x)
}

#' @export
"rep.magick-image" <- function(x, ...){
  assert_image(x)
  rep_magick_image(x, ...)
}

rep_magick_image <- function(x, times){
  image_join(lapply(seq_len(times), function(...){
    x
  }))
}

#' @export
"print.magick-image" <- function(x, info = TRUE, ...){
  img <- x
  is_knit_image <- isTRUE(getOption('knitr.in.progress'))
  if(isTRUE(getOption('jupyter.in_kernel'))){
    jupyter_print_image(img)
  } else if(!is_knit_image && !magick_image_dead(x) && length(img)){
    previewer <- getOption('magick.viewer')
    if(is.function(previewer)){
      previewer(img)
    }
  }
  if(isTRUE(info))
    print(image_info(x))
  if(is_knit_image)
    `knit_print.magick-image`(x)
  else
    invisible()
}

# This is registered as an S3 method in .onLoad()
"knit_print.magick-image" <- function(x, ...){
  if(!length(x))
    return(invisible())
  plot_counter <- utils::getFromNamespace('plot_counter', 'knitr')
  in_base_dir <- utils::getFromNamespace('in_base_dir', 'knitr')
  ext <- ifelse(all(tolower(image_info(x)$format) == "gif"), "gif", "png")
  tmp <- knitr::fig_path(ext, number = plot_counter())

  # save relative to 'base' directory, see discussion in #110
  in_base_dir({
    dir.create(dirname(tmp), showWarnings = FALSE, recursive = TRUE)
    image_write(x, path = tmp, format = ext)
  })
  knitr::include_graphics(tmp)
}

#' @export
#' @importFrom grDevices as.raster
"as.raster.magick-image" <- function(x, native = FALSE, ...){
  assert_image(x)
  if(isTRUE(native))
    return(as.integer(x))
  bitmap <- image_write_frame(x, format = "rgba")
  magick_image_as_raster(bitmap)
}

#' @export
#' @importFrom graphics plot
"plot.magick-image" <- function(x, ...){
  plot(as.raster(x), ...)
}

image_preview <- function(img, max_width = 800, max_len = 10, viewer = getOption('viewer')){
  if(is.function(viewer)){
    viewer_supported <- c("bmp", "png", "jpeg", "jpg", "svg", "gif", "webp")
    format <- tolower(image_info(img[1])$format)
    len <- length(img)
    info <- image_info(img)
    if(len > 1 && format != "gif"){
      if(info$width[1] > max_width){
        img <- image_resize(img, paste0(max_width, 'x'))
      }
      if(len > max_len){
        i <- round(seq(1, len, length.out = max_len))
        img <- img[i]
      } else {
        i <- seq_len(len)
      }
      img <- image_annotate(img, paste0("[preview] frame ", i, "/", len), size = 18, font = 'mono',
                            location = '+10+10', color = 'white', boxcolor = 'black')
      img <- image_animate(img, fps = 1)
      format <- "gif"
    } else if(is.na(match(format, viewer_supported))){
      img <- image_convert(img, "PNG")
      format <- 'png'
    }
    tmp <- file.path(tempdir(), paste0("preview.", format))
    image_write(img, path = tmp, format = format)
    viewer(tmp)
  }
}

jupyter_print_image <- function(img){
  if(!length(img))
    return()
  format <- tolower(image_info(img[1])$format)
  if(!(format %in% c("png", "jpg", "jpeg", "svg", "gif")))
    format <- "png"
  tmp <- image_write(img, format = format)
  switch (format,
    png = IRdisplay::display_png(tmp),
    jpg = IRdisplay::display_jpeg(tmp),
    jpeg = IRdisplay::display_jpeg(tmp),
    gif = display_gif(tmp),
    svg = IRdisplay::display_svg(rawToChar(tmp))
  )
}

## Placeholder until IRdisplay::display_gif() is available
display_gif <- function(buf){
  contents <- jsonlite::base64_enc(buf)
  IRdisplay::display_html(sprintf('<img src="data:image/gif;base64,%s">', contents))
}

#' Magick Configuration
#'
#' ImageMagick can be configured to support various additional tool and formats
#' via external libraries. These functions show which features ImageMagick supports
#' on your system.
#'
#' Note that \code{coder_info} raises an error for unsupported formats.
#'
#' @export
#' @rdname config
#' @param format image format such as \code{png}, \code{tiff} or \code{pdf}.
#' @references \url{https://www.imagemagick.org/Magick++/CoderInfo.html}
#' @examples coder_info("png")
#' coder_info("jpg")
#' coder_info("pdf")
#' coder_info("tiff")
#' coder_info("gif")
coder_info <- function(format){
  magick_coder_info(format)
}

#' @rdname config
#' @export
magick_config <- function(){
  out <- magick_config_internal()
  out$version = numeric_version(out$version)
  out$threads = magick_threads()
  out
}

#' @rdname config
#' @export
#' @param seed integer with seed value to use
#' @examples # Reproduce random image
#' magick_set_seed(123)
#' image_blank(200,200, pseudo_image = "plasma:fractal")
magick_set_seed <- function(seed){
  stopifnot(length(seed) == 1)
  seed <- as.integer(seed)
  set_magick_seed(seed)
}

#' Magick Graphics Device
#'
#' Graphics device that produces a Magick image. Can either be used like a regular
#' device for making plots, or alternatively via \code{image_draw} to open a device
#' which draws onto an existing image using pixel coordinates. The latter is vectorized,
#' i.e. drawing operations are applied to each frame in the image.
#'
#' The device is a relatively recent feature of the package. It should support all
#' operations but there might still be small inaccuracies. Also it is a bit slower than
#' some of the other devices, in particular for rendering text and clipping. Hopefully
#' this can be optimized in the next version.
#'
#' By default \code{image_draw} sets all margins to 0 and uses graphics coordinates to
#' match image size in pixels (width x height) where \code{(0,0)} is the top left corner.
#' Note that this means the y axis increases from top to bottom which is the opposite
#' of typical graphics coordinates.  You can override all this by passing custom
#' \code{xlim}, \code{ylim} or \code{mar} values to \code{image_draw}.
#'
#' The \code{image_capture} function returns the current device as an image. This only
#' works if the current device is a magick device or supports \link{dev.capture}.
#'
#' @export
#' @aliases image_device
#' @family image
#' @rdname device
#' @name device
#' @param width in pixels
#' @param height in pixels
#' @param bg background color
#' @param pointsize size of fonts
#' @param res resolution in pixels
#' @param clip enable clipping in the device. Because clipping can slow things down
#' a lot, you can disable it if you don't need it.
#' @param antialias TRUE/FALSE: enables anti-aliasing for text and strokes
#' @param ... additional device parameters passed to \link{plot.window} such as
#' \code{xlim}, \code{ylim}, or \code{mar}.
#' @examples # Regular image
#' frink <- image_read("https://jeroen.github.io/images/frink.png")
#'
#' # Produce image using graphics device
#' fig <- image_graph(res = 96)
#' ggplot2::qplot(mpg, wt, data = mtcars, colour = cyl)
#' dev.off()
#'
#' # Combine
#' out <- image_composite(fig, frink, offset = "+70+30")
#' print(out)
#'
#' # Or paint over an existing image
#' img <- image_draw(frink)
#' rect(20, 20, 200, 100, border = "red", lty = "dashed", lwd = 5)
#' abline(h = 300, col = 'blue', lwd = '10', lty = "dotted")
#' text(10, 250, "Hoiven-Glaven", family = "monospace", cex = 4, srt = 90)
#' palette(rainbow(11, end = 0.9))
#' symbols(rep(200, 11), seq(0, 400, 40), circles = runif(11, 5, 35),
#'   bg = 1:11, inches = FALSE, add = TRUE)
#' dev.off()
#' print(img)
#'
#' \donttest{
#' # Vectorized example with custom coordinates
#' earth <- image_read("https://jeroen.github.io/images/earth.gif")
#' img <- image_draw(earth, xlim = c(0,1), ylim = c(0,1))
#' rect(.1, .1, .9, .9, border = "red", lty = "dashed", lwd = 5)
#' text(.5, .9, "Our planet", cex = 3, col = "white")
#' dev.off()
#' print(img)
#' }
image_graph <- function(width = 800, height = 600, bg = "white", pointsize = 12, res = 72,
                        clip = TRUE, antialias = TRUE) {
  img <- magick_device_internal(bg = bg, width = width, height = height, pointsize = pointsize,
                                res = res, clip = clip, antialias = antialias, drawing = FALSE)
  class(img) <- c("magick-device", class(img))
  img
}

#' @export
image_device <- image_graph

#' @rdname device
#' @export
#' @param image an existing image on which to start drawing
image_draw <- function(image, pointsize = 12, res = 72, antialias = TRUE, ...){
  assert_image(image)
  width <- max(image_info(image)$width)
  height <- max(image_info(image)$height)
  antialias <- as.logical(antialias)
  device <- magick_device_internal(bg = "transparent", width = width, height = height, pointsize = pointsize,
                                   res = res, clip = TRUE, antialias = antialias, drawing = TRUE)
  setup_device(list(width = width, height = height), ...)
  magick_image_copy(device, image)
  magick_attr_text_antialias(device, antialias)
  magick_attr_stroke_antialias(device, antialias)
  return(device)
}

#' @export
#' @rdname device
image_capture <- function(){
  which <- grDevices::dev.cur()
  if(which < 2)
    stop("No current open device")
  if(identical(names(which), "magick")){
    magick_device_get(which)
  } else {
    capt <- grDevices::dev.capture(TRUE)
    if(!is.matrix(capt))
      stop("Current device cannot be captured")
    image_read(capt)
  }
}

setup_device <- function(info, xlim = NULL, ylim = NULL, xaxs = "i", yaxs = "i", mar = c(0,0,0,0), ...){
  if(!length(xlim))
    xlim <- c(0, info$width)
  if(!length(ylim))
    ylim <- c(info$height, 0)
  graphics::plot.new()
  graphics::par(mar = mar)
  graphics::plot.window(xlim = xlim, ylim = ylim, xaxs = xaxs, yaxs = yaxs, ...)
}

#' Magick Graphics Device
#'
#' Graphics device that produces a Magick image. Can either be used like a regular
#' device for making plots, or alternatively via \code{image_draw} to open a device
#' which draws onto an existing image using pixel coordinates. The latter is vectorized,
#' i.e. drawing operations are applied to each frame in the image.
#'
#' The device is a relatively recent feature of the package. It should support all
#' operations but there might still be small inaccuracies. Also it is a bit slower than
#' some of the other devices, in particular for rendering text and clipping. Hopefully
#' this can be optimized in the next version.
#'
#' By default \code{image_draw} sets all margins to 0 and uses graphics coordinates to
#' match image size in pixels (width x height) where \code{(0,0)} is the top left corner.
#' Note that this means the y axis increases from top to bottom which is the opposite
#' of typical graphics coordinates.  You can override all this by passing custom
#' \code{xlim}, \code{ylim} or \code{mar} values to \code{image_draw}.
#'
#' The \code{image_capture} function returns the current device as an image. This only
#' works if the current device is a magick device or supports \link{dev.capture}.
#'
#' @export
#' @aliases image_device
#' @family image
#' @rdname device
#' @name device
#' @param width in pixels
#' @param height in pixels
#' @param bg background color
#' @param pointsize size of fonts
#' @param res resolution in pixels
#' @param clip enable clipping in the device. Because clipping can slow things down
#' a lot, you can disable it if you don't need it.
#' @param antialias TRUE/FALSE: enables anti-aliasing for text and strokes
#' @param ... additional device parameters passed to \link{plot.window} such as
#' \code{xlim}, \code{ylim}, or \code{mar}.
#' @examples # Regular image
#' frink <- image_read("https://jeroen.github.io/images/frink.png")
#'
#' # Produce image using graphics device
#' fig <- image_graph(res = 96)
#' ggplot2::qplot(mpg, wt, data = mtcars, colour = cyl)
#' dev.off()
#'
#' # Combine
#' out <- image_composite(fig, frink, offset = "+70+30")
#' print(out)
#'
#' # Or paint over an existing image
#' img <- image_draw(frink)
#' rect(20, 20, 200, 100, border = "red", lty = "dashed", lwd = 5)
#' abline(h = 300, col = 'blue', lwd = '10', lty = "dotted")
#' text(10, 250, "Hoiven-Glaven", family = "monospace", cex = 4, srt = 90)
#' palette(rainbow(11, end = 0.9))
#' symbols(rep(200, 11), seq(0, 400, 40), circles = runif(11, 5, 35),
#'   bg = 1:11, inches = FALSE, add = TRUE)
#' dev.off()
#' print(img)
#'
#' \donttest{
#' # Vectorized example with custom coordinates
#' earth <- image_read("https://jeroen.github.io/images/earth.gif")
#' img <- image_draw(earth, xlim = c(0,1), ylim = c(0,1))
#' rect(.1, .1, .9, .9, border = "red", lty = "dashed", lwd = 5)
#' text(.5, .9, "Our planet", cex = 3, col = "white")
#' dev.off()
#' print(img)
#' }
image_graph <- function(width = 800, height = 600, bg = "white", pointsize = 12, res = 72,
                        clip = TRUE, antialias = TRUE) {
  img <- magick_device_internal(bg = bg, width = width, height = height, pointsize = pointsize,
                                res = res, clip = clip, antialias = antialias, drawing = FALSE)
  class(img) <- c("magick-device", class(img))
  img
}

#' @export
image_device <- image_graph

#' @rdname device
#' @export
#' @param image an existing image on which to start drawing
image_draw <- function(image, pointsize = 12, res = 72, antialias = TRUE, ...){
  assert_image(image)
  width <- max(image_info(image)$width)
  height <- max(image_info(image)$height)
  antialias <- as.logical(antialias)
  device <- magick_device_internal(bg = "transparent", width = width, height = height, pointsize = pointsize,
                                   res = res, clip = TRUE, antialias = antialias, drawing = TRUE)
  setup_device(list(width = width, height = height), ...)
  magick_image_copy(device, image)
  magick_attr_text_antialias(device, antialias)
  magick_attr_stroke_antialias(device, antialias)
  return(device)
}

#' @export
#' @rdname device
image_capture <- function(){
  which <- grDevices::dev.cur()
  if(which < 2)
    stop("No current open device")
  if(identical(names(which), "magick")){
    magick_device_get(which)
  } else {
    capt <- grDevices::dev.capture(TRUE)
    if(!is.matrix(capt))
      stop("Current device cannot be captured")
    image_read(capt)
  }
}

setup_device <- function(info, xlim = NULL, ylim = NULL, xaxs = "i", yaxs = "i", mar = c(0,0,0,0), ...){
  if(!length(xlim))
    xlim <- c(0, info$width)
  if(!length(ylim))
    ylim <- c(info$height, 0)
  graphics::par(mar = mar)
  graphics::plot.new()
  graphics::plot.window(xlim = xlim, ylim = ylim, xaxs = xaxs, yaxs = yaxs, ...)
}

#' Image Segmentation
#'
#' Basic image segmentation like connected components labelling, blob extraction and fuzzy c-means
#'
#' - [image_connect] Connect adjacent pixels with the same pixel intensities to do blob extraction
#' - [image_split] Splits the image according to pixel intensities
#' - [image_fuzzycmeans] Fuzzy c-means segmentation of the histogram of color components
#'
#' [image_connect] performs blob extraction by scanning the image, pixel-by-pixel from top-left
#' to bottom-right where regions of adjacent pixels which share the same set of intensity values
#' get combined.
#'
#' @name segmentation
#' @rdname segmentation
#' @export
#' @family image
#' @details
#' @inheritParams editing
#' @param connectivity number neighbor colors which are considered part of a unique object
#' @examples # Split an image by color
#' img <- image_quantize(logo, 4)
#' layers <- image_split(img)
#' layers
#'
#' # This returns the original image
#' image_flatten(layers)
#'
#' # From the IM website
#' objects <- image_convert(demo_image("objects.gif"), colorspace = "Gray")
#' objects
#'
#' \donttest{
#' # Split image in blobs of connected pixel levels
#' if(magick_config()$version > "6.9.0"){
#' objects %>%
#'   image_connect(connectivity = 4) %>%
#'   image_split()
#'
#' # Fuzzy c-means
#' image_fuzzycmeans(logo)
#'
#' logo %>%
#'   image_convert(colorspace = "HCL") %>%
#'   image_fuzzycmeans(smoothing = 5)
#' }
#' }
image_connect <- function(image, connectivity = 4){
  assert_image(image)
  connectivity <- as.integer(connectivity)
  stopifnot(connectivity >= 0)
  magick_image_connect(image, connectivity)
}

#' @export
#' @rdname segmentation
#' @param keep_color if TRUE the output images retain the color of the input pixel.
#' If FALSE all matching pixels are set black to retain only the image mask.
image_split <- function(image, keep_color = TRUE){
  assert_image(image)
  pixels <- as.integer(image)
  colors <- sort(unique(c(pixels)))
  blobs <- lapply(colors, function(col){ #-256^3 = solid black
    val <- ifelse(isTRUE(keep_color), col, -16777216L)
    image_read((pixels == col) * val)
  })
  image_join(blobs)
}

#' @export
#' @rdname segmentation
#' @param min_pixels the minimum number of pixels contained in a hexahedra before it can be considered valid (expressed as a percentage)
#' @param smoothing the smoothing threshold which eliminates noise in the second derivative of the histogram (higher values gives smoother second derivative)
image_fuzzycmeans <- function(image, min_pixels = 1, smoothing = 1.5){
  assert_image(image)
  min_pixels <- as.numeric(min_pixels)
  smoothing <- as.numeric(smoothing)
  magick_image_fuzzycmeans(image, min_pixels, smoothing)
}

#' Magick Image Processing
#'
#' The `magick` package for graphics and image processing in R. Important resources:
#' - [R introduction vignette](https://docs.ropensci.org/magick/articles/intro.html): getting started
#' - [Magick++ API](https://www.imagemagick.org/Magick++/Image++.html) and
#'     [Magick++ STL](https://www.imagemagick.org/Magick++/STL.html) detailed descriptions of methods and parameters
#'
#' Documentation is split into the following pages:
#' - [analysis] - metrics and calculations: `compare`, `fft`
#' - [animation] - manipulate or combine multiple frames: `animate`,
#'  `morph`, `mosaic`, `montage`, `average`, `append`, `apply`
#' - [attributes] - image properties: `comment`, `info`
#' - [color] - contrast, brightness, colors: `modulate`, `quantize`, `map`, `transparent`,
#'  `background`, `colorize`, `contrast`, `normalize`, `enhance`, `equalize`, `median`
#' - [composite] - advanced joining: `composite`, `border`, `frame`
#' - [device] - creating graphics and drawing on images
#' - [editing] - basic image IO: `read`, `write`, `convert`, `join`, `display`, `brose`
#' - [effects] - fun effects: `despecle`, `reducenoise`, `noise`, `blur`, `charcoal`,
#'   `edge`, `oilpaint`, `emboss`, `implode`, `negate`
#' - [geometry] - specify points, areas and sizes using geometry syntax
#' - [ocr] - extract text from image using [tesseract][tesseract::tesseract] package
#' - [options] - list option types and values supported in your version of ImageMagick
#' - [painting] - flood fill and annotating text
#' - [transform] - shape operations: `trim`, `chop`, `rotate`, `resize`, `scale`, `sample`
#'   `crop`, `flip`, `flop`, `deskew`, `page`
#'
#' @name _index_
#' @rdname magick
#' @family image
#' @aliases magick magick-package imagemagick
NULL

#' Example Images
#' 
#' Example images included with ImageMagick:
#' 
#'  - `logo`: ImageMagick Logo, 640x480
#'  - `wizard`: ImageMagick Wizard, 480x640
#'  - `rose` : Picture of a rose, 70x46	
#'  - `granite` : Granite texture pattern, 128x128
#' 
#' @rdname wizard
#' @name wizard
#' @aliases logo rose granite
#' @export logo wizard rose granite
delayedAssign('logo', image_read("logo:"))
delayedAssign('wizard', image_read("wizard:"))
delayedAssign('rose', image_read("rose:"))
delayedAssign('granite', image_read("granite:"))

#' Image Attributes
#'
#' Attributes are properties of the image that might be present on some images
#' and might affect image manipulation methods.
#'
#' Each attribute can be get and set with the same function. The [image_info()]
#' function returns a data frame with some commonly used attributes.
#'
#' @export
#' @family image
#' @inheritParams editing
#' @name attributes
#' @rdname attributes
#' @param comment string to set an image comment
image_comment <- function(image, comment = NULL){
  assert_image(image)
  comment <- as.character(comment)
  magick_attr_comment(image, comment)
}

#' @export
#' @rdname attributes
image_info <- function(image){
  assert_image(image)
  df <- magick_image_info(image)
  df_to_tibble(df)
}

#' Geometry Helpers
#'
#' ImageMagick uses a handy geometry syntax to specify coordinates and shapes
#' for use in image transformations. You can either specify these manually as
#' strings or use the helper functions below.
#'
#' See [ImageMagick Manual](http://www.imagemagick.org/Magick++/Geometry.html)
#' for details about the syntax specification.
#' Examples of `geometry` strings:
#'  - __`"500x300"`__       -- *Resize image keeping aspect ratio, such that width does not exceed 500 and the height does not exceed 300.*
#'  - __`"500x300!"`__      -- *Resize image to 500 by 300, ignoring aspect ratio*
#'  - __`"500x"`__          -- *Resize width to 500 keep aspect ratio*
#'  - __`"x300"`__          -- *Resize height to 300 keep aspect ratio*
#'  - __\code{"50\%x20\%"}__ -- *Resize width to 50 percent and height to 20 percent of original*
#'  - __`"500x300+10+20"`__ -- *Crop image to 500 by 300 at position 10,20*
#'
#' @export
#' @family image
#' @rdname geometry
#' @name geometry
#' @param x left offset in pixels
#' @param y top offset in pixels
#' @param width in pixels
#' @param height in pixels
#' @param x_off offset in pixels on x axis
#' @param y_off offset in pixels on y axis
#' @param preserve_aspect if FALSE, resize to width and height exactly, loosing original
#' aspect ratio. Only one of `percent` and `preserve_aspect` may be `TRUE`.
#' @examples # Specify a point
#' logo <- image_read("logo:")
#' image_annotate(logo, "Some text", location = geometry_point(100, 200), size = 24)
#'
geometry_point <- function(x, y){
  sprintf("%+d%+d", as.integer(x), as.integer(y))
}

#' @export
#' @rdname geometry
#' @examples # Specify image area
#' image_crop(logo, geometry_area(300, 300), repage = FALSE)
#' image_crop(logo, geometry_area(300, 300, 100, 100), repage = FALSE)
#'
geometry_area <- function(width = NULL, height = NULL, x_off = 0, y_off = 0){
  wstr <- sprintf("%d", as.integer(width))
  hstr <- sprintf("x%d", as.integer(height))
  offstr <- if(x_off || y_off)
    geometry_point(x_off, y_off)
  paste0(wstr, hstr, offstr)
}

#' @export
#' @rdname geometry
#' @examples # Specify image size
#' image_resize(logo, geometry_size_pixels(300))
#' image_resize(logo, geometry_size_pixels(height = 300))
#' image_resize(logo, geometry_size_pixels(300, 300, preserve_aspect = FALSE))
#'
geometry_size_pixels <- function(width = NULL, height = NULL, preserve_aspect = TRUE){
  if(!length(width) && !length(height))
    stop("Either 'width' or 'height' must be set")
  wstr <- sprintf("%d", as.integer(width))
  hstr <- sprintf("x%d", as.integer(height))
  arstr <- if(!isTRUE(preserve_aspect)) "!"
  paste0(wstr, hstr, arstr)
}

#' @export
#' @rdname geometry
#' @examples # resize relative to current size
#' image_resize(logo, geometry_size_percent(50))
#' image_resize(logo, geometry_size_percent(50, 20))
#'
geometry_size_percent <- function(width = 100, height = NULL){
  if(width == height || !length(height)){
    paste0(width, "%")
  } else {
    paste0(width, "x", height, "%")
  }
}

#' Set encoder defines
#'
#' So called 'defines' are properties that are passed along to external
#' filters and libraries. Usually defines are used in [image_read] or
#' [image_write] to control the image encoder/decoder, but you can also
#' set these manually on the image object.
#'
#' The defines values must be a character string, where the names contain
#' the defines keys. Each name must be of the format "enc:key" where the
#' first part is the encoder or filter to which the key is passed. For
#' example `"png:...."` defines can control the encoding and decoding of
#' png images.
#'
#' The [image_set_defines] function does not make a copy of the image, so
#' the defined values remain in the image object until they are overwritten
#' or unset.
#'
#' @family image
#' @export
#' @inheritParams editing
#' @param defines a named character vector with extra options to control reading.
#' These are the `-define key{=value}` settings in the [command line tool](http://www.imagemagick.org/script/command-line-options.php#define).
#' Use an empty string for value-less defines, and NA to unset a define.
#' @rdname defines
#' @name defines
#' @examples # Write an image
#' x <- image_read("https://jeroen.github.io/images/frink.png")
#' image_write(x, "frink.png")
#'
#' # Pass some properties to PNG encoder
#' defines <- c("png:compression-filter" = "1", "png:compression-level" = "0")
#' image_set_defines(x, defines)
#' image_write(x, "frink-uncompressed.png")
#'
#' # Unset properties
#' defines[1:2] = NA
#' image_set_defines(x, defines)
#' image_write(x, "frink-final.png")
#'
#' # Compare size and cleanup
#' file.info(c("frink.png", "frink-uncompressed.png", "frink-final.png"))
#' unlink(c("frink.png", "frink-uncompressed.png", "frink-final.png"))
image_set_defines <- function(image, defines){
  assert_image(image)
  validate_defines(defines)
  for(i in seq_along(defines)){
    val <- defines[i]
    nm <- strsplit(names(val), ":", fixed = TRUE)[[1]]
    magick_image_set_define(image, nm[1], nm[2], val)
  }
}

validate_defines <- function(defines){
  if(length(defines)){
    if(!is.character(defines))
      stop("Argumet 'defines' must be named character vector")
    nms <- names(defines)
    if(length(unique(nms)) != length(defines))
      stop("Argument 'defines' does not have proper names")
    if(any(vapply(strsplit(nms, ":", fixed = TRUE), length, integer(1)) < 2))
      stop("Each define name must contain a ':' separator")
    return(defines)
  } else {
    return(character())
  }
}

#' @importFrom magrittr %>%
#' @export
magrittr::`%>%`

is_url <- function(path){
  grepl("^https?://", path)
}

is_svg <- function(path){
  # svg files ending in "</svg>" with or without whitespace following it
  grepl("<\\/svg>\\s?$", path)
}

replace_url <- function(path){
  if(is_svg(path))
    return(path)
  if(is_url(path)){
    pattern <- "\\[[-,0-9]+\\]$"
    suffix <- regmatches(path, regexpr(pattern, path))
    path <- sub(pattern, "", path)
    paste0(download_url(path), suffix)
  } else if(grepl("^[^/\\]+:($|[^/\\])", path)) {
    # demo images e.g. "logo:" or "wizard:" or "cr2:myfile.cr2"
    return(path)
  } else {
    normalizePath(path, mustWork = FALSE)
  }
}

# Uses file extension from HTTP content-type if available to help IM guess type.
download_url <- function(url){
  tmp <- tempfile(fileext = sub("\\?.*", "", basename(url)))
  if(requireNamespace('curl', quietly = TRUE)){
    h <- curl::new_handle()
    curl::curl_download(url, tmp, handle = h)
    headers <- curl::parse_headers_list(curl::handle_data(h)$headers)
    ctype <- headers[['content-type']]
    matches <- match(ctype, mimetypes$type)
    if(length(matches) && !is.na(matches) && !grepl("(text|octet)", ctype)){
      extension <- sub("*.", ".", mimetypes$pattern[matches[1]], fixed = TRUE)
      outfile <- tempfile(fileext = extension)
      file.rename(tmp, outfile)
      return(outfile)
    }
  } else {
    utils::download.file(url, tmp, quiet = TRUE) #Fallback for webR
  }
  return(tmp)
}

assert_image <- function(image){
  if(!inherits(image, "magick-image"))
    stop("The 'image' argument is not a magick image object.", call. = FALSE)
  if(magick_image_dead(image))
    stop("Image pointer is dead. You cannot save or cache image objects between R sessions.", call. = FALSE)
}

df_to_tibble <- function(df){
  stopifnot(inherits(df, 'data.frame'))
  class(df) <- c("tbl_df", "tbl", "data.frame")
  df
}

is_windows <- function(){
  identical(.Platform$OS.type, 'windows')
}

#' @importFrom magrittr %>%
#' @export
magrittr::`%>%`

read_path <- function(path){
  image <- if(is.character(path)){
    if(is_url(path)){
      read_url(path)
    } else {
      read_file(path)
    }
  } else if(is.raw(path)){
    path
  } else {
    stop("Parameter 'image' must be an image object, file path or raw vector with image data")
  }
  return(image)
}

is_url <- function(path){
  grepl("^https?://", path)
}

is_svg <- function(path){
  # svg files ending in "</svg>" with or without whitespace following it
  grepl("<\\/svg>\\s?$", path)
}

replace_url <- function(path){
  if(is_svg(path))
    return(path)
  if(is_url(path)){
    pattern <- "\\[[-,0-9]+\\]$"
    suffix <- regmatches(path, regexpr(pattern, path))
    path <- sub(pattern, "", path)
    paste0(download_url(path), suffix)
  } else if(grepl("^[^/\\]+:($|[^/\\])", path)) {
    # demo images e.g. "logo:" or "wizard:" or "cr2:myfile.cr2"
    return(path)
  } else {
    normalizePath(path, mustWork = FALSE)
  }
}

download_url <- function(url){
  req <- curl::curl_fetch_memory(url)
  if(req$status_code >= 400)
    stop(sprintf("Failed to download %s (HTTP %d)", url, req$status))
  headers <- curl::parse_headers_list(req$headers)
  ctype <- headers[['content-type']]
  matches <- match(ctype, mimetypes$type)
  extension <- if(length(matches) && !is.na(matches) && !grepl("(text|octet)", ctype)){
    sub("*.", ".", mimetypes$pattern[matches[1]], fixed = TRUE)
  } else {
    sub("\\?.*", "", basename(url))
  }
  filename <- tempfile(fileext = extension)
  writeBin(req$content, filename)
  return(filename)
}

read_url <- function(path){
  req <- curl::curl_fetch_memory(path)
  if(req$status_code >= 400)
    stop(sprintf("Failed to download %s (HTTP %d)", path, req$status))
  return(req$content)
}

read_file <- function(path){
  readBin(normalizePath(path, mustWork = TRUE), raw(), file.info(path)$size)
}

assert_image <- function(image){
  if(!inherits(image, "magick-image"))
    stop("The 'image' argument is not a magick image object.", call. = FALSE)
  if(magick_image_dead(image))
    stop("Image pointer is dead. You cannot save or cache image objects between R sessions.", call. = FALSE)
}

df_to_tibble <- function(df){
  stopifnot(inherits(df, 'data.frame'))
  class(df) <- c("tbl_df", "tbl", "data.frame")
  df
}

is_windows <- function(){
  identical(.Platform$OS.type, 'windows')
}

library(xml2)
doc <- read_xml('https://raw.githubusercontent.com/ImageMagick/ImageMagick/master/www/source/mime.xml')
nodes <- xml_find_all(xml_children(doc), "//*[@pattern]")
mimetypes <- data.frame(
  type = xml_attr(nodes, "type"),
  description = xml_attr(nodes, "description"),
  pattern = xml_attr(nodes, "pattern"),
  stringsAsFactors = FALSE
)
save(mimetypes, file = "R/sysdata.rda")

#' Image Color
#'
#' Functions to adjust contrast, brightness, colors of the image. Details below.
#'
#' For details see [Magick++ STL](https://www.imagemagick.org/Magick++/STL.html)
#' documentation. Short descriptions:
#'
#'  - [image_modulate] adjusts brightness, saturation and hue of image relative to current.
#'  - [image_quantize] reduces number of unique colors in the image.
#'  - [image_ordered_dither] reduces number of unique colors using a dithering threshold map.
#'  - [image_map] replaces colors of image with the closest color from a reference image.
#'  - [image_channel] extracts a single channel from an image and returns as grayscale.
#'  - [image_transparent] sets pixels approximately matching given color to transparent.
#'  - [image_background] sets background color. When image is flattened, transparent pixels get background color.
#'  - [image_colorize] overlays a solid color frame using specified opacity.
#'  - [image_contrast] enhances intensity differences in image
#'  - [image_normalize] increases contrast by normalizing the pixel values to span the full range of colors
#'  - [image_enhance] tries to minimize noise
#'  - [image_equalize] equalizes using histogram equalization
#'  - [image_median] replaces each pixel with the median color in a circular neighborhood
#'
#' Note that
#' colors are also determined by image properties
#' [imagetype](https://www.imagemagick.org/Magick++/Enumerations.html#ImageType) and
#' [colorspace](https://www.imagemagick.org/Magick++/Enumerations.html#ColorspaceType)
#' which can be modified via [image_convert()].
#'
#' @export
#' @name color
#' @rdname color
#' @inheritParams painting
#' @family image
#' @param brightness modulation of brightness as percentage of the current value (100 for no change)
#' @param saturation modulation of saturation as percentage of the current value (100 for no change)
#' @param hue modulation of hue is an absolute rotation of -180 degrees to +180 degrees from the
#' current position corresponding to an argument range of 0 to 200 (100 for no change)
#' @examples
#' # manually adjust colors
#' logo <- image_read("logo:")
#' image_modulate(logo, brightness = 200)
#' image_modulate(logo, saturation = 150)
#' image_modulate(logo, hue = 200)
#'
image_modulate <- function(image, brightness = 100, saturation = 100, hue = 100){
  assert_image(image)
  brightness <- as.numeric(brightness)
  saturation <- as.numeric(saturation)
  hue <- as.numeric(hue)
  magick_image_modulate(image, brightness, saturation, hue)
}

#' @export
#' @rdname color
#' @param max preferred number of colors in the image. The actual number of colors in the image may
#' be less than your request, but never more.
#' @param dither a boolean (defaults to `TRUE`) specifying whether to apply Floyd/Steinberg error
#' diffusion to the image: averages intensities of several neighboring pixels
#' @param treedepth depth of the quantization color classification tree. Values of 0 or 1 allow
#' selection of the optimal tree depth for the color reduction algorithm. Values between 2 and 8
#' may be used to manually adjust the tree depth.
#' @examples
#' # Reduce image to 10 different colors using various spaces
#' image_quantize(logo, max = 10, colorspace = 'gray')
#' image_quantize(logo, max = 10, colorspace = 'rgb')
#' image_quantize(logo, max = 10, colorspace = 'cmyk')
#'
image_quantize <- function(image, max = 256, colorspace = 'rgb', dither = TRUE, treedepth = NULL){
  assert_image(image)
  max <- as.integer(max)
  colorspace <- as.character(colorspace)
  dither <- as.logical(dither)
  treedepth <- as.integer(treedepth)
  magick_image_quantize(image, max, colorspace, dither, treedepth)
}

#' @export
#' @rdname color
#' @param map reference image to map colors from
image_map <- function(image, map, dither = FALSE){
  assert_image(image)
  assert_image(map)
  dither <- as.logical(dither)
  magick_image_map(image, map, dither)
}

#' @export
#' @rdname color
#' @param threshold_map A string giving the dithering pattern to use. See
#' [the ImageMagick documentation](https://legacy.imagemagick.org/Usage/option_link.cgi?ordered-dither)
#' for possible values
#' @examples image_ordered_dither(logo, 'o8x8')
image_ordered_dither <- function(image, threshold_map){
  assert_image(image)
  threshold_map <- as.character(threshold_map)
  magick_image_ordered_dither(image, threshold_map)
}

#' @export
#' @rdname color
#' @param channel a string with a
#' [channel](https://www.imagemagick.org/Magick++/Enumerations.html#ChannelType) from
#' [channel_types][channel_types] for example `"alpha"` or `"hue"` or `"cyan"`
image_channel <- function(image, channel = 'lightness'){
  magick_image_channel(image, channel)
}

#' @export
#' @rdname color
image_separate <- function(image, channel = 'default'){
  magick_image_separate(image, channel)
}

#' @export
#' @rdname color
image_combine <- function(image, colorspace = 'sRGB', channel = 'default'){
  channel <- as.character(channel)
  colorspace <- as.character(colorspace)
  magick_image_combine(image, colorspace, channel)
}

#' @export
#' @rdname color
image_transparent <- function(image, color, fuzz = 0){
  assert_image(image)
  if(fuzz > 100)
    stop("Parameter 'fuzz' must be percentage value (0-100)")
  magick_image_transparent(image, color, fuzz)
}

#' @export
#' @rdname color
#' @inheritParams editing
#' @examples
#' # Change background color
#' translogo <- image_transparent(logo, 'white')
#' image_background(translogo, "pink", flatten = TRUE)
#'
#' # Compare to flood-fill method:
#' image_fill(logo, "pink", fuzz = 20)
#'
image_background <- function(image, color, flatten = TRUE){
  assert_image(image)
  out <- magick_image_background(image, color)
  if(isTRUE(flatten)){
    image_apply(out, image_flatten)
  } else {
    return(out)
  }
}

#' @export
#' @rdname color
#' @param opacity percentage of opacity used for coloring
#' @examples
#' # Other color tweaks
#' image_colorize(logo, 50, "red")
image_colorize <- function(image, opacity, color){
  assert_image(image)
  magick_image_colorize(image, opacity, color)
}

#' @export
#' @rdname color
#' @param sharpen enhance intensity differences in image
#' @examples
#' image_contrast(logo)
image_contrast <- function(image, sharpen = 1){
  assert_image(image)
  magick_image_contrast(image, sharpen)
}

#' @export
#' @rdname color
#' @examples
#' image_normalize(logo)
image_normalize <- function(image){
  assert_image(image)
  magick_image_normalize(image)
}

#' @export
#' @rdname color
#' @examples
#' image_enhance(logo)
image_enhance <- function(image){
  assert_image(image)
  magick_image_enhance(image)
}

#' @export
#' @rdname color
#' @examples
#' image_equalize(logo)
image_equalize <- function(image){
  assert_image(image)
  magick_image_equalize(image)
}

#' @export
#' @rdname color
#' @param radius replace each pixel with the median color in a circular neighborhood
#' @examples image_median(logo)
#'
#' # Alternate way to convert into black-white
#' image_convert(logo, type = 'grayscale')
image_median <- function(image, radius = 1.0){
  assert_image(image)
  magick_image_median(image, radius)
}

#' Image Frames and Animation
#'
#' Operations to manipulate or combine multiple frames of an image. Details below.
#'
#' For details see [Magick++ STL](https://www.imagemagick.org/Magick++/STL.html)
#' documentation. Short descriptions:
#'
#'  - [image_animate] coalesces frames by playing the sequence and converting to `gif` format.
#'  - [image_morph] expands number of frames by interpolating intermediate frames to blend
#'  into each other when played as an animation.
#'  - [image_mosaic] inlays images to form a single coherent picture.
#'  - [image_montage] creates a composite image by combining frames.
#'  - [image_flatten] merges frames as layers into a single frame using a given operator.
#'  - [image_average] averages frames into single frame.
#'  - [image_append] stack images left-to-right (default) or top-to-bottom.
#'  - [image_apply] applies a function to each frame
#'
#' The [image_apply] function calls an image function to each frame and joins
#' results back into a single image. Because most operations are already vectorized
#' this is often not needed. Note that `FUN()` should return an image. To apply other
#' kinds of functions to image frames simply use [lapply], [vapply], etc.
#'
#' @export
#' @inheritParams editing
#' @rdname animation
#' @name animation
#' @family image
#' @aliases image_coalesce
#' @param dispose a frame [disposal method](https://legacy.imagemagick.org/Usage/anim_basics/#dispose)
#' from [dispose_types()][dispose_types]
#' @param fps frames per second. Ignored if `delay` is not `NULL`.
#' @param delay delay after each frame, in 1/100 seconds.
#' Must be length 1, or number of frames. If specified, then `fps` is ignored.
#' @param loop how many times to repeat the animation. Default is infinite.
#' @param optimize optimize the `gif` animation by storing only the differences
#' between frames. Input images must be exactly the same size.
image_animate <- function(image, fps = 10, delay = NULL, loop = 0,
                          dispose = c("background", "previous", "none"),
                          optimize = FALSE){
  assert_image(image)
  stopifnot(is.numeric(fps))
  stopifnot(is.null(delay) || (
    is.numeric(delay) && length(delay) %in% c(1, length(image))))
  stopifnot(is.numeric(loop))
  stopifnot(is.logical(optimize))
  if(is.null(delay)) {
    if(100 %% fps)
      stop("argument 'fps' must be a factor of 100")
    delay <- 100/fps
  }
  dispose <- match.arg(dispose)
  magick_image_animate(image, as.integer(delay), as.integer(loop), dispose,
                       optimize)
}

#' @export
#' @rdname animation
image_coalesce <- function(image){
  magick_image_coalesce(image)
}

#' @export
#' @rdname animation
#' @param frames number of frames to use in output animation
#' @examples
#' # Combine images
#' logo <- image_read("https://jeroen.github.io/images/Rlogo.png")
#' oldlogo <- image_read("https://jeroen.github.io/images/Rlogo-old.png")
#'
#' # Create morphing animation
#' both <- image_scale(c(oldlogo, logo), "400")
#' image_average(image_crop(both))
#' image_animate(image_morph(both, 10))
#'
image_morph <- function(image, frames = 8){
  assert_image(image)
  stopifnot(is.numeric(frames))
  magick_image_morph(image, frames)
}

#' @export
#' @rdname animation
#' @inheritParams composite
image_mosaic <- function(image, operator = NULL){
  assert_image(image)
  operator <- as.character(operator)
  magick_image_mosaic(image, operator)
}

#' @export
#' @rdname animation
image_flatten <- function(image, operator = NULL){
  assert_image(image)
  operator <- as.character(operator)
  magick_image_flatten(image, operator)
}

#' @export
#' @rdname animation
image_average <- function(image){
  assert_image(image)
  magick_image_average(image)
}

#' @export
#' @rdname animation
#' @param stack place images top-to-bottom (TRUE) or left-to-right (FALSE)
#' @examples
#' # Create thumbnails from GIF
#' banana <- image_read("https://jeroen.github.io/images/banana.gif")
#' length(banana)
#' image_average(banana)
#' image_flatten(banana)
#' image_append(banana)
#' image_append(banana, stack = TRUE)
#'
#' # Append images together
#' wizard <- image_read("wizard:")
#' image_append(image_scale(c(image_append(banana[c(1,3)], stack = TRUE), wizard)))
#'
#' image_composite(banana, image_scale(logo, "300"))
#'
#' # Break down and combine frames
#' front <- image_scale(banana, "300")
#' background <- image_background(image_scale(logo, "400"), 'white')
#' frames <- image_apply(front, function(x){image_composite(background, x, offset = "+70+30")})
#' image_animate(frames, fps = 10)
image_append <- function(image, stack = FALSE){
  assert_image(image)
  magick_image_append(image, stack)
}

#' @rdname animation
#' @export
#' @param FUN a function to be called on each frame in the image
#' @param ... additional parameters for \code{FUN}
image_apply <- function(image, FUN, ...){
  assert_image(image)
  out <- lapply(image, FUN, ...)
  if(!all(vapply(out, inherits, logical(1), what = "magick-image")))
    stop(sprintf("Function %s did not return a valid image.", deparse(substitute(FUN))))
  image_join(out)
}

#' @export
#' @rdname animation
#' @param geometry a [geometry][geometry] string that defines the size the individual
#' thumbnail images, and the spacing between them.
#' @param tile a [geometry][geometry] string for example "4x5 with limits on how the
#' tiled images are to be laid out on the final result.
#' @param gravity a gravity direction, if the image is smaller than the frame, where
#'in the frame is the image to be placed.
#' @param bg a background color string
#' @param shadow enable shadows between images
#' @examples # Simple 4x3 montage
#' input <- rep(logo, 12)
#' image_montage(input, geometry = 'x100+10+10', tile = '4x3', bg = 'pink', shadow = TRUE)
#'
#' # With varying frame size
#' input <- c(wizard, wizard, logo, logo)
#' image_montage(input, tile = '2x2', bg = 'pink', gravity = 'southwest')
image_montage <- function(image, geometry = NULL, tile = NULL, gravity = 'Center',
                          bg = 'white', shadow = FALSE){
  assert_image(image)
  geometry <- as.character(geometry)
  tile <- as.character(tile)
  gravity <- as.character(gravity)
  bg <- as.character(bg)
  shadow <- as.logical(shadow)
  magick_image_montage(image, geometry, tile, gravity, bg, shadow)
}

#' Image Analysis
#'
#' Functions for image calculations and analysis. This part of the package needs more work.
#'
#' For details see [Image++](https://www.imagemagick.org/Magick++/Image++.html)
#' documentation. Short descriptions:
#'
#'  - [image_compare] calculates a metric by comparing image with a reference image.
#'  - [image_fft] returns Discrete Fourier Transform (DFT) of the image as a
#'  magnitude / phase image pair. I wish I knew what this means.
#'
#' Here `image_compare()` is vectorized over the first argument and returns the diff image
#' with the calculated distortion value as an attribute.
#'
#' @export
#' @family image
#' @name analysis
#' @rdname analysis
#' @inheritParams editing
#' @inheritParams painting
#' @param reference_image another image to compare to
#' @param metric string with a [metric](http://www.imagemagick.org/script/command-line-options.php#metric)
#' from [metric_types()][metric_types] such as `"AE"` or `"phash"`
#' @examples
#' out1 <- image_blur(logo, 3)
#' out2 <- image_oilpaint(logo, 3)
#' input <- c(logo, out1, out2, logo)
#' if(magick_config()$version >= "6.8.7"){
#'   diff_img <- image_compare(input, logo, metric = "AE")
#'   attributes(diff_img)
#' }
image_compare <- function(image, reference_image, metric = "", fuzz = 0){
  metric <- as.character(metric)
  magick_image_compare(image, reference_image, metric, fuzz)
}

#' @rdname analysis
#' @export
image_compare_dist <- function(image, reference_image, metric = "", fuzz = 0){
  out <- attributes(image_compare(image, reference_image, metric, fuzz))
  out$class = NULL
  out
}

#' @export
#' @rdname analysis
image_fft <- function(image){
  if(!isTRUE(magick_config()$fftw))
    stop("ImageMagick was configured without FFTW support.")
  assert_image(image)
  magick_image_fft(image)
}

#' Image Effects
#'
#' High level effects applied to an entire image.
#' These are mostly just for fun.
#'
#' @name effects
#' @inheritParams editing
#' @family image
#' @rdname effects
#' @export
#' @param times number of times to repeat the despeckle operation
#' @examples logo <- image_read("logo:")
#' image_despeckle(logo)
image_despeckle <- function(image, times = 1L){
  assert_image(image)
  magick_image_despeckle(image, times)
}

#' @export
#' @rdname effects
#' @examples image_reducenoise(logo)
image_reducenoise <- function(image, radius = 1L){
  assert_image(image)
  magick_image_reducenoise(image, radius)
}

#' @export
#' @rdname effects
#' @param noisetype string with a
#' [noisetype](https://www.imagemagick.org/Magick++/Enumerations.html#NoiseType) value
#' from [noise_types][noise_types].
#' @examples
#' image_noise(logo)
image_noise <- function(image, noisetype = "gaussian"){
  assert_image(image)
  magick_image_noise(image, noisetype)
}

#' @export
#' @rdname effects
#' @param radius radius, in pixels, for various transformations
#' @param sigma the standard deviation of the Laplacian, in pixels.
#' @examples
#' image_blur(logo, 10, 10)
image_blur <- function(image, radius = 1, sigma = 0.5){
  assert_image(image)
  magick_image_blur(image, radius, sigma)
}

#' @export
#' @rdname effects
#' @param angle angle, in degrees, for various transformations
#' @examples
#' image_motion_blur(logo, 10, 10, 45)
image_motion_blur <- function(image, radius = 1, sigma = 0.5, angle = 0){
  assert_image(image)
  magick_image_motion_blur(image, radius, sigma, angle)
}

#' @export
#' @rdname effects
#' @examples
#' image_charcoal(logo)
image_charcoal <- function(image, radius = 1, sigma = 0.5){
  assert_image(image)
  magick_image_charcoal(image, radius, sigma)
}

#' @export
#' @rdname effects
#' @examples
#' image_oilpaint(logo, radius = 3)
image_oilpaint <- function(image, radius = 1){
  assert_image(image)
  magick_image_oilpaint(image, radius)
}

#' @export
#' @rdname effects
#' @examples
#' image_emboss(logo)
image_emboss <- function(image, radius = 1, sigma = 0.5){
  assert_image(image)
  magick_image_emboss(image, radius, sigma)
}

#' @export
#' @rdname effects
#' @param factor image implode factor (special effect)
#' @examples
#' image_implode(logo)
image_implode <- function(image, factor = 0.5){
  assert_image(image)
  magick_image_implode(image, factor)
}

#' @export
#' @rdname effects
#' @examples
#' image_negate(logo)
image_negate <- function(image){
  assert_image(image)
  magick_image_negate(image)
}

#' Image Transform
#'
#' Basic transformations like rotate, resize, crop and flip. The [geometry] syntax
#' is used to specify sizes and areas.
#'
#' For details see [Magick++ STL](https://www.imagemagick.org/Magick++/STL.html)
#' documentation. Short descriptions:
#'
#' - [image_trim] removes edges that are the background color from the image.
#' - [image_chop] removes vertical or horizontal subregion of image.
#' - [image_crop] cuts out a subregion of original image
#' - [image_rotate] rotates and increases size of canvas to fit rotated image.
#' - [image_deskew] auto rotate to correct skewed images
#' - [image_resize] resizes using custom [filterType](https://www.imagemagick.org/Magick++/Enumerations.html#FilterTypes)
#' - [image_scale] and [image_sample] resize using simple ratio and pixel sampling algorithm.
#' - [image_flip] and [image_flop] invert image vertically and horizontally
#'
#' The most powerful resize function is [image_resize] which allows for setting
#' a custom resize filter. Output of [image_scale] is similar to `image_resize(img, filter = "point")`.
#'
#' For resize operations it holds that if no `geometry` is specified, all frames
#' are rescaled to match the top frame.
#' @name transform
#' @rdname transform
#' @inheritParams effects
#' @inheritParams painting
#' @family image
#' @export
#' @examples
#' logo <- image_read("logo:")
#' logo <- image_scale(logo, "400")
#' image_trim(logo)
image_trim <- function(image, fuzz = 0){
  assert_image(image)
  fuzz <- as.numeric(fuzz)
  magick_image_trim(image, fuzz)
}

#' @export
#' @rdname transform
#' @param geometry a [geometry][geometry] string specifying area (for cropping) or size (for resizing).
#' @examples
#' image_chop(logo, "100x20")
image_chop <- function(image, geometry){
  assert_image(image)
  stopifnot(is.character(geometry))
  magick_image_chop(image, geometry)
}

#' @export
#' @rdname transform
#' @param degrees value between 0 and 360 for how many degrees to rotate
#' @examples
#' image_rotate(logo, 45)
image_rotate <- function(image, degrees){
  assert_image(image)
  magick_image_rotate(image, degrees)
}

#' @export
#' @rdname transform
#' @param filter string with [filter](https://www.imagemagick.org/Magick++/Enumerations.html#FilterTypes)
#' type from: [filter_types][filter_types]
#' @examples # Small image
#' rose <- image_convert(image_read("rose:"), "png")
#'
#' # Resize to 400 width or height:
#' image_resize(rose, "400x")
#' image_resize(rose, "x400")
#'
#' # Resize keeping ratio
#' image_resize(rose, "400x400")
#'
#' # Resize, force size losing ratio
#' image_resize(rose, "400x400!")
#'
#' # Different filters
#' image_resize(rose, "400x", filter = "Triangle")
#' image_resize(rose, "400x", filter = "Point")
image_resize <- function(image, geometry = NULL, filter = NULL){
  assert_image(image)
  geometry <- as.character(geometry)
  filter <- as.character(filter)
  magick_image_resize(image, geometry, filter)
}

#' @export
#' @rdname transform
#' @examples # simple pixel resize
#' image_scale(rose, "400x")
image_scale <- function(image, geometry = NULL){
  assert_image(image)
  geometry <- as.character(geometry)
  magick_image_scale(image, geometry)
}

#' @export
#' @rdname transform
#' @examples image_sample(rose, "400x")
image_sample <- function(image, geometry = NULL){
  assert_image(image)
  magick_image_sample(image, geometry)
}

#' @export
#' @rdname transform
#' @inheritParams painting
#' @param repage resize the canvas to the cropped area
#' @examples image_crop(logo, "400x400+200+200")
image_crop <- function(image, geometry = NULL, gravity = NULL, repage = TRUE){
  assert_image(image)
  geometry <- as.character(geometry)
  gravity <- as.character(gravity)
  magick_image_crop(image, geometry, gravity, repage)
}

#' @export
#' @rdname transform
#' @examples image_extent(rose, '200x200', color = 'pink')
image_extent <- function(image, geometry, gravity = "center", color = "none"){
  assert_image(image)
  geometry <- as.character(geometry)
  gravity <- as.character(gravity)
  magick_image_extent(image, geometry, gravity, color)
}

#' @export
#' @rdname transform
#' @examples
#' image_flip(logo)
image_flip <- function(image){
  assert_image(image)
  magick_image_flip(image)
}

#' @export
#' @rdname transform
#' @examples
#' image_flop(logo)
image_flop <- function(image){
  assert_image(image)
  magick_image_flop(image)
}

#' @export
#' @rdname transform
#' @param threshold straightens an image. A threshold of 40 works for most images.
#' @examples skewed <- image_rotate(logo, 5)
#' deskewed <- image_deskew(skewed)
#' attr(deskewed, 'angle')
image_deskew <- function(image, threshold = 40){
  assert_image(image)
  out <- magick_image_deskew(image, threshold)
  attr(out, 'angle') <- as.numeric(image_get_artifact(out, 'deskew:angle'))
  out
}

#' @export
#' @rdname transform
image_deskew_angle <- function(image, threshold = 40){
  attr(image_deskew(image), 'angle')
}

#' @export
#' @rdname transform
#' @param pagesize geometry string with preferred size and location of an image canvas
#' @param density geometry string with vertical and horizontal resolution in pixels of
#' the image. Specifies an image density when decoding a Postscript or PDF.
image_page <- function(image, pagesize = NULL, density = NULL){
  assert_image(image)
  pagesize <- as.character(pagesize)
  density <- as.character(density)
  magick_image_page(image, pagesize, density)
}

#' @export
#' @rdname transform
image_repage <- function(image){
  magick_image_repage(image)
}

#' @export
#' @rdname transform
#' @param orientation string to set image orientation one of the [orientation_types].
#' If `NULL` it applies auto-orientation which tries to infer the correct orientation
#' from the Exif data.
#' @examples
#' if(magick_config()$version > "6.8.6")
#'   image_orient(logo)
image_orient <- function(image, orientation = NULL){
  assert_image(image)
  orientation <- as.character(orientation)
  magick_image_orient(image, orientation)
}

#' @export
#' @rdname transform
#' @examples
#' image_shear(logo, "10x10")
image_shear <- function(image, geometry = "10x10", color = "none"){
  assert_image(image)
  stopifnot(is.character(geometry))
  stopifnot(is.character(color))
  magick_image_shear(image, geometry, color)
}

#' @export
#' @rdname transform
#' @param distortion string to set image orientation one of the [distort_types].
#' @param coordinates numeric vector (typically of length 12) with distortion coordinates
#' @param bestfit if set to `TRUE` the size of the output image can be different from input
#' @examples building <- demo_image('building.jpg')
#' image_distort(building, 'perspective', c(7,40,4,30,4,124,4,123,85,122,100,123,85,2,100,30))
image_distort <- function(image, distortion = 'perspective', coordinates, bestfit = FALSE){
  assert_image(image)
  stopifnot(is.character(distortion))
  stopifnot(is.numeric(coordinates))
  stopifnot(is.logical(bestfit))
  magick_image_distort(image, distortion, coordinates, bestfit)
}

#' RStudio Graphics AutoViewer
#'
#' This enables a \link{addTaskCallback} that automatically updates the viewer after
#' the state of a magick graphics device has changed. This is enabled by default in
#' RStudio.
#'
#' @export
#' @rdname autoviewer
#' @name autoviewer
#' @examples
#' # Only has effect in RStudio (or other GUI with a viewer):
#' autoviewer_enable()
#'
#' img <- magick::image_graph()
#' plot(1)
#' abline(0, 1, col = "blue", lwd = 2, lty = "solid")
#' abline(0.1, 1, col = "red", lwd = 3, lty = "dotted")
#'
#' autoviewer_disable()
#' abline(0.2, 1, col = "green", lwd = 4, lty = "twodash")
#' abline(0.3, 1, col = "black", lwd = 5, lty = "dotdash")
#'
#' autoviewer_enable()
#' abline(0.4, 1, col = "purple", lwd = 6, lty = "dashed")
#' abline(0.5, 1, col = "yellow", lwd = 7, lty = "longdash")
autoviewer_enable <- function(){
  autoviewer$enable()
}

#' @export
#' @rdname autoviewer
autoviewer_disable <- function(){
  autoviewer$disable()
}

# Automatically update viewer after graphics events have finished
autoviewer <- local({
  id <- NULL
  this <- environment()
  enable <- function(){
    this$disable()
    magick_device_pop() # clear existing plots
    if(interactive() && is.function(getOption("viewer"))){
      id <<- addTaskCallback(function(...){
        tryCatch({
          dirty <- magick_device_pop()
          if(length(dirty))
            print(dirty, info = FALSE)
        }, error = function(...){});
        return(TRUE)
      })
    }
  }
  disable <- function(...){
    if(length(id))
      removeTaskCallback(id)
  }
  reg.finalizer(this, this$disable, onexit = TRUE)
  this
})

#' Convert to EBImage
#'
#' Convert a Magick image to [EBImage](https://bioconductor.org/packages/release/bioc/html/EBImage.html)
#' class. Note that EBImage only supports multi-frame images in greyscale.
#'
#' @export
#' @inheritParams editing
as_EBImage <- function(image){
  Grayscale <- utils::getFromNamespace('Grayscale', 'EBImage')
  Color <- utils::getFromNamespace('Color', 'EBImage')
  Image <- utils::getFromNamespace('Image', 'EBImage')
  assert_image(image)
  info <- image_info(image)
  if(length(image) > 1){
    if(!all(tolower(info$colorspace) == 'gray'))
      warning('EBImage does not supported colored multi-frame images. Converting to grayscale.')
    bitmap <- vapply(image, function(x){
      image_data(x, channels = 'gray')[1,,]
    }, matrix(raw(1), info$width, info$height))
    Image(as.double(bitmap) / 255, dim(bitmap), Grayscale)
  } else {
    bitmap <- aperm(image_data(image), c(2,3,1))
    if(tolower(info$colorspace) == 'gray') {
      Image(as.double(bitmap) / 255, dim(bitmap)[1:2], Grayscale)
    } else {
      Image(as.double(bitmap) / 255, dim(bitmap), Color)
    }
  }
}

#' Image Painting
#'
#' The [image_fill()] function performs flood-fill by painting starting point and all
#' neighboring pixels of approximately the same color. Annotate prints some text on
#' the image.
#'
#' Note that more sophisticated drawing mechanisms are available via the graphics
#' device using [image_draw].
#'
#' @export
#' @name painting
#' @rdname painting
#' @family image
#' @inheritParams editing
#' @param color a valid [color string](https://www.imagemagick.org/Magick++/Color.html) such as
#' `"navyblue"` or `"#000080"`. Use `"none"` for transparency.
#' @param point a geometry_point string indicating the starting point of the flood-fill
#' @param fuzz relative color distance (value between 0 and 100) to be considered similar
#' in the filling algorithm
#' @param refcolor if set, `fuzz` color distance will be measured against this color,
#' not the color of the starting `point`. Any color (within `fuzz` color distance of
#' the given `refcolor`), connected to starting point will be replaced with the `color`.
#' If the pixel at the starting point does not itself match the given `refcolor`
#' (according to `fuzz`) then no action will be taken.
#' @examples
#' logo <- image_read("logo:")
#' logo <- image_background(logo, 'white')
#' image_fill(logo, "pink", point = "+450+400")
#' image_fill(logo, "pink", point = "+450+400", fuzz = 25)
image_fill <- function(image, color, point = "+1+1", fuzz = 0, refcolor = NULL){
  assert_image(image)
  color <- as.character(color)
  point <- as.character(point)
  refcolor <- as.character(refcolor)
  if(fuzz > 100)
    stop("Parameter 'fuzz' must be percentage value (0-100)")
  magick_image_fill(image, color, point, fuzz, refcolor)
}

#' @export
#' @details Setting a font, weight, style only works if your imagemagick is compiled
#' with fontconfig support.
#' @rdname painting
#' @param text character vector of length equal to 'image' or length 1
#' @param degrees rotates text around center point
#' @param gravity string with
#' [gravity](https://www.imagemagick.org/Magick++/Enumerations.html#GravityType)
#' value from [gravity_types].
#' @param location geometry string with location relative to `gravity`
#' @param size font-size in pixels
#' @param strokecolor a [color string](https://www.imagemagick.org/Magick++/Color.html)
#' adds a stroke (border around the text)
#' @param boxcolor a [color string](https://www.imagemagick.org/Magick++/Color.html)
#' for background color that annotation text is rendered on.
#' @param font string with font family such as `"sans"`, `"mono"`, `"serif"`,
#' `"Times"`, `"Helvetica"`, `"Trebuchet"`, `"Georgia"`, `"Palatino"` or `"Comic Sans"`.
#' @param style value of [style_types][style_types] for example `"italic"`
#' @param weight thickness of the font, 400 is normal and 700 is bold.
#' @param kerning increases or decreases whitespace between letters
#' @param decoration value of [decoration_types][decoration_types] for example `"underline"`
#' @examples # Add some text to an image
#' image_annotate(logo, "This is a test")
#' image_annotate(logo, "CONFIDENTIAL", size = 50, color = "red", boxcolor = "pink",
#'  degrees = 30, location = "+100+100")
#'
#' # Setting fonts requires fontconfig support (and that you have the font)
#' image_annotate(logo, "The quick brown fox", font = "monospace", size = 50)
image_annotate <- function(image, text, gravity = "northwest", location = "+0+0", degrees = 0,
                           size = 10, font = "", style = "normal", weight = 400, kerning = 0,
                           decoration = NULL, color = NULL, strokecolor = NULL, boxcolor = NULL){
  assert_image(image)
  font <- as.character(font)
  size <- as.integer(size)
  weight <- as.numeric(weight)
  kerning <- as.numeric(kerning)
  color <- as.character(color)
  strokecolor <- as.character(strokecolor)
  boxcolor <- as.character(boxcolor)
  decoration <- as.character(decoration)
  magick_image_annotate(image, text, gravity, location, degrees, size,
                        font, style, weight, kerning, decoration,
                        color, strokecolor, boxcolor)
}

#' Image Painting
#'
#' The [image_fill()] function performs flood-fill by painting starting point and all
#' neighboring pixels of approximately the same color. Annotate prints some text on
#' the image.
#'
#' Note that more sophisticated drawing mechanisms are available via the graphics
#' device using [image_draw].
#'
#' @export
#' @name painting
#' @rdname painting
#' @family image
#' @inheritParams editing
#' @param color a valid [color string](https://www.imagemagick.org/Magick++/Color.html) such as
#' `"navyblue"` or `"#000080"`. Use `"none"` for transparency.
#' @param point a geometry_point string indicating the starting point of the flood-fill
#' @param fuzz relative color distance (value between 0 and 100) to be considered similar
#' in the filling algorithm
#' @param refcolor if set, `fuzz` color distance will be measured against this color,
#' not the color of the starting `point`. Any color (within `fuzz` color distance of
#' the given `refcolor`), connected to starting point will be replaced with the `color`.
#' If the pixel at the starting point does not itself match the given `refcolor`
#' (according to `fuzz`) then no action will be taken.
#' @examples
#' logo <- image_read("logo:")
#' logo <- image_background(logo, 'white')
#' image_fill(logo, "pink", point = "+450+400")
#' image_fill(logo, "pink", point = "+450+400", fuzz = 25)
image_fill <- function(image, color, point = "+1+1", fuzz = 0, refcolor = NULL){
  assert_image(image)
  color <- as.character(color)
  point <- as.character(point)
  refcolor <- as.character(refcolor)
  if(fuzz > 100)
    stop("Parameter 'fuzz' must be percentage value (0-100)")
  magick_image_fill(image, color, point, fuzz, refcolor)
}

#' @export
#' @details Setting a font, weight, style only works if your imagemagick is compiled
#' with fontconfig support.
#' @rdname painting
#' @param text character vector of length equal to 'image' or length 1
#' @param degrees rotates text around center point
#' @param gravity string with
#' [gravity](https://www.imagemagick.org/Magick++/Enumerations.html#GravityType)
#' value from [gravity_types].
#' @param location geometry string with location relative to `gravity`
#' @param size font-size in pixels
#' @param strokecolor a [color string](https://www.imagemagick.org/Magick++/Color.html)
#' adds a stroke (border around the text)
#' @param strokewidth set the strokewidth of the border around the text
#' @param boxcolor a [color string](https://www.imagemagick.org/Magick++/Color.html)
#' for background color that annotation text is rendered on.
#' @param font string with font family such as `"sans"`, `"mono"`, `"serif"`,
#' `"Times"`, `"Helvetica"`, `"Trebuchet"`, `"Georgia"`, `"Palatino"` or `"Comic Sans"`.
#' @param style value of [style_types][style_types] for example `"italic"`
#' @param weight thickness of the font, 400 is normal and 700 is bold.
#' @param kerning increases or decreases whitespace between letters
#' @param decoration value of [decoration_types][decoration_types] for example `"underline"`
#' @examples # Add some text to an image
#' image_annotate(logo, "This is a test")
#' image_annotate(logo, "CONFIDENTIAL", size = 50, color = "red", boxcolor = "pink",
#'  degrees = 30, location = "+100+100")
#'
#' # Setting fonts requires fontconfig support (and that you have the font)
#' image_annotate(logo, "The quick brown fox", font = "monospace", size = 50)
image_annotate <- function(image, text, gravity = "northwest", location = "+0+0", degrees = 0,
                           size = 10, font = "", style = "normal", weight = 400, kerning = 0,
                           decoration = NULL, color = NULL, strokecolor = NULL,
                           strokewidth = NULL, boxcolor = NULL){
  assert_image(image)
  font <- as.character(font)
  size <- as.integer(size)
  weight <- as.numeric(weight)
  kerning <- as.numeric(kerning)
  color <- as.character(color)
  strokecolor <- as.character(strokecolor)
  strokewidth <- as.integer(strokewidth)
  boxcolor <- as.character(boxcolor)
  decoration <- as.character(decoration)
  magick_image_annotate(image, text, gravity, location, degrees, size,
                        font, style, weight, kerning, decoration,
                        color, strokecolor, strokewidth, boxcolor)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

magick_image_animate <- function(input, delay, iter, method, optimize) {
    .Call('_magick_magick_image_animate', PACKAGE = 'magick', input, delay, iter, method, optimize)
}

magick_image_coalesce <- function(input) {
    .Call('_magick_magick_image_coalesce', PACKAGE = 'magick', input)
}

magick_image_morph <- function(image, frames) {
    .Call('_magick_magick_image_morph', PACKAGE = 'magick', image, frames)
}

magick_image_mosaic <- function(input, composite) {
    .Call('_magick_magick_image_mosaic', PACKAGE = 'magick', input, composite)
}

magick_image_flatten <- function(input, composite) {
    .Call('_magick_magick_image_flatten', PACKAGE = 'magick', input, composite)
}

magick_image_average <- function(image) {
    .Call('_magick_magick_image_average', PACKAGE = 'magick', image)
}

magick_image_append <- function(image, stack) {
    .Call('_magick_magick_image_append', PACKAGE = 'magick', image, stack)
}

magick_attr_comment <- function(input, set) {
    .Call('_magick_magick_attr_comment', PACKAGE = 'magick', input, set)
}

magick_attr_text_antialias <- function(input, set) {
    .Call('_magick_magick_attr_text_antialias', PACKAGE = 'magick', input, set)
}

magick_attr_stroke_antialias <- function(input, set) {
    .Call('_magick_magick_attr_stroke_antialias', PACKAGE = 'magick', input, set)
}

magick_attr_animationdelay <- function(input, delay) {
    .Call('_magick_magick_attr_animationdelay', PACKAGE = 'magick', input, delay)
}

magick_attr_backgroundcolor <- function(input, color) {
    .Call('_magick_magick_attr_backgroundcolor', PACKAGE = 'magick', input, color)
}

magick_attr_boxcolor <- function(input, color) {
    .Call('_magick_magick_attr_boxcolor', PACKAGE = 'magick', input, color)
}

magick_attr_fillcolor <- function(input, color) {
    .Call('_magick_magick_attr_fillcolor', PACKAGE = 'magick', input, color)
}

magick_attr_font <- function(input, font) {
    .Call('_magick_magick_attr_font', PACKAGE = 'magick', input, font)
}

magick_attr_fontsize <- function(input, pointsize) {
    .Call('_magick_magick_attr_fontsize', PACKAGE = 'magick', input, pointsize)
}

magick_attr_label <- function(input, label) {
    .Call('_magick_magick_attr_label', PACKAGE = 'magick', input, label)
}

magick_attr_format <- function(input, format) {
    .Call('_magick_magick_attr_format', PACKAGE = 'magick', input, format)
}

magick_attr_quality <- function(input, quality) {
    .Call('_magick_magick_attr_quality', PACKAGE = 'magick', input, quality)
}

magick_attr_quantize <- function(input, numcolors) {
    .Call('_magick_magick_attr_quantize', PACKAGE = 'magick', input, numcolors)
}

magick_attr_density <- function(input, density) {
    .Call('_magick_magick_attr_density', PACKAGE = 'magick', input, density)
}

magick_image_info <- function(input) {
    .Call('_magick_magick_image_info', PACKAGE = 'magick', input)
}

magick_image_as_raster <- function(data) {
    .Call('_magick_magick_image_as_raster', PACKAGE = 'magick', data)
}

magick_threads <- function(i = 0L) {
    .Call('_magick_magick_threads', PACKAGE = 'magick', i)
}

magick_image_dead <- function(image) {
    .Call('_magick_magick_image_dead', PACKAGE = 'magick', image)
}

magick_image_length <- function(image) {
    .Call('_magick_magick_image_length', PACKAGE = 'magick', image)
}

create <- function(len) {
    .Call('_magick_create', PACKAGE = 'magick', len)
}

magick_image_blank <- function(width, height, color, pseudo_image, options) {
    .Call('_magick_magick_image_blank', PACKAGE = 'magick', width, height, color, pseudo_image, options)
}

copy <- function(image) {
    .Call('_magick_copy', PACKAGE = 'magick', image)
}

magick_image_rev <- function(input) {
    .Call('_magick_magick_image_rev', PACKAGE = 'magick', input)
}

magick_image_join <- function(input) {
    .Call('_magick_magick_image_join', PACKAGE = 'magick', input)
}

magick_image_subset <- function(image, index) {
    .Call('_magick_magick_image_subset', PACKAGE = 'magick', image, index)
}

magick_image_replace <- function(image, index, value) {
    .Call('_magick_magick_image_replace', PACKAGE = 'magick', image, index, value)
}

magick_image_copy <- function(image, add) {
    .Call('_magick_magick_image_copy', PACKAGE = 'magick', image, add)
}

magick_image_destroy <- function(image) {
    invisible(.Call('_magick_magick_image_destroy', PACKAGE = 'magick', image))
}

autobrewed <- function() {
    .Call('_magick_autobrewed', PACKAGE = 'magick')
}

magick_resource_limits <- function() {
    .Call('_magick_magick_resource_limits', PACKAGE = 'magick')
}

magick_image_contrast <- function(input, sharpen) {
    .Call('_magick_magick_image_contrast', PACKAGE = 'magick', input, sharpen)
}

magick_image_normalize <- function(input) {
    .Call('_magick_magick_image_normalize', PACKAGE = 'magick', input)
}

magick_image_modulate <- function(input, brightness, saturation, hue) {
    .Call('_magick_magick_image_modulate', PACKAGE = 'magick', input, brightness, saturation, hue)
}

magick_image_map <- function(input, map_image, dither) {
    .Call('_magick_magick_image_map', PACKAGE = 'magick', input, map_image, dither)
}

magick_image_channel <- function(input, channel) {
    .Call('_magick_magick_image_channel', PACKAGE = 'magick', input, channel)
}

magick_image_colorize <- function(input, opacity, color) {
    .Call('_magick_magick_image_colorize', PACKAGE = 'magick', input, opacity, color)
}

magick_image_enhance <- function(input) {
    .Call('_magick_magick_image_enhance', PACKAGE = 'magick', input)
}

magick_image_equalize <- function(input) {
    .Call('_magick_magick_image_equalize', PACKAGE = 'magick', input)
}

magick_image_median <- function(input, radius) {
    .Call('_magick_magick_image_median', PACKAGE = 'magick', input, radius)
}

magick_image_quantize <- function(input, max, space, dither, depth) {
    .Call('_magick_magick_image_quantize', PACKAGE = 'magick', input, max, space, dither, depth)
}

magick_image_ordered_dither <- function(input, threshold_map) {
    .Call('_magick_magick_image_ordered_dither', PACKAGE = 'magick', input, threshold_map)
}

magick_image_transparent <- function(input, color, fuzz_percent) {
    .Call('_magick_magick_image_transparent', PACKAGE = 'magick', input, color, fuzz_percent)
}

magick_image_background <- function(input, color) {
    .Call('_magick_magick_image_background', PACKAGE = 'magick', input, color)
}

magick_image_lat <- function(input, geomstr) {
    .Call('_magick_magick_image_lat', PACKAGE = 'magick', input, geomstr)
}

magick_image_threshold_black <- function(input, threshold, channel) {
    .Call('_magick_magick_image_threshold_black', PACKAGE = 'magick', input, threshold, channel)
}

magick_image_threshold_white <- function(input, threshold, channel) {
    .Call('_magick_magick_image_threshold_white', PACKAGE = 'magick', input, threshold, channel)
}

magick_image_level <- function(input, black_pct, white_pct, mid_point, channel) {
    .Call('_magick_magick_image_level', PACKAGE = 'magick', input, black_pct, white_pct, mid_point, channel)
}

magick_image_composite <- function(input, composite_image, offset, gravity, composite, args) {
    .Call('_magick_magick_image_composite', PACKAGE = 'magick', input, composite_image, offset, gravity, composite, args)
}

magick_image_border <- function(input, color, geometry, composite) {
    .Call('_magick_magick_image_border', PACKAGE = 'magick', input, color, geometry, composite)
}

magick_image_frame <- function(input, color, geometry) {
    .Call('_magick_magick_image_frame', PACKAGE = 'magick', input, color, geometry)
}

magick_image_shadow_mask <- function(input, geomstr) {
    .Call('_magick_magick_image_shadow_mask', PACKAGE = 'magick', input, geomstr)
}

magick_image_shade <- function(input, azimuth = 30, elevation = 30, color = FALSE) {
    .Call('_magick_magick_image_shade', PACKAGE = 'magick', input, azimuth, elevation, color)
}

magick_image_crop <- function(input, geometry, gravity, repage) {
    .Call('_magick_magick_image_crop', PACKAGE = 'magick', input, geometry, gravity, repage)
}

magick_image_extent <- function(input, geometry, gravity, color) {
    .Call('_magick_magick_image_extent', PACKAGE = 'magick', input, geometry, gravity, color)
}

magick_image_artifact <- function(input, name) {
    .Call('_magick_magick_image_artifact', PACKAGE = 'magick', input, name)
}

magick_coder_info <- function(format) {
    .Call('_magick_magick_coder_info', PACKAGE = 'magick', format)
}

magick_config_internal <- function() {
    .Call('_magick_magick_config_internal', PACKAGE = 'magick')
}

magick_image_fx <- function(input, expression, channel) {
    .Call('_magick_magick_image_fx', PACKAGE = 'magick', input, expression, channel)
}

magick_image_fx_sequence <- function(input, expr) {
    .Call('_magick_magick_image_fx_sequence', PACKAGE = 'magick', input, expr)
}

magick_image_morphology <- function(input, method, kernel, iter, opt_names, opt_values) {
    .Call('_magick_magick_image_morphology', PACKAGE = 'magick', input, method, kernel, iter, opt_names, opt_values)
}

magick_image_convolve_kernel <- function(input, kernel, iter, scaling, bias) {
    .Call('_magick_magick_image_convolve_kernel', PACKAGE = 'magick', input, kernel, iter, scaling, bias)
}

magick_image_convolve_matrix <- function(input, matrix, iter, scaling, bias) {
    .Call('_magick_magick_image_convolve_matrix', PACKAGE = 'magick', input, matrix, iter, scaling, bias)
}

magick_device_internal <- function(bg, width, height, pointsize, res, clip, antialias, drawing) {
    .Call('_magick_magick_device_internal', PACKAGE = 'magick', bg, width, height, pointsize, res, clip, antialias, drawing)
}

magick_device_get <- function(n) {
    .Call('_magick_magick_device_get', PACKAGE = 'magick', n)
}

magick_device_pop <- function() {
    .Call('_magick_magick_device_pop', PACKAGE = 'magick')
}

magick_image_edge <- function(input, radius) {
    .Call('_magick_magick_image_edge', PACKAGE = 'magick', input, radius)
}

magick_image_canny <- function(input, geomstr) {
    .Call('_magick_magick_image_canny', PACKAGE = 'magick', input, geomstr)
}

magick_image_houghline <- function(input, geomstr, col, bg, lwd) {
    .Call('_magick_magick_image_houghline', PACKAGE = 'magick', input, geomstr, col, bg, lwd)
}

magick_image_readbitmap_native <- function(x) {
    .Call('_magick_magick_image_readbitmap_native', PACKAGE = 'magick', x)
}

magick_image_readbitmap_raster1 <- function(x) {
    .Call('_magick_magick_image_readbitmap_raster1', PACKAGE = 'magick', x)
}

magick_image_readbitmap_raster2 <- function(x) {
    .Call('_magick_magick_image_readbitmap_raster2', PACKAGE = 'magick', x)
}

magick_image_readbitmap_raw <- function(x) {
    .Call('_magick_magick_image_readbitmap_raw', PACKAGE = 'magick', x)
}

magick_image_readbitmap_double <- function(x) {
    .Call('_magick_magick_image_readbitmap_double', PACKAGE = 'magick', x)
}

magick_image_readbin <- function(x, density, depth, strip, defines) {
    .Call('_magick_magick_image_readbin', PACKAGE = 'magick', x, density, depth, strip, defines)
}

magick_image_readpath <- function(paths, density, depth, strip, defines) {
    .Call('_magick_magick_image_readpath', PACKAGE = 'magick', paths, density, depth, strip, defines)
}

magick_image_read_list <- function(list) {
    .Call('_magick_magick_image_read_list', PACKAGE = 'magick', list)
}

magick_image_write <- function(input, format, quality, depth, density, comment, compression) {
    .Call('_magick_magick_image_write', PACKAGE = 'magick', input, format, quality, depth, density, comment, compression)
}

magick_image_write_frame <- function(input, format, i = 1L) {
    .Call('_magick_magick_image_write_frame', PACKAGE = 'magick', input, format, i)
}

magick_image_write_integer <- function(input) {
    .Call('_magick_magick_image_write_integer', PACKAGE = 'magick', input)
}

magick_image_display <- function(image, animate) {
    .Call('_magick_magick_image_display', PACKAGE = 'magick', image, animate)
}

magick_image_fft <- function(image) {
    .Call('_magick_magick_image_fft', PACKAGE = 'magick', image)
}

magick_image_montage <- function(image, geometry, tile, gravity, bg = "white", shadow = FALSE) {
    .Call('_magick_magick_image_montage', PACKAGE = 'magick', image, geometry, tile, gravity, bg, shadow)
}

magick_image_strip <- function(input) {
    .Call('_magick_magick_image_strip', PACKAGE = 'magick', input)
}

magick_image_separate <- function(input, channel) {
    .Call('_magick_magick_image_separate', PACKAGE = 'magick', input, channel)
}

magick_image_combine <- function(input, colorspace, channel) {
    .Call('_magick_magick_image_combine', PACKAGE = 'magick', input, colorspace, channel)
}

magick_image_set_define <- function(input, format, name, value) {
    .Call('_magick_magick_image_set_define', PACKAGE = 'magick', input, format, name, value)
}

list_options <- function(str) {
    .Call('_magick_list_options', PACKAGE = 'magick', str)
}

set_magick_tempdir <- function(new_tmpdir) {
    .Call('_magick_set_magick_tempdir', PACKAGE = 'magick', new_tmpdir)
}

set_magick_seed <- function(seed) {
    invisible(.Call('_magick_set_magick_seed', PACKAGE = 'magick', seed))
}

magick_image_properties <- function(input) {
    .Call('_magick_magick_image_properties', PACKAGE = 'magick', input)
}

magick_image_scale <- function(input, geometry) {
    .Call('_magick_magick_image_scale', PACKAGE = 'magick', input, geometry)
}

magick_image_sample <- function(input, geometry) {
    .Call('_magick_magick_image_sample', PACKAGE = 'magick', input, geometry)
}

magick_image_resize <- function(input, geometry, filter) {
    .Call('_magick_magick_image_resize', PACKAGE = 'magick', input, geometry, filter)
}

magick_image_rotate <- function(input, degrees) {
    .Call('_magick_magick_image_rotate', PACKAGE = 'magick', input, degrees)
}

magick_image_chop <- function(input, geometry) {
    .Call('_magick_magick_image_chop', PACKAGE = 'magick', input, geometry)
}

magick_image_trim <- function(input, fuzz_percent) {
    .Call('_magick_magick_image_trim', PACKAGE = 'magick', input, fuzz_percent)
}

magick_image_flip <- function(input) {
    .Call('_magick_magick_image_flip', PACKAGE = 'magick', input)
}

magick_image_flop <- function(input) {
    .Call('_magick_magick_image_flop', PACKAGE = 'magick', input)
}

magick_image_shear <- function(input, geometry, color) {
    .Call('_magick_magick_image_shear', PACKAGE = 'magick', input, geometry, color)
}

magick_image_fuzzycmeans <- function(input, min_pixels = 1.0, smoothing = 1.5) {
    .Call('_magick_magick_image_fuzzycmeans', PACKAGE = 'magick', input, min_pixels, smoothing)
}

magick_image_connect <- function(input, connectivity = 4L) {
    .Call('_magick_magick_image_connect', PACKAGE = 'magick', input, connectivity)
}

magick_image_noise <- function(input, noisetype) {
    .Call('_magick_magick_image_noise', PACKAGE = 'magick', input, noisetype)
}

magick_image_blur <- function(input, radius = 1, sigma = 0.5) {
    .Call('_magick_magick_image_blur', PACKAGE = 'magick', input, radius, sigma)
}

magick_image_motion_blur <- function(input, radius = 1, sigma = 0.5, angle = 0.0) {
    .Call('_magick_magick_image_motion_blur', PACKAGE = 'magick', input, radius, sigma, angle)
}

magick_image_charcoal <- function(input, radius = 1, sigma = 0.5) {
    .Call('_magick_magick_image_charcoal', PACKAGE = 'magick', input, radius, sigma)
}

magick_image_deskew <- function(input, treshold) {
    .Call('_magick_magick_image_deskew', PACKAGE = 'magick', input, treshold)
}

magick_image_emboss <- function(input, radius = 1, sigma = 0.5) {
    .Call('_magick_magick_image_emboss', PACKAGE = 'magick', input, radius, sigma)
}

magick_image_fill <- function(input, color, point, fuzz_percent, border_color) {
    .Call('_magick_magick_image_fill', PACKAGE = 'magick', input, color, point, fuzz_percent, border_color)
}

magick_image_negate <- function(input) {
    .Call('_magick_magick_image_negate', PACKAGE = 'magick', input)
}

magick_image_oilpaint <- function(input, radius) {
    .Call('_magick_magick_image_oilpaint', PACKAGE = 'magick', input, radius)
}

magick_image_implode <- function(input, factor) {
    .Call('_magick_magick_image_implode', PACKAGE = 'magick', input, factor)
}

magick_image_format <- function(input, format, type, space, depth, antialias, matte, interlace) {
    .Call('_magick_magick_image_format', PACKAGE = 'magick', input, format, type, space, depth, antialias, matte, interlace)
}

magick_image_page <- function(input, pagesize, density) {
    .Call('_magick_magick_image_page', PACKAGE = 'magick', input, pagesize, density)
}

magick_image_repage <- function(input) {
    .Call('_magick_magick_image_repage', PACKAGE = 'magick', input)
}

magick_image_orient <- function(input, orientation) {
    .Call('_magick_magick_image_orient', PACKAGE = 'magick', input, orientation)
}

magick_image_despeckle <- function(input, times) {
    .Call('_magick_magick_image_despeckle', PACKAGE = 'magick', input, times)
}

magick_image_reducenoise <- function(input, radius) {
    .Call('_magick_magick_image_reducenoise', PACKAGE = 'magick', input, radius)
}

magick_image_annotate <- function(input, text, gravity, location, rot, size, font, style, weight, kerning, decoration, color, strokecolor, boxcolor) {
    .Call('_magick_magick_image_annotate', PACKAGE = 'magick', input, text, gravity, location, rot, size, font, style, weight, kerning, decoration, color, strokecolor, boxcolor)
}

magick_image_compare <- function(input, reference_image, metric, fuzz_percent) {
    .Call('_magick_magick_image_compare', PACKAGE = 'magick', input, reference_image, metric, fuzz_percent)
}

magick_image_distort <- function(input, method, values, bestfit) {
    .Call('_magick_magick_image_distort', PACKAGE = 'magick', input, method, values, bestfit)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

magick_image_animate <- function(input, delay, iter, method, optimize) {
    .Call('_magick_magick_image_animate', PACKAGE = 'magick', input, delay, iter, method, optimize)
}

magick_image_coalesce <- function(input) {
    .Call('_magick_magick_image_coalesce', PACKAGE = 'magick', input)
}

magick_image_morph <- function(image, frames) {
    .Call('_magick_magick_image_morph', PACKAGE = 'magick', image, frames)
}

magick_image_mosaic <- function(input, composite) {
    .Call('_magick_magick_image_mosaic', PACKAGE = 'magick', input, composite)
}

magick_image_flatten <- function(input, composite) {
    .Call('_magick_magick_image_flatten', PACKAGE = 'magick', input, composite)
}

magick_image_average <- function(image) {
    .Call('_magick_magick_image_average', PACKAGE = 'magick', image)
}

magick_image_append <- function(image, stack) {
    .Call('_magick_magick_image_append', PACKAGE = 'magick', image, stack)
}

magick_attr_comment <- function(input, set) {
    .Call('_magick_magick_attr_comment', PACKAGE = 'magick', input, set)
}

magick_attr_text_antialias <- function(input, set) {
    .Call('_magick_magick_attr_text_antialias', PACKAGE = 'magick', input, set)
}

magick_attr_stroke_antialias <- function(input, set) {
    .Call('_magick_magick_attr_stroke_antialias', PACKAGE = 'magick', input, set)
}

magick_attr_animationdelay <- function(input, delay) {
    .Call('_magick_magick_attr_animationdelay', PACKAGE = 'magick', input, delay)
}

magick_attr_backgroundcolor <- function(input, color) {
    .Call('_magick_magick_attr_backgroundcolor', PACKAGE = 'magick', input, color)
}

magick_attr_boxcolor <- function(input, color) {
    .Call('_magick_magick_attr_boxcolor', PACKAGE = 'magick', input, color)
}

magick_attr_fillcolor <- function(input, color) {
    .Call('_magick_magick_attr_fillcolor', PACKAGE = 'magick', input, color)
}

magick_attr_font <- function(input, font) {
    .Call('_magick_magick_attr_font', PACKAGE = 'magick', input, font)
}

magick_attr_fontsize <- function(input, pointsize) {
    .Call('_magick_magick_attr_fontsize', PACKAGE = 'magick', input, pointsize)
}

magick_attr_label <- function(input, label) {
    .Call('_magick_magick_attr_label', PACKAGE = 'magick', input, label)
}

magick_attr_format <- function(input, format) {
    .Call('_magick_magick_attr_format', PACKAGE = 'magick', input, format)
}

magick_attr_quality <- function(input, quality) {
    .Call('_magick_magick_attr_quality', PACKAGE = 'magick', input, quality)
}

magick_attr_quantize <- function(input, numcolors) {
    .Call('_magick_magick_attr_quantize', PACKAGE = 'magick', input, numcolors)
}

magick_attr_density <- function(input, density) {
    .Call('_magick_magick_attr_density', PACKAGE = 'magick', input, density)
}

magick_image_info <- function(input) {
    .Call('_magick_magick_image_info', PACKAGE = 'magick', input)
}

magick_image_as_raster <- function(data) {
    .Call('_magick_magick_image_as_raster', PACKAGE = 'magick', data)
}

magick_threads <- function(i = 0L) {
    .Call('_magick_magick_threads', PACKAGE = 'magick', i)
}

magick_image_dead <- function(image) {
    .Call('_magick_magick_image_dead', PACKAGE = 'magick', image)
}

magick_image_length <- function(image) {
    .Call('_magick_magick_image_length', PACKAGE = 'magick', image)
}

create <- function(len) {
    .Call('_magick_create', PACKAGE = 'magick', len)
}

magick_image_blank <- function(width, height, color, pseudo_image, options) {
    .Call('_magick_magick_image_blank', PACKAGE = 'magick', width, height, color, pseudo_image, options)
}

copy <- function(image) {
    .Call('_magick_copy', PACKAGE = 'magick', image)
}

magick_image_rev <- function(input) {
    .Call('_magick_magick_image_rev', PACKAGE = 'magick', input)
}

magick_image_join <- function(input) {
    .Call('_magick_magick_image_join', PACKAGE = 'magick', input)
}

magick_image_subset <- function(image, index) {
    .Call('_magick_magick_image_subset', PACKAGE = 'magick', image, index)
}

magick_image_replace <- function(image, index, value) {
    .Call('_magick_magick_image_replace', PACKAGE = 'magick', image, index, value)
}

magick_image_copy <- function(image, add) {
    .Call('_magick_magick_image_copy', PACKAGE = 'magick', image, add)
}

magick_image_destroy <- function(image) {
    invisible(.Call('_magick_magick_image_destroy', PACKAGE = 'magick', image))
}

autobrewed <- function() {
    .Call('_magick_autobrewed', PACKAGE = 'magick')
}

magick_resource_limits <- function() {
    .Call('_magick_magick_resource_limits', PACKAGE = 'magick')
}

magick_image_contrast <- function(input, sharpen) {
    .Call('_magick_magick_image_contrast', PACKAGE = 'magick', input, sharpen)
}

magick_image_normalize <- function(input) {
    .Call('_magick_magick_image_normalize', PACKAGE = 'magick', input)
}

magick_image_modulate <- function(input, brightness, saturation, hue) {
    .Call('_magick_magick_image_modulate', PACKAGE = 'magick', input, brightness, saturation, hue)
}

magick_image_map <- function(input, map_image, dither) {
    .Call('_magick_magick_image_map', PACKAGE = 'magick', input, map_image, dither)
}

magick_image_channel <- function(input, channel) {
    .Call('_magick_magick_image_channel', PACKAGE = 'magick', input, channel)
}

magick_image_colorize <- function(input, opacity, color) {
    .Call('_magick_magick_image_colorize', PACKAGE = 'magick', input, opacity, color)
}

magick_image_enhance <- function(input) {
    .Call('_magick_magick_image_enhance', PACKAGE = 'magick', input)
}

magick_image_equalize <- function(input) {
    .Call('_magick_magick_image_equalize', PACKAGE = 'magick', input)
}

magick_image_median <- function(input, radius) {
    .Call('_magick_magick_image_median', PACKAGE = 'magick', input, radius)
}

magick_image_quantize <- function(input, max, space, dither, depth) {
    .Call('_magick_magick_image_quantize', PACKAGE = 'magick', input, max, space, dither, depth)
}

magick_image_ordered_dither <- function(input, threshold_map) {
    .Call('_magick_magick_image_ordered_dither', PACKAGE = 'magick', input, threshold_map)
}

magick_image_transparent <- function(input, color, fuzz_percent) {
    .Call('_magick_magick_image_transparent', PACKAGE = 'magick', input, color, fuzz_percent)
}

magick_image_background <- function(input, color) {
    .Call('_magick_magick_image_background', PACKAGE = 'magick', input, color)
}

magick_image_lat <- function(input, geomstr) {
    .Call('_magick_magick_image_lat', PACKAGE = 'magick', input, geomstr)
}

magick_image_threshold_black <- function(input, threshold, channel) {
    .Call('_magick_magick_image_threshold_black', PACKAGE = 'magick', input, threshold, channel)
}

magick_image_threshold_white <- function(input, threshold, channel) {
    .Call('_magick_magick_image_threshold_white', PACKAGE = 'magick', input, threshold, channel)
}

magick_image_level <- function(input, black_pct, white_pct, mid_point, channel) {
    .Call('_magick_magick_image_level', PACKAGE = 'magick', input, black_pct, white_pct, mid_point, channel)
}

magick_image_composite <- function(input, composite_image, offset, gravity, composite, args) {
    .Call('_magick_magick_image_composite', PACKAGE = 'magick', input, composite_image, offset, gravity, composite, args)
}

magick_image_border <- function(input, color, geometry, composite) {
    .Call('_magick_magick_image_border', PACKAGE = 'magick', input, color, geometry, composite)
}

magick_image_frame <- function(input, color, geometry) {
    .Call('_magick_magick_image_frame', PACKAGE = 'magick', input, color, geometry)
}

magick_image_shadow_mask <- function(input, geomstr) {
    .Call('_magick_magick_image_shadow_mask', PACKAGE = 'magick', input, geomstr)
}

magick_image_shade <- function(input, azimuth = 30, elevation = 30, color = FALSE) {
    .Call('_magick_magick_image_shade', PACKAGE = 'magick', input, azimuth, elevation, color)
}

magick_image_crop <- function(input, geometry, gravity, repage) {
    .Call('_magick_magick_image_crop', PACKAGE = 'magick', input, geometry, gravity, repage)
}

magick_image_extent <- function(input, geometry, gravity, color) {
    .Call('_magick_magick_image_extent', PACKAGE = 'magick', input, geometry, gravity, color)
}

magick_image_artifact <- function(input, name) {
    .Call('_magick_magick_image_artifact', PACKAGE = 'magick', input, name)
}

magick_coder_info <- function(format) {
    .Call('_magick_magick_coder_info', PACKAGE = 'magick', format)
}

magick_config_internal <- function() {
    .Call('_magick_magick_config_internal', PACKAGE = 'magick')
}

magick_image_fx <- function(input, expression, channel) {
    .Call('_magick_magick_image_fx', PACKAGE = 'magick', input, expression, channel)
}

magick_image_fx_sequence <- function(input, expr) {
    .Call('_magick_magick_image_fx_sequence', PACKAGE = 'magick', input, expr)
}

magick_image_morphology <- function(input, method, kernel, iter, opt_names, opt_values) {
    .Call('_magick_magick_image_morphology', PACKAGE = 'magick', input, method, kernel, iter, opt_names, opt_values)
}

magick_image_convolve_kernel <- function(input, kernel, iter, scaling, bias) {
    .Call('_magick_magick_image_convolve_kernel', PACKAGE = 'magick', input, kernel, iter, scaling, bias)
}

magick_image_convolve_matrix <- function(input, matrix, iter, scaling, bias) {
    .Call('_magick_magick_image_convolve_matrix', PACKAGE = 'magick', input, matrix, iter, scaling, bias)
}

magick_device_internal <- function(bg, width, height, pointsize, res, clip, antialias, drawing) {
    .Call('_magick_magick_device_internal', PACKAGE = 'magick', bg, width, height, pointsize, res, clip, antialias, drawing)
}

magick_device_get <- function(n) {
    .Call('_magick_magick_device_get', PACKAGE = 'magick', n)
}

magick_device_pop <- function() {
    .Call('_magick_magick_device_pop', PACKAGE = 'magick')
}

magick_image_edge <- function(input, radius) {
    .Call('_magick_magick_image_edge', PACKAGE = 'magick', input, radius)
}

magick_image_canny <- function(input, geomstr) {
    .Call('_magick_magick_image_canny', PACKAGE = 'magick', input, geomstr)
}

magick_image_houghline <- function(input, geomstr, col, bg, lwd) {
    .Call('_magick_magick_image_houghline', PACKAGE = 'magick', input, geomstr, col, bg, lwd)
}

magick_image_readbitmap_native <- function(x) {
    .Call('_magick_magick_image_readbitmap_native', PACKAGE = 'magick', x)
}

magick_image_readbitmap_raster1 <- function(x) {
    .Call('_magick_magick_image_readbitmap_raster1', PACKAGE = 'magick', x)
}

magick_image_readbitmap_raster2 <- function(x) {
    .Call('_magick_magick_image_readbitmap_raster2', PACKAGE = 'magick', x)
}

magick_image_readbitmap_raw <- function(x) {
    .Call('_magick_magick_image_readbitmap_raw', PACKAGE = 'magick', x)
}

magick_image_readbitmap_double <- function(x) {
    .Call('_magick_magick_image_readbitmap_double', PACKAGE = 'magick', x)
}

magick_image_readbin <- function(x, density, depth, strip, defines) {
    .Call('_magick_magick_image_readbin', PACKAGE = 'magick', x, density, depth, strip, defines)
}

magick_image_readpath <- function(paths, density, depth, strip, defines) {
    .Call('_magick_magick_image_readpath', PACKAGE = 'magick', paths, density, depth, strip, defines)
}

magick_image_read_list <- function(list) {
    .Call('_magick_magick_image_read_list', PACKAGE = 'magick', list)
}

magick_image_write <- function(input, format, quality, depth, density, comment, compression) {
    .Call('_magick_magick_image_write', PACKAGE = 'magick', input, format, quality, depth, density, comment, compression)
}

magick_image_write_frame <- function(input, format, i = 1L) {
    .Call('_magick_magick_image_write_frame', PACKAGE = 'magick', input, format, i)
}

magick_image_write_integer <- function(input) {
    .Call('_magick_magick_image_write_integer', PACKAGE = 'magick', input)
}

magick_image_display <- function(image, animate) {
    .Call('_magick_magick_image_display', PACKAGE = 'magick', image, animate)
}

magick_image_fft <- function(image) {
    .Call('_magick_magick_image_fft', PACKAGE = 'magick', image)
}

magick_image_montage <- function(image, geometry, tile, gravity, bg = "white", shadow = FALSE) {
    .Call('_magick_magick_image_montage', PACKAGE = 'magick', image, geometry, tile, gravity, bg, shadow)
}

magick_image_strip <- function(input) {
    .Call('_magick_magick_image_strip', PACKAGE = 'magick', input)
}

magick_image_separate <- function(input, channel) {
    .Call('_magick_magick_image_separate', PACKAGE = 'magick', input, channel)
}

magick_image_combine <- function(input, colorspace, channel) {
    .Call('_magick_magick_image_combine', PACKAGE = 'magick', input, colorspace, channel)
}

magick_image_set_define <- function(input, format, name, value) {
    .Call('_magick_magick_image_set_define', PACKAGE = 'magick', input, format, name, value)
}

list_options <- function(str) {
    .Call('_magick_list_options', PACKAGE = 'magick', str)
}

set_magick_tempdir <- function(new_tmpdir) {
    .Call('_magick_set_magick_tempdir', PACKAGE = 'magick', new_tmpdir)
}

set_magick_seed <- function(seed) {
    invisible(.Call('_magick_set_magick_seed', PACKAGE = 'magick', seed))
}

magick_image_properties <- function(input) {
    .Call('_magick_magick_image_properties', PACKAGE = 'magick', input)
}

magick_image_scale <- function(input, geometry) {
    .Call('_magick_magick_image_scale', PACKAGE = 'magick', input, geometry)
}

magick_image_sample <- function(input, geometry) {
    .Call('_magick_magick_image_sample', PACKAGE = 'magick', input, geometry)
}

magick_image_resize <- function(input, geometry, filter) {
    .Call('_magick_magick_image_resize', PACKAGE = 'magick', input, geometry, filter)
}

magick_image_rotate <- function(input, degrees) {
    .Call('_magick_magick_image_rotate', PACKAGE = 'magick', input, degrees)
}

magick_image_chop <- function(input, geometry) {
    .Call('_magick_magick_image_chop', PACKAGE = 'magick', input, geometry)
}

magick_image_trim <- function(input, fuzz_percent) {
    .Call('_magick_magick_image_trim', PACKAGE = 'magick', input, fuzz_percent)
}

magick_image_flip <- function(input) {
    .Call('_magick_magick_image_flip', PACKAGE = 'magick', input)
}

magick_image_flop <- function(input) {
    .Call('_magick_magick_image_flop', PACKAGE = 'magick', input)
}

magick_image_shear <- function(input, geometry, color) {
    .Call('_magick_magick_image_shear', PACKAGE = 'magick', input, geometry, color)
}

magick_image_fuzzycmeans <- function(input, min_pixels = 1.0, smoothing = 1.5) {
    .Call('_magick_magick_image_fuzzycmeans', PACKAGE = 'magick', input, min_pixels, smoothing)
}

magick_image_connect <- function(input, connectivity = 4L) {
    .Call('_magick_magick_image_connect', PACKAGE = 'magick', input, connectivity)
}

magick_image_noise <- function(input, noisetype) {
    .Call('_magick_magick_image_noise', PACKAGE = 'magick', input, noisetype)
}

magick_image_blur <- function(input, radius = 1, sigma = 0.5) {
    .Call('_magick_magick_image_blur', PACKAGE = 'magick', input, radius, sigma)
}

magick_image_motion_blur <- function(input, radius = 1, sigma = 0.5, angle = 0.0) {
    .Call('_magick_magick_image_motion_blur', PACKAGE = 'magick', input, radius, sigma, angle)
}

magick_image_charcoal <- function(input, radius = 1, sigma = 0.5) {
    .Call('_magick_magick_image_charcoal', PACKAGE = 'magick', input, radius, sigma)
}

magick_image_deskew <- function(input, treshold) {
    .Call('_magick_magick_image_deskew', PACKAGE = 'magick', input, treshold)
}

magick_image_emboss <- function(input, radius = 1, sigma = 0.5) {
    .Call('_magick_magick_image_emboss', PACKAGE = 'magick', input, radius, sigma)
}

magick_image_fill <- function(input, color, point, fuzz_percent, border_color) {
    .Call('_magick_magick_image_fill', PACKAGE = 'magick', input, color, point, fuzz_percent, border_color)
}

magick_image_negate <- function(input) {
    .Call('_magick_magick_image_negate', PACKAGE = 'magick', input)
}

magick_image_oilpaint <- function(input, radius) {
    .Call('_magick_magick_image_oilpaint', PACKAGE = 'magick', input, radius)
}

magick_image_implode <- function(input, factor) {
    .Call('_magick_magick_image_implode', PACKAGE = 'magick', input, factor)
}

magick_image_format <- function(input, format, type, space, depth, antialias, matte, interlace) {
    .Call('_magick_magick_image_format', PACKAGE = 'magick', input, format, type, space, depth, antialias, matte, interlace)
}

magick_image_page <- function(input, pagesize, density) {
    .Call('_magick_magick_image_page', PACKAGE = 'magick', input, pagesize, density)
}

magick_image_repage <- function(input) {
    .Call('_magick_magick_image_repage', PACKAGE = 'magick', input)
}

magick_image_orient <- function(input, orientation) {
    .Call('_magick_magick_image_orient', PACKAGE = 'magick', input, orientation)
}

magick_image_despeckle <- function(input, times) {
    .Call('_magick_magick_image_despeckle', PACKAGE = 'magick', input, times)
}

magick_image_reducenoise <- function(input, radius) {
    .Call('_magick_magick_image_reducenoise', PACKAGE = 'magick', input, radius)
}

magick_image_annotate <- function(input, text, gravity, location, rot, size, font, style, weight, kerning, decoration, color, strokecolor, strokewidth, boxcolor) {
    .Call('_magick_magick_image_annotate', PACKAGE = 'magick', input, text, gravity, location, rot, size, font, style, weight, kerning, decoration, color, strokecolor, strokewidth, boxcolor)
}

magick_image_compare <- function(input, reference_image, metric, fuzz_percent) {
    .Call('_magick_magick_image_compare', PACKAGE = 'magick', input, reference_image, metric, fuzz_percent)
}

magick_image_distort <- function(input, method, values, bestfit) {
    .Call('_magick_magick_image_distort', PACKAGE = 'magick', input, method, values, bestfit)
}

#' Image thresholding
#'
#' Thresholding an image can be used for simple and straightforward image segmentation.
#' The function [image_threshold()] allows to do black and white thresholding whereas
#' [image_lat()] performs local adaptive thresholding.
#'
#'  - `image_threshold(type = "black")`: Forces all pixels below the threshold into black while leaving all pixels
#' at or above the threshold unchanged
#'  - `image_threshold(type = "white")`: Forces all pixels above the threshold into white while leaving all pixels
#' at or below the threshold unchanged
#'  - `image_lat()`: Local Adaptive Thresholding. Looks in a box (width x height) around the
#' pixel neighborhood if the pixel value is bigger than the average minus an offset.
#'
#' @export
#' @name thresholding
#' @rdname thresholding
#' @inheritParams editing
#' @param type type of thresholding, either one of lat, black or white (see details below)
#' @param threshold pixel intensity threshold percentage for black or white thresholding
#' @param channel a value of [channel_types()] specifying which channel(s) to set
#' @examples
#' test <- image_convert(logo, colorspace = "Gray")
#' image_threshold(test, type = "black", threshold = "50%")
#' image_threshold(test, type = "white", threshold = "50%")
#'
#' # Turn image into BW
#' test %>%
#'   image_threshold(type = "white", threshold = "50%") %>%
#'   image_threshold(type = "black", threshold = "50%")
#'
#' # adaptive thresholding
#' image_lat(test, geometry = '10x10+5%')
image_threshold <- function(image, type = c("black", "white"), threshold = "50%", channel = NULL){
  type <- match.arg(type)
  assert_image(image)
  if(type == "black"){
    magick_image_threshold_black(image, as.character(threshold), as.character(channel))
  } else {
    magick_image_threshold_white(image, as.character(threshold), as.character(channel))
  }
}

#' @export
#' @rdname thresholding
#' @param black_point value between 0 and 100, the darkest color in the image
#' @param white_point value between 0 and 100, the lightest color in the image
#' @param mid_point value between 0 and 10 used for gamma correction
image_level <- function(image, black_point = 0, white_point = 100, mid_point = 1, channel = NULL){
  assert_image(image)
  black_point <- as.numeric(black_point)
  white_point <- as.numeric(white_point)
  mid_point <- as.numeric(mid_point)
  channel <- as.character(channel)
  magick_image_level(image, black_point, white_point, mid_point, channel)
}

#' @export
#' @rdname thresholding
#' @param geometry pixel window plus offset for LAT algorithm
image_lat <- function(image, geometry = '10x10+5%'){
  assert_image(image)
  magick_image_lat(image, geometry)
}

#' Image Editing
#'
#' Read, write and join or combine images. All image functions are vectorized, meaning
#' they operate either on a single frame or a series of frames (e.g. a collage, video,
#' or animation). Besides paths and URLs, [image_read()] supports commonly used bitmap
#' and raster object types.
#'
#' All standard base vector methods such as \link{[}, \link{[[}, [c()], [as.list()],
#' [as.raster()], [rev()], [length()], and [print()] can be used to work with magick
#' image objects. Use the standard \code{img[i]} syntax to extract a subset of the frames
#' from an image. The \code{img[[i]]} method is an alias for [image_data()] which extracts
#' a single frame as a raw bitmap matrix with pixel values.
#'
#' For reading svg or pdf it is recommended to use `image_read_svg()` and `image_read_pdf()`
#' if the [rsvg][rsvg::rsvg] and [pdftools][pdftools::pdf_render_page] R packages are available.
#' These functions provide more rendering options (including rendering of literal svg) and
#' better quality than built-in svg/pdf rendering delegates from imagemagick itself.
#'
#' X11 is required for `image_display()` which is only works on some platforms. A more
#' portable method is `image_browse()` which opens the image in a browser. RStudio has
#' an embedded viewer that does this automatically which is quite nice.
#'
#' Image objects are automatically released by the garbage collector when they are no longer
#' reachable. Because the GC only runs once in a while, you can also call `image_destroy()`
#' explicitly to release the memory immediately. This is usually only needed if you create
#' a lot of images in a short period of time, and you might run out of memory.
#'
#' @importFrom Rcpp sourceCpp
#' @useDynLib magick
#' @export
#' @family image
#' @rdname editing
#' @name editing
#' @inheritParams defines
#' @param path a file, url, or raster object or bitmap array
#' @param image magick image object returned by [image_read()] or [image_graph()]
#' @param density resolution to render pdf or svg
#' @param strip drop image comments and metadata
#' @param coalesce automatically [image_coalesce()] gif images
#' @examples
#' # Download image from the web
#' frink <- image_read("https://jeroen.github.io/images/frink.png")
#' worldcup_frink <- image_fill(frink, "orange", "+100+200", 20)
#' image_write(worldcup_frink, "output.png")
#'
#' # extract raw bitmap array
#' bitmap <- frink[[1]]
#'
#' # replace pixels with #FF69B4 ('hot pink') and convert back to image
#' bitmap[,50:100, 50:100] <- as.raw(c(0xff, 0x69, 0xb4, 0xff))
#' image_read(bitmap)
#'
#' # Plot to graphics device via legacy raster format
#' raster <- as.raster(frink)
#' par(ask=FALSE)
#' plot(raster)
#'
#' # Read bitmap arrays from from other image packages
#' curl::curl_download("https://jeroen.github.io/images/example.webp", "example.webp")
#' if(require(webp)) image_read(webp::read_webp("example.webp"))
#' unlink(c("example.webp", "output.png"))
image_read <- function(path, density = NULL, depth = NULL, strip = FALSE, coalesce = TRUE, defines = NULL){
  if(is.numeric(density))
    density <- paste0(density, "x", density)
  density <- as.character(density)
  depth <- as.integer(depth)
  defines <- validate_defines(defines)
  image <- if(isS4(path) && methods::is(path, "Image")){
    convert_EBImage(path)
  } else if(inherits(path, "nativeRaster") || (is.matrix(path) && is.integer(path))){
    image_read_nativeraster(path)
  } else if(inherits(path, "cimg")) {
    image_read_cimg((path))
  } else if (grDevices::is.raster(path)) {
    image_read_raster2(path)
  } else if (is.matrix(path) && is.character(path)){
    image_read_raster2(grDevices::as.raster(path))
  } else if(is.array(path)){
    image_readbitmap(path)
  } else if(is.raw(path)) {
    magick_image_readbin(path, density, depth, strip, defines)
  } else if(is.character(path) && all(nchar(path))){
    path <- vapply(path, replace_url, character(1))
    # See https://github.com/ropensci/magick/issues/285
    path <- if(is_windows()){
      enc2utf8(path)
    } else {
      enc2native(path)
    }
    magick_image_readpath(path, density, depth, strip, defines)
  } else {
    stop("path must be URL, filename or raw vector")
  }
  if(is.character(path) && !isTRUE(magick_config()$rsvg)){
    if(any(grepl("\\.svg$", tolower(path))) || any(grepl("svg|mvg", tolower(image_info(image)$format)))){
      warning("ImageMagick was built without librsvg which causes poor qualty of SVG rendering.
For better results use image_read_svg() which uses the rsvg package.", call. = FALSE)
    }
  }
  if(isTRUE(coalesce) && length(image) > 1 && identical('GIF', toupper(image_info(image)$format[1]))){
    return(image_coalesce(image))
  }
  return(image)
}

#' @export
#' @rdname editing
#' @examples if(require(rsvg)){
#' tiger <- image_read_svg("http://jeroen.github.io/images/tiger.svg")
#' svgtxt <- '<?xml version="1.0" encoding="UTF-8"?>
#' <svg width="400" height="400" viewBox="0 0 400 400" fill="none">
#'  <circle fill="steelblue" cx="200" cy="200" r="100" />
#'  <circle fill="yellow" cx="200" cy="200" r="90" />
#' </svg>'
#' circles <- image_read_svg(svgtxt)
#' }
image_read_svg <- function(path, width = NULL, height = NULL){
  path <- vapply(path, replace_url, character(1))
  images <- lapply(path, function(x){
    if(is_svg(x)) x <- charToRaw(x)
    bitmap <- rsvg::rsvg_raw(x, width = width, height = height)
    magick_image_readbitmap_raw(bitmap)
  })
  image_join(images)
}

#' @export
#' @rdname editing
#' @param pages integer vector with page numbers. Defaults to all pages.
#' @param password user [password][pdftools::pdf_render_page] to open protected pdf files
#' @examples if(require(pdftools))
#' image_read_pdf(file.path(R.home('doc'), 'NEWS.pdf'), pages = 1, density = 100)
image_read_pdf <- function(path, pages = NULL, density = 300, password = ""){
  path <- replace_url(path)
  if(!length(pages))
    pages <- seq_len(pdftools::pdf_info(path, opw = password)$pages)
  images <- lapply(pages, function(page){
    bitmap <- pdftools::pdf_render_page(path, page = page, dpi = density, opw = password)
    magick_image_readbitmap_raw(bitmap)
  })
  out <- image_join(images)
  if(length(density))
    try(magick_attr_density(out, paste0(density, 'x', density)))
  out
}

#' @export
#' @rdname editing
#' @param fps how many images to capture per second of video. Set to
#' `NULL` to get all frames from the input video.
image_read_video <- function(path, fps = 1, format = 'png'){
  if(!requireNamespace('av', quietly = TRUE) || utils::packageVersion('av') < '0.3')
    stop("Please install or update the 'av' package")
  images <- av::av_video_images(path, fps = fps, format = format)
  on.exit(unlink(images))
  image_read(images)
}

image_readbitmap <- function(x){
  if(length(dim(x)) != 3)
    stop("Only 3D arrays can be converted to bitmaps")
  if(is.raw(x)){
    magick_image_readbitmap_raw(x)
  } else if(is.double(x)){
    magick_image_readbitmap_double(aperm(x))
  } else {
    stop("Unsupported bitmap array type")
  }
}

# output of dev.caputure(native = TRUE)
image_read_nativeraster <- function(x){
  stopifnot(is.matrix(x) && is.integer(x))
  magick_image_readbitmap_native(x)
}

# output of dev.caputure(native = FALSE) or as.raster()
image_read_raster1 <- function(x){
  stopifnot(is.matrix(x) && is.character(x))
  magick_image_readbitmap_raster1(t(x))
}

# output of as.raster()
image_read_raster2 <- function(x){
  stopifnot(is.matrix(x) && is.character(x))
  magick_image_readbitmap_raster2(x)
}

# This can probably be more efficient
image_read_cimg <- function(x){
  rast <- as.raster(x)
  if(is.list(rast)){
    image_join(lapply(rast, image_read))
  } else {
    image_read(rast)
  }
}

#EBImage BioConductor class
convert_EBImage <- function(x){
  if(length(dim(x@.Data)) == 2)
    dim(x@.Data) <- c(dim(x@.Data), 1L)
  img <- if(x@colormode == 2L){
    image_read(x@.Data)
  } else if(x@colormode == 0L){
    image_join(lapply(seq_len(dim(x@.Data)[3]), function(i){
      image_read(x@.Data[,,i,drop = FALSE])
    }))
  } else {
    stop("Unknown colormode in EBImage class")
  }
  image_flop(image_rotate(img, 90))
}

#' @export
#' @inheritParams effects
#' @rdname editing
#' @param flatten should image be flattened before writing? This also replaces
#' transparency with background color.
#' @param quality number between 0 and 100 for jpeg quality. Defaults to 75.
#' @param comment text string added to the image metadata for supported formats
#' @param compression a string with compression type from [compress_types]
image_write <- function(image, path = NULL, format = NULL, quality = NULL,
                        depth = NULL, density = NULL, comment = NULL, flatten = FALSE,
                        defines = NULL, compression = NULL){
  assert_image(image)
  if(!length(image))
    warning("Writing image with 0 frames")
  if(isTRUE(flatten))
    image <- image_flatten(image)
  format <- toupper(as.character(format))
  quality <- as.integer(quality)
  depth <- as.integer(depth)
  density <- as.character(density)
  comment <- as.character(comment)
  compression <- as.character(compression)
  if(length(defines)){
    image_set_defines(image, defines = defines)
    defines[seq_along(defines)] = NA_character_;
    on.exit(image_set_defines(image, defines = defines))
  }
  buf <- magick_image_write(image, format, quality, depth, density, comment, compression)
  if(is.character(path)){
    writeBin(buf, path)
    return(invisible(path))
  }
  return(buf)
}

#' @export
#' @rdname editing
#' @param format output format such as `"png"`, `"jpeg"`, `"gif"`, `"rgb"` or `"rgba"`.
#' @param depth color depth (either 8 or 16)
#' @param antialias enable anti-aliasing for text and strokes
#' @param type string with [imagetype](https://www.imagemagick.org/Magick++/Enumerations.html#ImageType)
#' value from [image_types][image_types] for example `grayscale` to convert into black/white
#' @param colorspace string with a [`colorspace`](https://www.imagemagick.org/Magick++/Enumerations.html#ColorspaceType)
#' from [colorspace_types][colorspace_types] for example `"gray"`, `"rgb"` or `"cmyk"`
#' @param matte set to `TRUE` or `FALSE` to enable or disable transparency
#' @param interlace string with [interlace](https://www.imagemagick.org/Magick++/Enumerations.html#InterlaceType)
image_convert <- function(image, format = NULL, type = NULL, colorspace = NULL,
                          depth = NULL, antialias = NULL, matte = NULL, interlace = NULL){
  assert_image(image)
  depth <- as.integer(depth)
  antialias <- as.logical(antialias)
  type <- as.character(type)
  colorspace <- as.character(colorspace)
  matte <- as.logical(matte)
  interlace <- as.character(interlace)
  if(length(depth) && is.na(match(depth, c(8, 16))))
    stop('depth must be 8 or 16 bit')
  magick_image_format(image, toupper(format), type, colorspace, depth, antialias, matte, interlace)
}

image_write_frame <- function(image, format = "rgba", i = 1){
  magick_image_write_frame(image, format = format, i = i)
}

#' @export
#' @rdname editing
#' @param channels string with image channel(s) for example `"rgb"`, `"rgba"`,
#' `"cmyk"`,`"gray"`, or `"ycbcr"`. Default is either `"gray"`, `"rgb"` or `"rgba"`
#' depending on the image
#' @param frame integer setting which frame to extract from the image
image_data <- function(image, channels = NULL, frame = 1){
  if(length(image) > 1 || frame > 1)
    image <- image[frame]
  if(!length(channels) || !nchar(channels)){
    info <- image_info(image)
    channels <- if(tolower(info$colorspace) == "gray"){
      "gray"
    } else if(isTRUE(info$matte)){
      "rgba"
    } else {
      "rgb"
    }
  }
  if(!grepl("a$", channels)) #output has no transparency channel
    image <- image_flatten(image)
  image_write_frame(image, format = channels)
}

#' @export
#' @rdname editing
#' @param tidy converts raster data to long form for use with [geom_raster][ggplot2::geom_raster].
#' If `FALSE` output is the same as `as.raster()`.
image_raster <- function(image, frame = 1, tidy = TRUE){
  raster <- as.raster(image[frame])
  if(!isTRUE(tidy))
    return(raster)
  data.frame(x = rep(seq_len(ncol(raster)), nrow(raster)),
             y = rep(seq_len(nrow(raster)), each = ncol(raster)),
             col = as.vector(raster),
             stringsAsFactors = FALSE)
}


#' @export
#' @param animate support animations in the X11 display
#' @rdname editing
image_display <- function(image, animate = TRUE){
  magick_image_display(image, animate)
}

#' @export
#' @param browser argument passed to [browseURL][utils::browseURL]
#' @rdname editing
image_browse <- function(image, browser = getOption("browser")){
  ext <- ifelse(length(image), tolower(image_info(image[1])$format), "gif")
  tmp <- tempfile(fileext = paste0(".", ext))
  image_write(image, path = tmp)
  utils::browseURL(tmp)
}

#' @export
#' @rdname editing
image_strip <- function(image){
  assert_image(image)
  magick_image_strip(image)
}


#' @export
#' @rdname editing
#' @inheritParams device
#' @inheritParams painting
#' @param pseudo_image string with [pseudo image](http://www.imagemagick.org/script/formats.php#pseudo)
#' specification for example `"radial-gradient:purple-yellow"`
#' @examples # create a solid canvas
#' image_blank(600, 400, "green")
#' image_blank(600, 400, pseudo_image = "radial-gradient:purple-yellow")
#' image_blank(200, 200, pseudo_image = "gradient:#3498db-#db3a34",
#'   defines = c('gradient:direction' = 'east'))
image_blank <- function(width, height, color = "none", pseudo_image = "", defines = NULL){
  width <- as.numeric(width)
  height <- as.numeric(height)
  color <- as.character(color)
  defines <- validate_defines(defines)
  magick_image_blank(width, height, color, pseudo_image, defines)
}

#' @export
#' @rdname editing
image_destroy <- function(image){
  assert_image(image)
  magick_image_destroy(image)
}

#' @export
#' @rdname editing
#' @param ... several images or lists of images to be combined
image_join <- function(...){
  x <- unlist(list(...))
  stopifnot(all(vapply(x, inherits, logical(1), "magick-image")))
  magick_image_join(x)
}

#' @export
#' @rdname editing
image_attributes <- function(image){
  assert_image(image)
  magick_image_properties(image)
}

#' @export
#' @rdname editing
#' @param artifact string with name of the artifact to extract, see the
#' [image_deskew] for an example.
image_get_artifact <- function(image, artifact = ""){
  assert_image(image)
  artifact <- as.character(artifact)
  magick_image_artifact(image, artifact)
}

#' @export
#' @rdname editing
demo_image <- function(path){
  image_read(system.file('images', path, package = 'magick'))
}

#' Image Editing
#'
#' Read, write and join or combine images. All image functions are vectorized, meaning
#' they operate either on a single frame or a series of frames (e.g. a collage, video,
#' or animation). Besides paths and URLs, [image_read()] supports commonly used bitmap
#' and raster object types.
#'
#' All standard base vector methods such as \link{[}, \link{[[}, [c()], [as.list()],
#' [as.raster()], [rev()], [length()], and [print()] can be used to work with magick
#' image objects. Use the standard \code{img[i]} syntax to extract a subset of the frames
#' from an image. The \code{img[[i]]} method is an alias for [image_data()] which extracts
#' a single frame as a raw bitmap matrix with pixel values.
#'
#' For reading svg or pdf it is recommended to use `image_read_svg()` and `image_read_pdf()`
#' if the [rsvg][rsvg::rsvg] and [pdftools][pdftools::pdf_render_page] R packages are available.
#' These functions provide more rendering options (including rendering of literal svg) and
#' better quality than built-in svg/pdf rendering delegates from imagemagick itself.
#'
#' X11 is required for `image_display()` which is only works on some platforms. A more
#' portable method is `image_browse()` which opens the image in a browser. RStudio has
#' an embedded viewer that does this automatically which is quite nice.
#'
#' Image objects are automatically released by the garbage collector when they are no longer
#' reachable. Because the GC only runs once in a while, you can also call `image_destroy()`
#' explicitly to release the memory immediately. This is usually only needed if you create
#' a lot of images in a short period of time, and you might run out of memory.
#'
#' @importFrom Rcpp sourceCpp
#' @useDynLib magick
#' @export
#' @family image
#' @rdname editing
#' @name editing
#' @inheritParams defines
#' @param path a file, url, or raster object or bitmap array
#' @param image magick image object returned by [image_read()] or [image_graph()]
#' @param density resolution to render pdf or svg
#' @param strip drop image comments and metadata
#' @param coalesce automatically [image_coalesce()] gif images
#' @examples
#' # Download image from the web
#' frink <- image_read("https://jeroen.github.io/images/frink.png")
#' worldcup_frink <- image_fill(frink, "orange", "+100+200", 20)
#' image_write(worldcup_frink, "output.png")
#'
#' # extract raw bitmap array
#' bitmap <- frink[[1]]
#'
#' # replace pixels with #FF69B4 ('hot pink') and convert back to image
#' bitmap[,50:100, 50:100] <- as.raw(c(0xff, 0x69, 0xb4, 0xff))
#' image_read(bitmap)
#'
#' # Plot to graphics device via legacy raster format
#' raster <- as.raster(frink)
#' par(ask=FALSE)
#' plot(raster)
#'
#' # Read bitmap arrays from other image packages
#' download.file("https://jeroen.github.io/images/example.webp", "example.webp", mode = 'wb')
#' if(require(webp)) image_read(webp::read_webp("example.webp"))
#' unlink(c("example.webp", "output.png"))
image_read <- function(path, density = NULL, depth = NULL, strip = FALSE, coalesce = TRUE, defines = NULL){
  if(is.numeric(density))
    density <- paste0(density, "x", density)
  density <- as.character(density)
  depth <- as.integer(depth)
  defines <- validate_defines(defines)
  image <- if(isS4(path) && methods::is(path, "Image")){
    convert_EBImage(path)
  } else if(inherits(path, "nativeRaster") || (is.matrix(path) && is.integer(path))){
    image_read_nativeraster(path)
  } else if(inherits(path, "cimg")) {
    image_read_cimg((path))
  } else if (grDevices::is.raster(path)) {
    image_read_raster2(path)
  } else if (is.matrix(path) && is.character(path)){
    image_read_raster2(grDevices::as.raster(path))
  } else if(is.array(path)){
    image_readbitmap(path)
  } else if(is.raw(path)) {
    magick_image_readbin(path, density, depth, strip, defines)
  } else if(is.character(path) && all(nchar(path))){
    path <- vapply(path, replace_url, character(1))
    # See https://github.com/ropensci/magick/issues/285
    path <- if(is_windows()){
      enc2utf8(path)
    } else {
      enc2native(path)
    }
    magick_image_readpath(path, density, depth, strip, defines)
  } else {
    stop("path must be URL, filename or raw vector")
  }
  if(is.character(path) && !isTRUE(magick_config()$rsvg)){
    if(any(grepl("\\.svg$", tolower(path))) || any(grepl("svg|mvg", tolower(image_info(image)$format)))){
      warning("ImageMagick was built without librsvg which causes poor qualty of SVG rendering.
For better results use image_read_svg() which uses the rsvg package.", call. = FALSE)
    }
  }
  if(isTRUE(coalesce) && length(image) > 1 && identical('GIF', toupper(image_info(image)$format[1]))){
    return(image_coalesce(image))
  }
  return(image)
}

#' @export
#' @rdname editing
#' @examples if(require(rsvg)){
#' tiger <- image_read_svg("http://jeroen.github.io/images/tiger.svg")
#' svgtxt <- '<?xml version="1.0" encoding="UTF-8"?>
#' <svg width="400" height="400" viewBox="0 0 400 400" fill="none">
#'  <circle fill="steelblue" cx="200" cy="200" r="100" />
#'  <circle fill="yellow" cx="200" cy="200" r="90" />
#' </svg>'
#' circles <- image_read_svg(svgtxt)
#' }
image_read_svg <- function(path, width = NULL, height = NULL){
  path <- vapply(path, replace_url, character(1))
  images <- lapply(path, function(x){
    if(is_svg(x)) x <- charToRaw(x)
    bitmap <- rsvg::rsvg_raw(x, width = width, height = height)
    magick_image_readbitmap_raw(bitmap)
  })
  image_join(images)
}

#' @export
#' @rdname editing
#' @param pages integer vector with page numbers. Defaults to all pages.
#' @param password user [password][pdftools::pdf_render_page] to open protected pdf files
#' @examples if(require(pdftools))
#' image_read_pdf(file.path(R.home('doc'), 'NEWS.pdf'), pages = 1, density = 100)
image_read_pdf <- function(path, pages = NULL, density = 300, password = ""){
  path <- replace_url(path)
  if(!length(pages))
    pages <- seq_len(pdftools::pdf_info(path, opw = password)$pages)
  images <- lapply(pages, function(page){
    bitmap <- pdftools::pdf_render_page(path, page = page, dpi = density, opw = password)
    magick_image_readbitmap_raw(bitmap)
  })
  out <- image_join(images)
  if(length(density))
    try(magick_attr_density(out, paste0(density, 'x', density)))
  out
}

#' @export
#' @rdname editing
#' @param fps how many images to capture per second of video. Set to
#' `NULL` to get all frames from the input video.
image_read_video <- function(path, fps = 1, format = 'png'){
  if(!requireNamespace('av', quietly = TRUE) || utils::packageVersion('av') < '0.3')
    stop("Please install or update the 'av' package")
  images <- av::av_video_images(path, fps = fps, format = format)
  on.exit(unlink(images))
  image_read(images)
}

image_readbitmap <- function(x){
  if(length(dim(x)) != 3)
    stop("Only 3D arrays can be converted to bitmaps")
  if(is.raw(x)){
    magick_image_readbitmap_raw(x)
  } else if(is.double(x)){
    magick_image_readbitmap_double(aperm(x))
  } else {
    stop("Unsupported bitmap array type")
  }
}

# output of dev.caputure(native = TRUE)
image_read_nativeraster <- function(x){
  stopifnot(is.matrix(x) && is.integer(x))
  magick_image_readbitmap_native(x)
}

# output of dev.caputure(native = FALSE) or as.raster()
image_read_raster1 <- function(x){
  stopifnot(is.matrix(x) && is.character(x))
  magick_image_readbitmap_raster1(t(x))
}

# output of as.raster()
image_read_raster2 <- function(x){
  stopifnot(is.matrix(x) && is.character(x))
  magick_image_readbitmap_raster2(x)
}

# This can probably be more efficient
image_read_cimg <- function(x){
  rast <- as.raster(x)
  if(is.list(rast)){
    image_join(lapply(rast, image_read))
  } else {
    image_read(rast)
  }
}

#EBImage BioConductor class
convert_EBImage <- function(x){
  if(length(dim(x@.Data)) == 2)
    dim(x@.Data) <- c(dim(x@.Data), 1L)
  img <- if(x@colormode == 2L){
    image_read(x@.Data)
  } else if(x@colormode == 0L){
    image_join(lapply(seq_len(dim(x@.Data)[3]), function(i){
      image_read(x@.Data[,,i,drop = FALSE])
    }))
  } else {
    stop("Unknown colormode in EBImage class")
  }
  image_flop(image_rotate(img, 90))
}

#' @export
#' @inheritParams effects
#' @rdname editing
#' @param flatten should image be flattened before writing? This also replaces
#' transparency with background color.
#' @param quality number between 0 and 100 for jpeg quality. Defaults to 75.
#' @param comment text string added to the image metadata for supported formats
#' @param compression a string with compression type from [compress_types]
image_write <- function(image, path = NULL, format = NULL, quality = NULL,
                        depth = NULL, density = NULL, comment = NULL, flatten = FALSE,
                        defines = NULL, compression = NULL){
  assert_image(image)
  if(!length(image))
    warning("Writing image with 0 frames")
  if(isTRUE(flatten))
    image <- image_flatten(image)
  format <- toupper(as.character(format))
  quality <- as.integer(quality)
  depth <- as.integer(depth)
  density <- as.character(density)
  comment <- as.character(comment)
  compression <- as.character(compression)
  if(length(defines)){
    image_set_defines(image, defines = defines)
    defines[seq_along(defines)] = NA_character_;
    on.exit(image_set_defines(image, defines = defines))
  }
  buf <- magick_image_write(image, format, quality, depth, density, comment, compression)
  if(is.character(path)){
    writeBin(buf, path)
    return(invisible(path))
  }
  return(buf)
}

#' @export
#' @rdname editing
#' @param format output format such as `"png"`, `"jpeg"`, `"gif"`, `"rgb"` or `"rgba"`.
#' @param depth color depth (either 8 or 16)
#' @param antialias enable anti-aliasing for text and strokes
#' @param type string with [imagetype](https://www.imagemagick.org/Magick++/Enumerations.html#ImageType)
#' value from [image_types][image_types] for example `grayscale` to convert into black/white
#' @param colorspace string with a [`colorspace`](https://www.imagemagick.org/Magick++/Enumerations.html#ColorspaceType)
#' from [colorspace_types][colorspace_types] for example `"gray"`, `"rgb"` or `"cmyk"`
#' @param matte set to `TRUE` or `FALSE` to enable or disable transparency
#' @param interlace string with [interlace](https://www.imagemagick.org/Magick++/Enumerations.html#InterlaceType)
image_convert <- function(image, format = NULL, type = NULL, colorspace = NULL,
                          depth = NULL, antialias = NULL, matte = NULL, interlace = NULL){
  assert_image(image)
  depth <- as.integer(depth)
  antialias <- as.logical(antialias)
  type <- as.character(type)
  colorspace <- as.character(colorspace)
  matte <- as.logical(matte)
  interlace <- as.character(interlace)
  if(length(depth) && is.na(match(depth, c(8, 16))))
    stop('depth must be 8 or 16 bit')
  magick_image_format(image, toupper(format), type, colorspace, depth, antialias, matte, interlace)
}

image_write_frame <- function(image, format = "rgba", i = 1){
  magick_image_write_frame(image, format = format, i = i)
}

#' @export
#' @rdname editing
#' @param channels string with image channel(s) for example `"rgb"`, `"rgba"`,
#' `"cmyk"`,`"gray"`, or `"ycbcr"`. Default is either `"gray"`, `"rgb"` or `"rgba"`
#' depending on the image
#' @param frame integer setting which frame to extract from the image
image_data <- function(image, channels = NULL, frame = 1){
  if(length(image) > 1 || frame > 1)
    image <- image[frame]
  if(!length(channels) || !nchar(channels)){
    info <- image_info(image)
    channels <- if(tolower(info$colorspace) == "gray"){
      "gray"
    } else if(isTRUE(info$matte)){
      "rgba"
    } else {
      "rgb"
    }
  }
  if(!grepl("a$", channels)) #output has no transparency channel
    image <- image_flatten(image)
  image_write_frame(image, format = channels)
}

#' @export
#' @rdname editing
#' @param tidy converts raster data to long form for use with [geom_raster][ggplot2::geom_raster].
#' If `FALSE` output is the same as `as.raster()`.
image_raster <- function(image, frame = 1, tidy = TRUE){
  raster <- as.raster(image[frame])
  if(!isTRUE(tidy))
    return(raster)
  data.frame(x = rep(seq_len(ncol(raster)), nrow(raster)),
             y = rep(seq_len(nrow(raster)), each = ncol(raster)),
             col = as.vector(raster),
             stringsAsFactors = FALSE)
}


#' @export
#' @param animate support animations in the X11 display
#' @rdname editing
image_display <- function(image, animate = TRUE){
  magick_image_display(image, animate)
}

#' @export
#' @param browser argument passed to [browseURL][utils::browseURL]
#' @rdname editing
image_browse <- function(image, browser = getOption("browser")){
  ext <- ifelse(length(image), tolower(image_info(image[1])$format), "gif")
  tmp <- tempfile(fileext = paste0(".", ext))
  image_write(image, path = tmp)
  utils::browseURL(tmp)
}

#' @export
#' @rdname editing
image_strip <- function(image){
  assert_image(image)
  magick_image_strip(image)
}


#' @export
#' @rdname editing
#' @inheritParams device
#' @inheritParams painting
#' @param pseudo_image string with [pseudo image](http://www.imagemagick.org/script/formats.php#pseudo)
#' specification for example `"radial-gradient:purple-yellow"`
#' @examples # create a solid canvas
#' image_blank(600, 400, "green")
#' image_blank(600, 400, pseudo_image = "radial-gradient:purple-yellow")
#' image_blank(200, 200, pseudo_image = "gradient:#3498db-#db3a34",
#'   defines = c('gradient:direction' = 'east'))
image_blank <- function(width, height, color = "none", pseudo_image = "", defines = NULL){
  width <- as.numeric(width)
  height <- as.numeric(height)
  color <- as.character(color)
  defines <- validate_defines(defines)
  magick_image_blank(width, height, color, pseudo_image, defines)
}

#' @export
#' @rdname editing
image_destroy <- function(image){
  assert_image(image)
  magick_image_destroy(image)
}

#' @export
#' @rdname editing
#' @param ... several images or lists of images to be combined
image_join <- function(...){
  x <- unlist(list(...))
  stopifnot(all(vapply(x, inherits, logical(1), "magick-image")))
  magick_image_join(x)
}

#' @export
#' @rdname editing
image_attributes <- function(image){
  assert_image(image)
  magick_image_properties(image)
}

#' @export
#' @rdname editing
#' @param artifact string with name of the artifact to extract, see the
#' [image_deskew] for an example.
image_get_artifact <- function(image, artifact = ""){
  assert_image(image)
  artifact <- as.character(artifact)
  magick_image_artifact(image, artifact)
}

#' @export
#' @rdname editing
demo_image <- function(path){
  image_read(system.file('images', path, package = 'magick'))
}

#' Image to ggplot
#'
#' Create a ggplot with axes set to pixel coordinates and plot the raster image
#' on it using [ggplot2::annotation_raster]. See examples for how to plot an image
#' onto an existing ggplot.
#'
#' @export
#' @inheritParams editing
#' @param interpolate passed to [ggplot2::annotation_raster]
#' @examples # Plot with base R
#' plot(logo)
#'
#' # Plot image with ggplot2
#' library(ggplot2)
#' myplot <- image_ggplot(logo)
#' myplot + ggtitle("Test plot")
#'
#' # Show that coordinates are reversed:
#' myplot + theme_classic()
#'
#' # Or add to plot as annotation
#' image <- image_fill(logo, 'none')
#' raster <- as.raster(image)
#' myplot <- qplot(mpg, wt, data = mtcars)
#' myplot + annotation_raster(raster, 25, 35, 3, 5)
#'
#' # Or overplot image using grid
#' library(grid)
#' qplot(speed, dist, data = cars, geom = c("point", "smooth"))
#' grid.raster(image)
image_ggplot <- function(image, interpolate = FALSE) {
  info <- image_info(image)
  ggplot2::ggplot(data.frame(x = 0, y = 0), ggplot2::aes_string('x', 'y')) +
    ggplot2::geom_blank() +
    ggplot2::theme_void() +
    ggplot2::coord_fixed(expand = FALSE, xlim = c(0, info$width), ylim = c(0, info$height)) +
    ggplot2::annotation_raster(image, 0, info$width, info$height, 0, interpolate = interpolate) +
    NULL
}

#' Write Video
#'
#' High quality video / gif exporter based on external packages [gifski][gifski::gifski]
#' and [av][av::av_encode_video].
#'
#' This requires an image with multiple frames. The GIF exporter accomplishes the same
#' thing as [image_animate][image_animate] but much faster and with better quality.
#'
#' @export
#' @rdname video
#' @name video
#' @family image
#' @inheritParams editing
#' @param path filename of the output gif or video. This is also the return value.
#' @param framerate frames per second, passed to [av_encode_video][av::av_encode_video]
#' @param ... additional parameters passed to [av_encode_video][av::av_encode_video] and
#' [gifski][gifski::gifski].
image_write_video <- function(image, path = NULL, framerate = 10, ...){
  png_files <- write_png_files(image)
  on.exit(unlink(png_files))
  av::av_encode_video(png_files, output = path, framerate = framerate, ...)
}

#' @param delay duration of each frame in seconds (inverse of framerate)
#' @rdname video
#' @export
image_write_gif <- function(image, path = NULL, delay = 1/10, ...){
  png_files <- write_png_files(image)
  on.exit(unlink(png_files))
  info <- image_info(image)
  width <- info$width[1]
  height <- info$height[1]
  gifski::gifski(png_files, gif_file = path, width = width,
                 height = height, delay = delay, ...)
}

write_png_files <- function(image){
  imgdir <- tempfile('tmppng')
  dir.create(imgdir)
  image <- image_convert(image, format = 'png')
  info <- image_info(image)
  width <- info$width[1]
  height <- info$height[1]
  if(length(unique(info$width)) > 1 || length(unique(info$height)) > 1){
    message(sprintf("Images are not the same size. Resizing to %dx%d...", width, height))
    image <- image_resize(image, sprintf('%dx%d!', width, height))
  }
  vapply(seq_along(image), function(i){
    tmppng <- file.path(imgdir, sprintf("tmpimg_%05d.png", i))
    image_write(image[i], path = tmppng, format = 'png')
  }, character(1))
}

#' Image FX
#'
#' Apply a custom an [fx expression](https://www.imagemagick.org/script/fx.php) to the image.
#'
#' There are two different interfaces. The [image_fx] function simply applies
#' the same fx to each frame in the input image. The [image_fx_sequence] function
#' on the other hand treats the entire input vector as a sequence, allowing you
#' to apply an expression with multiple input images. See examples.
#'
#' @export
#' @inheritParams thresholding
#' @rdname fx
#' @name fx
#' @family image
#' @param expression string with an [fx expression](https://www.imagemagick.org/script/fx.php)
#' @examples # Show image_fx() expression
#' img <- image_convert(logo, colorspace = "Gray")
#' gradient_x <- image_convolve(img, kernel = "Prewitt")
#' gradient_y <- image_convolve(img, kernel = "Prewitt:90")
#' gradient <- c(image_fx(gradient_x, expression = "p^2"),
#'                 image_fx(gradient_y, expression = "p^2"))
#' gradient <- image_flatten(gradient, operator = "Plus")
#' #gradient <- image_fx(gradient, expression = "sqrt(p)")
#' gradient
#'
#' \donttest{
#' image_fx(img, expression = "pow(p, 0.5)")
#' image_fx(img, expression = "rand()")
#' }
image_fx <- function(image, expression = "p", channel = NULL){
  assert_image(image)
  expression <- as.character(expression)
  channel <- as.character(channel)
  magick_image_fx(image, expression, channel)
}


#' @rdname fx
#' @export
#' @examples # Use multiple source images
#' \donttest{
#' input <- c(logo, image_flop(logo))
#' image_fx_sequence(input, "(u+v)/2")
#' }
image_fx_sequence <- function(image, expression = "p"){
  expression <- as.character(expression)
  magick_image_fx_sequence(image, expression)
}

# Build against imagemagick static website.
VERSION <- commandArgs(TRUE)
IM <- substr(VERSION, 1,1)
if(!file.exists(sprintf("../windows/imagemagick%s-%s/include/ImageMagick-%s/Magick++.h", IM, VERSION, IM))){
  if(getRversion() < "3.3.0") setInternet2()
  download.file(sprintf("https://github.com/rwinlib/imagemagick%s/archive/v%s.zip", IM, VERSION), "lib.zip", quiet = TRUE)
  dir.create("../windows", showWarnings = FALSE)
  unzip("lib.zip", exdir = "../windows")
  unlink("lib.zip")
}

if(!file.exists("base.o") && !file.exists("../.libs/imagemagick/include/ImageMagick-6/Magick++.h")){
  unlink("../.libs", recursive = TRUE)
  url <- if(grepl("aarch", R.version$platform)){
    "https://github.com/r-windows/bundles/releases/download/imagemagick-6.9.13.29/imagemagick-6.9.13.29-clang-aarch64.tar.xz"
  } else if(grepl("clang", Sys.getenv('R_COMPILED_BY'))){
    "https://github.com/r-windows/bundles/releases/download/imagemagick-6.9.13.29/imagemagick-6.9.13.29-clang-x86_64.tar.xz"
  } else if(getRversion() >= "4.3") {
    "https://github.com/r-windows/bundles/releases/download/imagemagick-6.9.13.29/imagemagick-6.9.13.29-ucrt-x86_64.tar.xz"
  } else {
    "https://github.com/rwinlib/imagemagick6/archive/v6.9.12-96.tar.gz"
  }
  options(timeout = 300)
  download.file(url, basename(url), quiet = TRUE)
  dir.create("../.libs", showWarnings = FALSE)
  untar(basename(url), exdir = "../.libs", tar = 'internal')
  unlink(basename(url))
  setwd("../.libs")
  file.rename(list.files(), 'imagemagick')
}

# Hack hack hack for old bundle
if(getRversion() >= "4.2.0" && getRversion() < "4.2.9"){
  file.copy('imagemagick/lib/libz.a', 'imagemagick/lib/libbrotlidec.a')
  file.copy('imagemagick/lib/libz.a', 'imagemagick/lib/libbrotlicommon.a')
}

#' Edge / Line Detection
#'
#' Best results are obtained by finding edges with [image_canny()] and
#' then performing Hough-line detection on the edge image.
#'
#' For Hough-line detection, the geometry format is `{W}x{H}+{threshold}`
#' defining the size and threshold of the filter used to find 'peaks' in
#' the intermediate search image. For canny edge detection the format is
#' `{radius}x{sigma}+{lower%}+{upper%}`. More details and examples are
#' available at the [imagemagick website](https://legacy.imagemagick.org/Usage/transform/).
#'
#' @export
#' @inheritParams editing
#' @rdname edges
#' @name edges
#' @family image
#' @param geometry geometry string, see details.
#' @param bg background color
#' @param radius edge size in pixels
#' @param format output format of the text, either `svg` or `mvg`
#' @param size size in points to draw the line
#' @examples if(magick_config()$version > "6.8.9"){
#' shape <- demo_image("shape_rectangle.gif")
#' rectangle <- image_canny(shape)
#' rectangle %>% image_hough_draw('5x5+20')
#' rectangle %>% image_hough_txt(format = 'svg') %>% cat()
#' }
image_edge <- function(image, radius = 1){
  assert_image(image)
  radius <- as.numeric(radius)
  magick_image_edge(image, radius)
}

#' @export
#' @rdname edges
image_canny <- function(image, geometry = '0x1+10%+30%'){
  assert_image(image)
  geometry <- as.character(geometry)
  magick_image_canny(image, geometry)
}

#' @export
#' @rdname edges
#' @param overlay composite the drawing atop the input image. Only for `bg = 'transparent'`.
image_hough_draw <- function(image, geometry = NULL, color = 'red',
                             bg = 'transparent', size = 3, overlay = FALSE){
  assert_image(image)
  geometry <- as.character(geometry)
  if(!length(geometry)){
    info <- image_info(image)
    w <- ceiling(info$width * 0.05)
    h <- ceiling(info$height * 0.05)
    tres <- ceiling(min(info$width, info$height) * 0.20)
    geometry <- sprintf('%dx%d+%d', w, h, tres)
  }
  color <- as.character(color)
  bg <- as.character(bg)
  size <- as.numeric(size)
  out <- magick_image_houghline(image, geometry, color, bg, size)
  if(isTRUE(overlay))
    out <- image_composite(image, out)
  out
}

#' @export
#' @rdname edges
image_hough_txt <- function(image, geometry = NULL, format = c("mvg", "svg")){
  format <- match.arg(format)
  out <- image_hough_draw(image, geometry = geometry)
  rawToChar(image_write(out, format = format))
}

#' Image Text OCR
#'
#' Extract text from an image using the [tesseract][tesseract::tesseract] package.
#'
#' To use this function you need to tesseract first:
#'
#' ```
#'   install.packages("tesseract")
#' ```
#'
#' Best results are obtained if you set the correct language in [tesseract][tesseract::tesseract].
#' To install additional languages see instructions in [tesseract_download()][tesseract::tesseract_download].
#'
#' @export
#' @family image
#' @name ocr
#' @rdname ocr
#' @inheritParams editing
#' @param language passed to [tesseract][tesseract::tesseract]. To install additional languages see
#' instructions in [tesseract_download()][tesseract::tesseract_download].
#' @param HOCR if `TRUE` return results as HOCR xml instead of plain text
#' @param ... additional parameters passed to [tesseract][tesseract::tesseract]
#' @examples
#' \donttest{
#' if(require("tesseract")){
#' img <- image_read("http://jeroen.github.io/images/testocr.png")
#' image_ocr(img)
#' image_ocr_data(img)
#' }
#' }
image_ocr <- function(image, language = "eng", HOCR = FALSE, ...){
  assert_image(image)
  tesseract::ocr(image, engine = tesseract::tesseract(language, ...), HOCR = HOCR)
}


#' @export
#' @rdname ocr
image_ocr_data <- function(image, language = "eng", ...){
  assert_image(image)
  tesseract::ocr_data(image, engine = tesseract::tesseract(language, ...))
}

#' Image Composite
#'
#' Similar to the ImageMagick `composite` utility: compose an image on top of another one using a
#' [CompositeOperator](https://www.imagemagick.org/Magick++/Enumerations.html#CompositeOperator).
#'
#' The `image_composite` function is vectorized over both image arguments: if the first image has
#' `n` frames and the second `m` frames, the output image will contain `n` * `m` frames.
#'
#' The [image_border] function creates a slightly larger solid color frame and then composes the
#' original frame on top. The [image_frame] function is similar but has an additional feature to
#' create a shadow effect on the border (which is really ugly).
#'
#' @export
#' @rdname composite
#' @name composite
#' @family image
#' @inheritParams editing
#' @inheritParams painting
#' @param offset string with either a [gravity_type][gravity_types] or a [geometry_point][geometry_point]
#' to set position of top image.
#' @param operator string with a
#' [composite operator](https://www.imagemagick.org/Magick++/Enumerations.html#CompositeOperator)
#' from [compose_types()][compose_types]
#' @param composite_image composition image
#' @param compose_args additional arguments needed for some composite operations
#' @examples # Compose images using one of many operators
#' imlogo <- image_scale(image_read("logo:"), "x275")
#' rlogo <- image_read("https://jeroen.github.io/images/Rlogo-old.png")
#'
#' # Standard is atop
#' image_composite(imlogo, rlogo)
#'
#' # Same as 'blend 50' in the command line
#' image_composite(imlogo, rlogo, operator = "blend", compose_args="50")
#'
#' # Offset can be geometry or gravity
#' image_composite(logo, rose, offset = "+100+100")
#' image_composite(logo, rose, gravity = "East")
image_composite <- function(image, composite_image, operator = "atop",
                            offset = "+0+0", gravity = "northwest", compose_args = ""){
  assert_image(image)
  assert_image(composite_image)
  compose_args <- as.character(compose_args)

  # avoid some overhead
  if(length(composite_image) == 1){
    magick_image_composite(image, composite_image, offset, gravity, operator, compose_args)
  } else {
    # vectorize over both 1st and 2nd argument
    image_apply(composite_image, function(x){
      magick_image_composite(image, x, offset, gravity, operator, compose_args)
    })
  }
}


#' @export
#' @rdname composite
#' @param geometry a [geometry string](https://www.imagemagick.org/Magick++/Geometry.html)
#' to set height and width of the border, e.g. `"10x8"`. In addition [image_frame] allows
#' for adding shadow by setting an offset e.g. `"20x10+7+2"`.
#' @examples
#'
#' # Add a border frame around the image
#' image_border(imlogo, "red", "10x10")
image_border <- function(image, color = "lightgray", geometry = "10x10", operator = "copy"){
  assert_image(image)
  color <- as.character(color)
  geometry <- as.character(geometry)
  operator <- as.character(operator)
  magick_image_border(image, color, geometry, operator)
}

#' @export
#' @rdname composite
#' @examples
#' image_frame(imlogo)
image_frame <- function(image, color = "lightgray", geometry = "25x25+6+6"){
  assert_image(image)
  color <- as.character(color)
  geometry <- as.character(geometry)
  magick_image_frame(image, color, geometry)
}

#' @export
#' @rdname composite
image_shadow_mask <- function(image, geometry = '50x10+30+30'){
  assert_image(image)
  geometry <- as.character(geometry)
  magick_image_shadow_mask(image, geometry)
}

#' @export
#' @rdname composite
#' @inheritParams edges
#' @examples image_shadow(imlogo)
image_shadow <- function(image, color = 'black', bg = 'white', geometry = '50x10+30+30',
                         operator = 'atop', offset = '+20+20'){
  assert_image(image)
  geometry <- as.character(geometry)
  input <- image_background(image, color, flatten = TRUE)
  shadow <- image_background(magick_image_shadow_mask(input, geometry), bg)
  image_join(lapply(seq_along(input), function(i){
    # Prevent double loop which results in n^2 output frames
    image_composite(shadow[i], image[i], operator = operator, offset = offset)
  }))
}

#' @export
#' @rdname composite
#' @inheritParams edges
#' @examples image_shade(imlogo)
#' @param azimuth position of light source
#' @param elevation position of light source
#' @param color Set to true to shade the red, green, and blue components of the image.
image_shade <- function(image, azimuth = 30, elevation = 30, color = FALSE){
  assert_image(image)
  azimuth <- as.numeric(azimuth)
  elevation <- as.numeric(elevation)
  color <- as.logical(color)
  magick_image_shade(image, azimuth, elevation, color)
}

#' Image Composite
#'
#' Similar to the ImageMagick `composite` utility: compose an image on top of another one using a
#' [CompositeOperator](https://www.imagemagick.org/Magick++/Enumerations.html#CompositeOperator).
#'
#' The `image_composite` function is vectorized over both image arguments: if the first image has
#' `n` frames and the second `m` frames, the output image will contain `n` * `m` frames.
#'
#' The [image_border] function creates a slightly larger solid color frame and then composes the
#' original frame on top. The [image_frame] function is similar but has an additional feature to
#' create a shadow effect on the border (which is really ugly).
#'
#' @export
#' @rdname composite
#' @name composite
#' @family image
#' @inheritParams editing
#' @inheritParams painting
#' @param offset string with either a [gravity_type][gravity_types] or a [geometry_point][geometry_point]
#' to set position of top image.
#' @param operator string with a
#' [composite operator](https://www.imagemagick.org/Magick++/Enumerations.html#CompositeOperator)
#' from [compose_types()][compose_types]
#' @param composite_image composition image
#' @param compose_args additional arguments needed for some composite operations
#' @examples # Compose images using one of many operators
#' imlogo <- image_scale(image_read("logo:"), "x275")
#' rlogo <- image_read("https://jeroen.github.io/images/Rlogo-old.png")
#'
#' # Standard is atop
#' image_composite(imlogo, rlogo)
#'
#' # Same as 'blend 50' in the command line
#' image_composite(imlogo, rlogo, operator = "blend", compose_args="50")
#'
#' # Offset can be geometry or gravity
#' image_composite(logo, rose, offset = "+100+100")
#' image_composite(logo, rose, gravity = "East")
image_composite <- function(image, composite_image, operator = "atop",
                            offset = "+0+0", gravity = "northwest", compose_args = ""){
  assert_image(image)
  assert_image(composite_image)
  compose_args <- as.character(compose_args)

  # avoid some overhead
  if(length(composite_image) == 1){
    magick_image_composite(image, composite_image, offset, gravity, operator, compose_args)
  } else {
    # vectorize over both 1st and 2nd argument
    image_apply(composite_image, function(x){
      magick_image_composite(image, x, offset, gravity, operator, compose_args)
    })
  }
}


#' @export
#' @rdname composite
#' @param geometry a [geometry string](https://www.imagemagick.org/Magick++/Geometry.html)
#' to set height and width of the border, e.g. `"10x8"`. In addition [image_frame] allows
#' for adding shadow by setting an offset e.g. `"20x10+7+2"`.
#' @examples
#'
#' # Add a border frame around the image
#' image_border(imlogo, "red", "10x10")
image_border <- function(image, color = "lightgray", geometry = "10x10", operator = "copy"){
  assert_image(image)
  color <- as.character(color)
  geometry <- as.character(geometry)
  operator <- as.character(operator)
  magick_image_border(image, color, geometry, operator)
}

#' @export
#' @rdname composite
#' @examples
#' image_frame(imlogo)
image_frame <- function(image, color = "lightgray", geometry = "25x25+6+6"){
  assert_image(image)
  color <- as.character(color)
  geometry <- as.character(geometry)
  magick_image_frame(image, color, geometry)
}

#' @export
#' @rdname composite
image_shadow_mask <- function(image, geometry = '50x10+30+30'){
  assert_image(image)
  geometry <- as.character(geometry)
  magick_image_shadow_mask(image, geometry)
}

#' @export
#' @rdname composite
#' @inheritParams edges
#' @examples image_shadow(imlogo)
image_shadow <- function(image, color = 'black', bg = 'none', geometry = '50x10+30+30',
                         operator = 'copy', offset = '+20+20'){
  assert_image(image)
  geometry <- as.character(geometry)
  input <- image_background(image, color, flatten = TRUE)
  shadow <- image_background(magick_image_shadow_mask(input, geometry), bg)
  image_join(lapply(seq_along(input), function(i){
    # Prevent double loop which results in n^2 output frames
    image_composite(shadow[i], image[i], operator = operator, offset = offset)
  }))
}

#' @export
#' @rdname composite
#' @inheritParams edges
#' @examples image_shade(imlogo)
#' @param azimuth position of light source
#' @param elevation position of light source
#' @param color Set to true to shade the red, green, and blue components of the image.
image_shade <- function(image, azimuth = 30, elevation = 30, color = FALSE){
  assert_image(image)
  azimuth <- as.numeric(azimuth)
  elevation <- as.numeric(elevation)
  color <- as.logical(color)
  magick_image_shade(image, azimuth, elevation, color)
}

.onAttach <- function(lib, pkg){
  # Test for features
  testfor <- c("cairo", "fontconfig", "freetype", "fftw", "ghostscript", "heic", "lcms", "pango", "raw", "rsvg", "webp", "x11")
  support <- unlist(magick_config()[testfor])
  has <- paste(names(which(support)), collapse = ", ")
  not <- paste(names(which(!support)), collapse = ", ")
  threads <- magick_threads(0)
  packageStartupMessage(sprintf("Linking to ImageMagick %s\nEnabled features: %s\nDisabled features: %s",
                                as.character(magick_config()$version), has, not))
  if(isTRUE(threads > 1))
    packageStartupMessage(sprintf("Using %d threads", threads))

  # For RStudio
  autoviewer_enable()
}

.onLoad <- function(lib, pkg){
  # Try to please cran...
  if(is_check() && isTRUE(magick_threads() > 2)){
    magick_threads(2)
  }

  # Set tempdir to R session
  set_magick_tempdir(tempdir())

  # Needed by older versions of IM:
  Sys.setenv(MAGICK_TMPDIR = tempdir())

  # Set the default viewer
  if(is.null(getOption('magick.viewer'))){
    fun <- function(x){}
    body(fun) <- parse(text = 'magick:::image_preview(x)')
    environment(fun) <- baseenv()
    options(magick.viewer = fun)
  }

  # NB: it is needed that autobrew fontconfig is newer than xQuarts
  # or at least new enough to read the fontconfig data from the xQuarts fontconfig
  if(autobrewed()){
    fontdir <- normalizePath(file.path(lib, pkg, "etc/fontconfig"), mustWork = FALSE)
    if(file.exists("/opt/X11/etc/X11/fontconfig")){
      Sys.setenv(FONTCONFIG_PATH = "/opt/X11/etc/X11/fontconfig")
    } else if(file.exists("/opt/X11/lib/X11/fontconfig")){
      Sys.setenv(FONTCONFIG_PATH = "/opt/X11/lib/X11/fontconfig")
    } else if(file.exists(fontdir)){
      Sys.setenv(FONTCONFIG_PATH = fontdir)
    }
  }
  register_s3_method("knitr", "knit_print", "magick-image")
}

register_s3_method <- function(pkg, generic, class, fun = NULL) {
  stopifnot(is.character(pkg), length(pkg) == 1)
  stopifnot(is.character(generic), length(generic) == 1)
  stopifnot(is.character(class), length(class) == 1)

  if (is.null(fun)) {
    fun <- get(paste0(generic, ".", class), envir = parent.frame())
  } else {
    stopifnot(is.function(fun))
  }

  if (pkg %in% loadedNamespaces()) {
    registerS3method(generic, class, fun, envir = asNamespace(pkg))
  }

  # Always register hook in case package is later unloaded & reloaded
  setHook(
    packageEvent(pkg, "onLoad"),
    function(...) {
      registerS3method(generic, class, fun, envir = asNamespace(pkg))
    }
  )
}

is_mac <- function(){
  grepl("darwin", R.Version()$platform)
}

is_check <- function(){
  grepl('magick.Rcheck', getwd(), fixed = TRUE)
}

---
title: "The magick package: Advanced Image-Processing in R"
date: "`r Sys.Date()`"
output:
  html_document:
    fig_caption: false
    toc: true
    toc_float:
      collapsed: false
      smooth_scroll: false
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{The magick package: Advanced Image-Processing in R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo = FALSE}
dev.off <- function(){
  invisible(grDevices::dev.off())
}
```

The [magick](https://cran.r-project.org/package=magick) package provide a modern and simple toolkit for image processing in R. It wraps the [ImageMagick STL](https://www.imagemagick.org/Magick++/STL.html) which is the most comprehensive open-source image processing library available today.

The ImageMagick library has an overwhelming amount of functionality. Magick exposes a decent subset of it, but it is impossible to document everything in detail. This article introduces some basic concepts and examples to get started.

## Installing `magick`

On Windows or macOS the package is most easily installed via CRAN.

```r
install.packages("magick")
```

The binary CRAN packages work out of the box and have most important features enabled.
Use `magick_config` to see which features and formats are supported by your version of ImageMagick.


```{r}
library(magick)
str(magick::magick_config())
```


### Build from source

On Linux you need to install the ImageMagick++ library: on Debian/Ubuntu this is called [libmagick++-dev](https://packages.debian.org/testing/libmagick++-dev):

```
sudo apt-get install libmagick++-dev
```

On Fedora or CentOS/RHEL we need [ImageMagick-c++-devel](https://src.fedoraproject.org/rpms/ImageMagick):

```
sudo yum install ImageMagick-c++-devel
```

To install from source on macOS you need `imagemagick@6` from homebrew.

```
brew install imagemagick@6
```

Unfortunately the current `imagemagick@6` configuration on homebrew disables a bunch of features, including librsvg and fontconfig. Therefore the quality of fonts and svg rendering might be suboptimal. The is not a problem for the CRAN binary package.

## Image IO

What makes magick so magical is that it automatically converts and renders all common image formats. ImageMagick supports dozens of formats and automatically detects the type. Use `magick::magick_config()` to list the formats that your version of ImageMagick supports.

### Read and write

Images can be read directly from a file path, URL, or raw vector with image data with `image_read`. The `image_info` function shows some meta data about the image, similar to the imagemagick `identify` command line utility.

```{r, eval = require(rsvg, quietly = TRUE)}
library(magick)
tiger <- image_read_svg('http://jeroen.github.io/images/tiger.svg', width = 350)
print(tiger)
```

We use `image_write` to export an image in any format to a file on disk, or in memory if `path = NULL`.

```r
# Render svg to png bitmap
image_write(tiger, path = "tiger.png", format = "png")
```

If `path` is a filename, `image_write` returns `path` on success such that the result can be piped into function taking a file path.

### Converting formats

Magick keeps the image in memory in its original format. Specify the `format` parameter `image_write` to convert to another format. You can also internally convert the image to another format earlier, before applying transformations. This can be useful if your original format is lossy.

```{r, eval = require(rsvg, quietly = TRUE)}
tiger_png <- image_convert(tiger, "png")
image_info(tiger_png)
```

Note that size is currently 0 because ImageMagick is lazy (in the good sense) and does not render until it has to.

### Preview

IDE's with a built-in web browser (such as RStudio) automatically display magick images in the viewer. This results in a neat interactive image editing environment.

<img id="rstudioimg" alt="rstudio">
<script>
//this is a hack to prevent pandoc 'self_contained' from embedding this image
//in future version of pandoc we can use the image and set 'data-external=1' instead
window.onload = function(){
  document.getElementById("rstudioimg").src = "https://jeroen.github.io/images/magick-rstudio.png";
}
</script>

Alternatively, on Linux you can use `image_display` to preview the image in an X11 window. Finally `image_browse` opens the image in your system's default application for a given type.

```r
# X11 only
image_display(tiger)

# System dependent
image_browse(tiger)
```

Another method is converting the image to a raster object and plot it on R's graphics display. However this is very slow and only useful in combination with other plotting functionality. See [#raster](#raster-images) below.

## Transformations

The best way to get a sense of available transformations is walk through the examples in the `?transformations` help page in RStudio. Below a few examples to get a sense of what is possible.

### Cut and edit

Several of the transformation functions take an `geometry` parameter which requires a special syntax of the form `AxB+C+D` where each element is optional. Some examples:

  - `image_crop(image, "100x150+50")`: *crop out `width:100px` and `height:150px` starting `+50px` from the left*
  - `image_scale(image, "200")`: *resize proportionally to width: `200px`*
  - `image_scale(image, "x200")`: *resize proportionally to height: `200px`*
  - `image_fill(image, "blue", "+100+200")`: *flood fill with blue starting at the point at `x:100, y:200`*
  - `image_border(frink, "red", "20x10")`: *adds a border of 20px left+right and 10px top+bottom*

The full syntax is specified in the [Magick::Geometry](http://www.imagemagick.org/Magick++/Geometry.html) documentation.

```{r}
# Example image
frink <- image_read("https://jeroen.github.io/images/frink.png")
```

```{r}

print(frink)

# Add 20px left/right and 10px top/bottom
image_border(image_background(frink, "hotpink"), "#000080", "20x10")

# Trim margins
image_trim(frink)

# Passport pica
image_crop(frink, "100x150+50")

# Resize
image_scale(frink, "300") # width: 300px
image_scale(frink, "x300") # height: 300px

# Rotate or mirror
image_rotate(frink, 45)
image_flip(frink)
image_flop(frink)

# Brightness, Saturation, Hue
image_modulate(frink, brightness = 80, saturation = 120, hue = 90)

# Paint the shirt orange
image_fill(frink, "orange", point = "+100+200", fuzz = 20)
```

With `image_fill` we can flood fill starting at pixel `point`. The `fuzz` parameter allows for the fill to cross for adjacent pixels with similarish colors. Its value must be between 0 and 256^2 specifying the max geometric distance between colors to be considered equal. Here we give professor frink an orange shirt for the World Cup.

### Filters and effects

ImageMagick also has a bunch of standard effects that are worth checking out.

```{r}
# Add randomness
image_blur(frink, 10, 5)
image_noise(frink)

# Silly filters
image_charcoal(frink)
image_oilpaint(frink)
image_negate(frink)
```

### Kernel convolution

The `image_convolve()` function applies a [kernel](https://en.wikipedia.org/wiki/Kernel_(image_processing)) over the image. Kernel convolution means that each pixel value is recalculated using the weighted neighborhood sum defined in the kernel matrix. For example lets look at this simple kernel:


```{r}
kern <- matrix(0, ncol = 3, nrow = 3)
kern[1, 2] <- 0.25
kern[2, c(1, 3)] <- 0.25
kern[3, 2] <- 0.25
kern
```

This kernel changes each pixel to the mean of its horizontal and vertical neighboring pixels, which results in a slight blurring effect in the right-hand image below:

```{r}
img <- image_resize(logo, "300x300")
img_blurred <- image_convolve(img, kern)
image_append(c(img, img_blurred))
```

Or use any of the [standard kernels](https://legacy.imagemagick.org/Usage/convolve/)

```{r}
img %>% image_convolve('Sobel') %>% image_negate()
img %>% image_convolve('DoG:0,0,2') %>% image_negate()
```

### Text annotation

Finally it can be useful to print some text on top of images:

```{r}
# Add some text
image_annotate(frink, "I like R!", size = 70, gravity = "southwest", color = "green")

# Customize text
image_annotate(frink, "CONFIDENTIAL", size = 30, color = "red", boxcolor = "pink",
  degrees = 60, location = "+50+100")

# Fonts may require ImageMagick has fontconfig
image_annotate(frink, "The quick brown fox", font = 'Times', size = 30)
```

Fonts that are supported on most platforms include `"sans"`, `"mono"`, `"serif"`, `"Times"`, `"Helvetica"`, `"Trebuchet"`, `"Georgia"`, `"Palatino"`or `"Comic Sans"`.

### Combining with pipes

Each of the image transformation functions returns a **modified copy** of the original image. It does not affect the original image.

```{r}
frink <- image_read("https://jeroen.github.io/images/frink.png")
frink2 <- image_scale(frink, "100")
image_info(frink)
image_info(frink2)
```

Hence to combine transformations you need to chain them:

```{r}
test <- image_rotate(frink, 90)
test <- image_background(test, "blue", flatten = TRUE)
test <- image_border(test, "red", "10x10")
test <- image_annotate(test, "This is how we combine transformations", color = "white", size = 30)
print(test)
```

Using `magrittr` pipe syntax makes it a bit more readable

```{r}
image_read("https://jeroen.github.io/images/frink.png") %>%
  image_rotate(270) %>%
  image_background("blue", flatten = TRUE) %>%
  image_border("red", "10x10") %>%
  image_annotate("The same thing with pipes", color = "white", size = 30)
```


## Image Vectors

The examples above concern single images. However all functions in magick have been vectorized to support working with layers, compositions or animation.

The standard base methods `[` `[[`, `c()` and `length()` are used to manipulate vectors of images which can then be treated as layers or frames. 

```{r}
# Download earth gif and make it a bit smaller for vignette
earth <- image_read("https://jeroen.github.io/images/earth.gif") %>%
  image_scale("200x") %>%
  image_quantize(128)

length(earth)
earth
head(image_info(earth))

rev(earth) %>% 
  image_flip() %>% 
  image_annotate("meanwhile in Australia", size = 20, color = "white")
```

### Layers

We can stack layers on top of each other as we would in Photoshop:

```{r}
bigdata <- image_read('https://jeroen.github.io/images/bigdata.jpg')
frink <- image_read("https://jeroen.github.io/images/frink.png")
logo <- image_read("https://jeroen.github.io/images/Rlogo.png")
img <- c(bigdata, logo, frink)
img <- image_scale(img, "300x300")
image_info(img)
```

A mosaic prints images on top of one another, expanding the output canvas such that that everything fits:

```{r}
image_mosaic(img)
```

Flattening combines the layers into a single image which has the size of the first image:

```{r}
image_flatten(img)
```

Flattening and mosaic allow for specifying alternative [composite operators](https://www.imagemagick.org/Magick++/Enumerations.html#CompositeOperator):

```{r}
image_flatten(img, 'Add')
image_flatten(img, 'Modulate')
image_flatten(img, 'Minus')
```

### Combining

Appending means simply putting the frames next to each other:

```{r}
image_append(image_scale(img, "x200"))
```

Use `stack = TRUE` to position them on top of each other:

```{r}
image_append(image_scale(img, "100"), stack = TRUE)
```

Composing allows for combining two images on a specific position:

```{r}
bigdatafrink <- image_scale(image_rotate(image_background(frink, "none"), 300), "x200")
image_composite(image_scale(bigdata, "x400"), bigdatafrink, offset = "+180+100")
```

### Pages

When reading a PDF document, each page becomes an element of the vector. Note that PDF gets rendered while reading so you need to specify the density immediately.

```{r, eval = require(pdftools, quietly = TRUE)}
manual <- image_read_pdf('https://cloud.r-project.org/web/packages/magick/magick.pdf', density = 72)
image_info(manual)
manual[1]
```

### Animation

Instead of treating vector elements as layers, we can also make them frames in an animation!

```{r}
image_animate(image_scale(img, "200x200"), fps = 1, dispose = "previous")
```

Morphing creates a sequence of `n` images that gradually morph one image into another. It makes animations 

```{r}
newlogo <- image_scale(image_read("https://jeroen.github.io/images/Rlogo.png"))
oldlogo <- image_scale(image_read("https://jeroen.github.io/images/Rlogo-old.png"))
image_resize(c(oldlogo, newlogo), '200x150!') %>%
  image_background('white') %>%
  image_morph() %>%
  image_animate(optimize = TRUE)
```


If you read in an existing GIF or Video file, each frame becomes a layer:

```{r}
# Foreground image
banana <- image_read("https://jeroen.github.io/images/banana.gif")
banana <- image_scale(banana, "150")
image_info(banana)
```

Manipulate the individual frames and put them back into an animation:

```{r}
# Background image
background <- image_background(image_scale(logo, "200"), "white", flatten = TRUE)

# Combine and flatten frames
frames <- image_composite(background, banana, offset = "+70+30")

# Turn frames into animation
animation <- image_animate(frames, fps = 10, optimize = TRUE)
print(animation)
```

Animations can be saved as GIF of MPEG files:

```r
image_write(animation, "Rlogo-banana.gif")
```

## Drawing and Graphics

A relatively recent addition to the package is a native R graphics device which produces a magick image object. This can either be used like a regular device for making plots, or alternatively to open a device which draws onto an existing image using pixel coordinates.

### Graphics device

The `image_graph()` function opens a new graphics device similar to e.g. `png()` or `x11()`. It returns an image object to which the plot(s) will be written. Each "page" in the plotting device will become a frame in the image object.

```{r}
# Produce image using graphics device
fig <- image_graph(width = 400, height = 400, res = 96)
ggplot2::qplot(mpg, wt, data = mtcars, colour = cyl)
dev.off()
```

We can easily post-process the figure using regular image operations.

```{r}
# Combine
out <- image_composite(fig, frink, offset = "+70+30")
print(out)
```

### Drawing device

Another way to use the graphics device is to draw on top of an exiting image using pixel coordinates.

```{r}
# Or paint over an existing image
img <- image_draw(frink)
rect(20, 20, 200, 100, border = "red", lty = "dashed", lwd = 5)
abline(h = 300, col = 'blue', lwd = '10', lty = "dotted")
text(30, 250, "Hoiven-Glaven", family = "monospace", cex = 4, srt = 90)
palette(rainbow(11, end = 0.9))
symbols(rep(200, 11), seq(0, 400, 40), circles = runif(11, 5, 35),
  bg = 1:11, inches = FALSE, add = TRUE)
dev.off()
```

```{r}
print(img)
```

By default `image_draw()` sets all margins to 0 and uses graphics coordinates to match image size in pixels (width x height) where (0,0) is the top left corner. Note that this means the y axis increases from top to bottom which is the opposite of typical graphics coordinates. You can override all this by passing custom `xlim`, `ylim` or `mar` values to `image_draw`.

### Animated Graphics

The graphics device supports multiple frames which makes it easy to create animated graphics. The code below shows how you would implement the example from the very cool [gganimate](https://gganimate.com/) package using the magick graphics device.


```{r}
library(gapminder)
library(ggplot2)
img <- image_graph(600, 340, res = 96)
datalist <- split(gapminder, gapminder$year)
out <- lapply(datalist, function(data){
  p <- ggplot(data, aes(gdpPercap, lifeExp, size = pop, color = continent)) +
    scale_size("population", limits = range(gapminder$pop)) + geom_point() + ylim(20, 90) + 
    scale_x_log10(limits = range(gapminder$gdpPercap)) + ggtitle(data$year) + theme_classic()
  print(p)
})
dev.off()
animation <- image_animate(img, fps = 2, optimize = TRUE)
print(animation)
```

To write it to a file you would simply do:

```r
image_write(animation, "gapminder.gif")
```

## Raster Images

Magick images can also be converted to raster objects for use with R's graphics device. Thereby we can combine it with other graphics tools. However do note that R's graphics device is very slow and has a very different coordinate system which reduces the quality of the image.

### Base R rasters

Base R has an `as.raster` format which converts the image to a vector of strings. The paper [Raster Images in R Graphics](https://journal.r-project.org/archive/2011-1/RJournal_2011-1_Murrell.pdf) by Paul Murrell gives a nice overview.

```{r}
plot(as.raster(frink))
```

```{r, fig.width=7, fig.height=5}
# Print over another graphic
plot(cars)
rasterImage(frink, 21, 0, 25, 80)
```

### The `grid` package

The `grid` package makes it easier to overlay a raster on the graphics device without having to adjust for the x/y coordinates of the plot.

```{r, fig.width=5, fig.height=3}
library(ggplot2)
library(grid)
qplot(speed, dist, data = cars, geom = c("point", "smooth"))
grid.raster(frink)
```

## OCR text extraction

A recent addition to the package is to extract text from images using OCR. This requires the tesseract package:

```{r eval=FALSE}
install.packages("tesseract")
```

```{r, eval = require(tesseract, quietly = TRUE)}
img <- image_read("http://jeroen.github.io/images/testocr.png")
print(img)

# Extract text
cat(image_ocr(img))
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/effects.R
\name{effects}
\alias{effects}
\alias{image_despeckle}
\alias{image_reducenoise}
\alias{image_noise}
\alias{image_blur}
\alias{image_motion_blur}
\alias{image_charcoal}
\alias{image_oilpaint}
\alias{image_emboss}
\alias{image_implode}
\alias{image_negate}
\title{Image Effects}
\usage{
image_despeckle(image, times = 1L)

image_reducenoise(image, radius = 1L)

image_noise(image, noisetype = "gaussian")

image_blur(image, radius = 1, sigma = 0.5)

image_motion_blur(image, radius = 1, sigma = 0.5, angle = 0)

image_charcoal(image, radius = 1, sigma = 0.5)

image_oilpaint(image, radius = 1)

image_emboss(image, radius = 1, sigma = 0.5)

image_implode(image, factor = 0.5)

image_negate(image)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{times}{number of times to repeat the despeckle operation}

\item{radius}{radius, in pixels, for various transformations}

\item{noisetype}{string with a
\href{https://www.imagemagick.org/Magick++/Enumerations.html#NoiseType}{noisetype} value
from \link{noise_types}.}

\item{sigma}{the standard deviation of the Laplacian, in pixels.}

\item{angle}{angle, in degrees, for various transformations}

\item{factor}{image implode factor (special effect)}
}
\description{
High level effects applied to an entire image.
These are mostly just for fun.
}
\examples{
logo <- image_read("logo:")
image_despeckle(logo)
image_reducenoise(logo)
image_noise(logo)
image_blur(logo, 10, 10)
image_motion_blur(logo, 10, 10, 45)
image_charcoal(logo)
image_oilpaint(logo, radius = 3)
image_emboss(logo)
image_implode(logo)
image_negate(logo)
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fx.R
\name{fx}
\alias{fx}
\alias{image_fx}
\alias{image_fx_sequence}
\title{Image FX}
\usage{
image_fx(image, expression = "p", channel = NULL)

image_fx_sequence(image, expression = "p")
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{expression}{string with an \href{https://www.imagemagick.org/script/fx.php}{fx expression}}

\item{channel}{a value of \code{\link[=channel_types]{channel_types()}} specifying which channel(s) to set}
}
\description{
Apply a custom an \href{https://www.imagemagick.org/script/fx.php}{fx expression} to the image.
}
\details{
There are two different interfaces. The \link{image_fx} function simply applies
the same fx to each frame in the input image. The \link{image_fx_sequence} function
on the other hand treats the entire input vector as a sequence, allowing you
to apply an expression with multiple input images. See examples.
}
\examples{
# Show image_fx() expression
img <- image_convert(logo, colorspace = "Gray")
gradient_x <- image_convolve(img, kernel = "Prewitt")
gradient_y <- image_convolve(img, kernel = "Prewitt:90")
gradient <- c(image_fx(gradient_x, expression = "p^2"),
                image_fx(gradient_y, expression = "p^2"))
gradient <- image_flatten(gradient, operator = "Plus")
#gradient <- image_fx(gradient, expression = "sqrt(p)")
gradient

\donttest{
image_fx(img, expression = "pow(p, 0.5)")
image_fx(img, expression = "rand()")
}
# Use multiple source images
\donttest{
input <- c(logo, image_flop(logo))
image_fx_sequence(input, "(u+v)/2")
}
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animation.R
\name{animation}
\alias{animation}
\alias{image_animate}
\alias{image_coalesce}
\alias{image_morph}
\alias{image_mosaic}
\alias{image_flatten}
\alias{image_average}
\alias{image_append}
\alias{image_apply}
\alias{image_montage}
\title{Image Frames and Animation}
\usage{
image_animate(
  image,
  fps = 10,
  delay = NULL,
  loop = 0,
  dispose = c("background", "previous", "none"),
  optimize = FALSE
)

image_coalesce(image)

image_morph(image, frames = 8)

image_mosaic(image, operator = NULL)

image_flatten(image, operator = NULL)

image_average(image)

image_append(image, stack = FALSE)

image_apply(image, FUN, ...)

image_montage(
  image,
  geometry = NULL,
  tile = NULL,
  gravity = "Center",
  bg = "white",
  shadow = FALSE
)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{fps}{frames per second. Ignored if \code{delay} is not \code{NULL}.}

\item{delay}{delay after each frame, in 1/100 seconds.
Must be length 1, or number of frames. If specified, then \code{fps} is ignored.}

\item{loop}{how many times to repeat the animation. Default is infinite.}

\item{dispose}{a frame \href{https://legacy.imagemagick.org/Usage/anim_basics/#dispose}{disposal method}
from \link[=dispose_types]{dispose_types()}}

\item{optimize}{optimize the \code{gif} animation by storing only the differences
between frames. Input images must be exactly the same size.}

\item{frames}{number of frames to use in output animation}

\item{operator}{string with a
\href{https://www.imagemagick.org/Magick++/Enumerations.html#CompositeOperator}{composite operator}
from \link[=compose_types]{compose_types()}}

\item{stack}{place images top-to-bottom (TRUE) or left-to-right (FALSE)}

\item{FUN}{a function to be called on each frame in the image}

\item{...}{additional parameters for \code{FUN}}

\item{geometry}{a \link{geometry} string that defines the size the individual
thumbnail images, and the spacing between them.}

\item{tile}{a \link{geometry} string for example "4x5 with limits on how the
tiled images are to be laid out on the final result.}

\item{gravity}{a gravity direction, if the image is smaller than the frame, where
in the frame is the image to be placed.}

\item{bg}{a background color string}

\item{shadow}{enable shadows between images}
}
\description{
Operations to manipulate or combine multiple frames of an image. Details below.
}
\details{
For details see \href{https://www.imagemagick.org/Magick++/STL.html}{Magick++ STL}
documentation. Short descriptions:
\itemize{
\item \link{image_animate} coalesces frames by playing the sequence and converting to \code{gif} format.
\item \link{image_morph} expands number of frames by interpolating intermediate frames to blend
into each other when played as an animation.
\item \link{image_mosaic} inlays images to form a single coherent picture.
\item \link{image_montage} creates a composite image by combining frames.
\item \link{image_flatten} merges frames as layers into a single frame using a given operator.
\item \link{image_average} averages frames into single frame.
\item \link{image_append} stack images left-to-right (default) or top-to-bottom.
\item \link{image_apply} applies a function to each frame
}

The \link{image_apply} function calls an image function to each frame and joins
results back into a single image. Because most operations are already vectorized
this is often not needed. Note that \code{FUN()} should return an image. To apply other
kinds of functions to image frames simply use \link{lapply}, \link{vapply}, etc.
}
\examples{
# Combine images
logo <- image_read("https://jeroen.github.io/images/Rlogo.png")
oldlogo <- image_read("https://jeroen.github.io/images/Rlogo-old.png")

# Create morphing animation
both <- image_scale(c(oldlogo, logo), "400")
image_average(image_crop(both))
image_animate(image_morph(both, 10))

# Create thumbnails from GIF
banana <- image_read("https://jeroen.github.io/images/banana.gif")
length(banana)
image_average(banana)
image_flatten(banana)
image_append(banana)
image_append(banana, stack = TRUE)

# Append images together
wizard <- image_read("wizard:")
image_append(image_scale(c(image_append(banana[c(1,3)], stack = TRUE), wizard)))

image_composite(banana, image_scale(logo, "300"))

# Break down and combine frames
front <- image_scale(banana, "300")
background <- image_background(image_scale(logo, "400"), 'white')
frames <- image_apply(front, function(x){image_composite(background, x, offset = "+70+30")})
image_animate(frames, fps = 10)
# Simple 4x3 montage
input <- rep(logo, 12)
image_montage(input, geometry = 'x100+10+10', tile = '4x3', bg = 'pink', shadow = TRUE)

# With varying frame size
input <- c(wizard, wizard, logo, logo)
image_montage(input, tile = '2x2', bg = 'pink', gravity = 'southwest')
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coder.R
\name{coder_info}
\alias{coder_info}
\alias{magick_config}
\alias{magick_set_seed}
\title{Magick Configuration}
\usage{
coder_info(format)

magick_config()

magick_set_seed(seed)
}
\arguments{
\item{format}{image format such as \code{png}, \code{tiff} or \code{pdf}.}

\item{seed}{integer with seed value to use}
}
\description{
ImageMagick can be configured to support various additional tool and formats
via external libraries. These functions show which features ImageMagick supports
on your system.
}
\details{
Note that \code{coder_info} raises an error for unsupported formats.
}
\examples{
coder_info("png")
coder_info("jpg")
coder_info("pdf")
coder_info("tiff")
coder_info("gif")
# Reproduce random image
magick_set_seed(123)
image_blank(200,200, pseudo_image = "plasma:fractal")
}
\references{
\url{https://www.imagemagick.org/Magick++/CoderInfo.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ocr.R
\name{ocr}
\alias{ocr}
\alias{image_ocr}
\alias{image_ocr_data}
\title{Image Text OCR}
\usage{
image_ocr(image, language = "eng", HOCR = FALSE, ...)

image_ocr_data(image, language = "eng", ...)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{language}{passed to \link[tesseract:tesseract]{tesseract}. To install additional languages see
instructions in \link[tesseract:tessdata]{tesseract_download()}.}

\item{HOCR}{if \code{TRUE} return results as HOCR xml instead of plain text}

\item{...}{additional parameters passed to \link[tesseract:tesseract]{tesseract}}
}
\description{
Extract text from an image using the \link[tesseract:tesseract]{tesseract} package.
}
\details{
To use this function you need to tesseract first:

\if{html}{\out{<div class="sourceCode">}}\preformatted{  install.packages("tesseract")
}\if{html}{\out{</div>}}

Best results are obtained if you set the correct language in \link[tesseract:tesseract]{tesseract}.
To install additional languages see instructions in \link[tesseract:tessdata]{tesseract_download()}.
}
\examples{
\donttest{
if(require("tesseract")){
img <- image_read("http://jeroen.github.io/images/testocr.png")
image_ocr(img)
image_ocr_data(img)
}
}
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/composite.R
\name{composite}
\alias{composite}
\alias{image_composite}
\alias{image_border}
\alias{image_frame}
\alias{image_shadow_mask}
\alias{image_shadow}
\alias{image_shade}
\title{Image Composite}
\usage{
image_composite(
  image,
  composite_image,
  operator = "atop",
  offset = "+0+0",
  gravity = "northwest",
  compose_args = ""
)

image_border(image, color = "lightgray", geometry = "10x10", operator = "copy")

image_frame(image, color = "lightgray", geometry = "25x25+6+6")

image_shadow_mask(image, geometry = "50x10+30+30")

image_shadow(
  image,
  color = "black",
  bg = "none",
  geometry = "50x10+30+30",
  operator = "copy",
  offset = "+20+20"
)

image_shade(image, azimuth = 30, elevation = 30, color = FALSE)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{composite_image}{composition image}

\item{operator}{string with a
\href{https://www.imagemagick.org/Magick++/Enumerations.html#CompositeOperator}{composite operator}
from \link[=compose_types]{compose_types()}}

\item{offset}{string with either a \link[=gravity_types]{gravity_type} or a \link{geometry_point}
to set position of top image.}

\item{gravity}{string with
\href{https://www.imagemagick.org/Magick++/Enumerations.html#GravityType}{gravity}
value from \link{gravity_types}.}

\item{compose_args}{additional arguments needed for some composite operations}

\item{color}{Set to true to shade the red, green, and blue components of the image.}

\item{geometry}{a \href{https://www.imagemagick.org/Magick++/Geometry.html}{geometry string}
to set height and width of the border, e.g. \code{"10x8"}. In addition \link{image_frame} allows
for adding shadow by setting an offset e.g. \code{"20x10+7+2"}.}

\item{bg}{background color}

\item{azimuth}{position of light source}

\item{elevation}{position of light source}
}
\description{
Similar to the ImageMagick \code{composite} utility: compose an image on top of another one using a
\href{https://www.imagemagick.org/Magick++/Enumerations.html#CompositeOperator}{CompositeOperator}.
}
\details{
The \code{image_composite} function is vectorized over both image arguments: if the first image has
\code{n} frames and the second \code{m} frames, the output image will contain \code{n} * \code{m} frames.

The \link{image_border} function creates a slightly larger solid color frame and then composes the
original frame on top. The \link{image_frame} function is similar but has an additional feature to
create a shadow effect on the border (which is really ugly).
}
\examples{
# Compose images using one of many operators
imlogo <- image_scale(image_read("logo:"), "x275")
rlogo <- image_read("https://jeroen.github.io/images/Rlogo-old.png")

# Standard is atop
image_composite(imlogo, rlogo)

# Same as 'blend 50' in the command line
image_composite(imlogo, rlogo, operator = "blend", compose_args="50")

# Offset can be geometry or gravity
image_composite(logo, rose, offset = "+100+100")
image_composite(logo, rose, gravity = "East")

# Add a border frame around the image
image_border(imlogo, "red", "10x10")
image_frame(imlogo)
image_shadow(imlogo)
image_shade(imlogo)
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/composite.R
\name{composite}
\alias{composite}
\alias{image_composite}
\alias{image_border}
\alias{image_frame}
\alias{image_shadow_mask}
\alias{image_shadow}
\alias{image_shade}
\title{Image Composite}
\usage{
image_composite(
  image,
  composite_image,
  operator = "atop",
  offset = "+0+0",
  gravity = "northwest",
  compose_args = ""
)

image_border(image, color = "lightgray", geometry = "10x10", operator = "copy")

image_frame(image, color = "lightgray", geometry = "25x25+6+6")

image_shadow_mask(image, geometry = "50x10+30+30")

image_shadow(
  image,
  color = "black",
  bg = "white",
  geometry = "50x10+30+30",
  operator = "atop",
  offset = "+20+20"
)

image_shade(image, azimuth = 30, elevation = 30, color = FALSE)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{composite_image}{composition image}

\item{operator}{string with a
\href{https://www.imagemagick.org/Magick++/Enumerations.html#CompositeOperator}{composite operator}
from \link[=compose_types]{compose_types()}}

\item{offset}{string with either a \link[=gravity_types]{gravity_type} or a \link{geometry_point}
to set position of top image.}

\item{gravity}{string with
\href{https://www.imagemagick.org/Magick++/Enumerations.html#GravityType}{gravity}
value from \link{gravity_types}.}

\item{compose_args}{additional arguments needed for some composite operations}

\item{color}{Set to true to shade the red, green, and blue components of the image.}

\item{geometry}{a \href{https://www.imagemagick.org/Magick++/Geometry.html}{geometry string}
to set height and width of the border, e.g. \code{"10x8"}. In addition \link{image_frame} allows
for adding shadow by setting an offset e.g. \code{"20x10+7+2"}.}

\item{bg}{background color}

\item{azimuth}{position of light source}

\item{elevation}{position of light source}
}
\description{
Similar to the ImageMagick \code{composite} utility: compose an image on top of another one using a
\href{https://www.imagemagick.org/Magick++/Enumerations.html#CompositeOperator}{CompositeOperator}.
}
\details{
The \code{image_composite} function is vectorized over both image arguments: if the first image has
\code{n} frames and the second \code{m} frames, the output image will contain \code{n} * \code{m} frames.

The \link{image_border} function creates a slightly larger solid color frame and then composes the
original frame on top. The \link{image_frame} function is similar but has an additional feature to
create a shadow effect on the border (which is really ugly).
}
\examples{
# Compose images using one of many operators
imlogo <- image_scale(image_read("logo:"), "x275")
rlogo <- image_read("https://jeroen.github.io/images/Rlogo-old.png")

# Standard is atop
image_composite(imlogo, rlogo)

# Same as 'blend 50' in the command line
image_composite(imlogo, rlogo, operator = "blend", compose_args="50")

# Offset can be geometry or gravity
image_composite(logo, rose, offset = "+100+100")
image_composite(logo, rose, gravity = "East")

# Add a border frame around the image
image_border(imlogo, "red", "10x10")
image_frame(imlogo)
image_shadow(imlogo)
image_shade(imlogo)
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis.R
\name{analysis}
\alias{analysis}
\alias{image_compare}
\alias{image_compare_dist}
\alias{image_fft}
\title{Image Analysis}
\usage{
image_compare(image, reference_image, metric = "", fuzz = 0)

image_compare_dist(image, reference_image, metric = "", fuzz = 0)

image_fft(image)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{reference_image}{another image to compare to}

\item{metric}{string with a \href{http://www.imagemagick.org/script/command-line-options.php#metric}{metric}
from \link[=metric_types]{metric_types()} such as \code{"AE"} or \code{"phash"}}

\item{fuzz}{relative color distance (value between 0 and 100) to be considered similar
in the filling algorithm}
}
\description{
Functions for image calculations and analysis. This part of the package needs more work.
}
\details{
For details see \href{https://www.imagemagick.org/Magick++/Image++.html}{Image++}
documentation. Short descriptions:
\itemize{
\item \link{image_compare} calculates a metric by comparing image with a reference image.
\item \link{image_fft} returns Discrete Fourier Transform (DFT) of the image as a
magnitude / phase image pair. I wish I knew what this means.
}

Here \code{image_compare()} is vectorized over the first argument and returns the diff image
with the calculated distortion value as an attribute.
}
\examples{
out1 <- image_blur(logo, 3)
out2 <- image_oilpaint(logo, 3)
input <- c(logo, out1, out2, logo)
if(magick_config()$version >= "6.8.7"){
  diff_img <- image_compare(input, logo, metric = "AE")
  attributes(diff_img)
}
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2.R
\name{image_ggplot}
\alias{image_ggplot}
\title{Image to ggplot}
\usage{
image_ggplot(image, interpolate = FALSE)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{interpolate}{passed to \link[ggplot2:annotation_raster]{ggplot2::annotation_raster}}
}
\description{
Create a ggplot with axes set to pixel coordinates and plot the raster image
on it using \link[ggplot2:annotation_raster]{ggplot2::annotation_raster}. See examples for how to plot an image
onto an existing ggplot.
}
\examples{
# Plot with base R
plot(logo)

# Plot image with ggplot2
library(ggplot2)
myplot <- image_ggplot(logo)
myplot + ggtitle("Test plot")

# Show that coordinates are reversed:
myplot + theme_classic()

# Or add to plot as annotation
image <- image_fill(logo, 'none')
raster <- as.raster(image)
myplot <- qplot(mpg, wt, data = mtcars)
myplot + annotation_raster(raster, 25, 35, 3, 5)

# Or overplot image using grid
library(grid)
qplot(speed, dist, data = cars, geom = c("point", "smooth"))
grid.raster(image)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform.R
\name{transform}
\alias{transform}
\alias{image_trim}
\alias{image_chop}
\alias{image_rotate}
\alias{image_resize}
\alias{image_scale}
\alias{image_sample}
\alias{image_crop}
\alias{image_extent}
\alias{image_flip}
\alias{image_flop}
\alias{image_deskew}
\alias{image_deskew_angle}
\alias{image_page}
\alias{image_repage}
\alias{image_orient}
\alias{image_shear}
\alias{image_distort}
\title{Image Transform}
\usage{
image_trim(image, fuzz = 0)

image_chop(image, geometry)

image_rotate(image, degrees)

image_resize(image, geometry = NULL, filter = NULL)

image_scale(image, geometry = NULL)

image_sample(image, geometry = NULL)

image_crop(image, geometry = NULL, gravity = NULL, repage = TRUE)

image_extent(image, geometry, gravity = "center", color = "none")

image_flip(image)

image_flop(image)

image_deskew(image, threshold = 40)

image_deskew_angle(image, threshold = 40)

image_page(image, pagesize = NULL, density = NULL)

image_repage(image)

image_orient(image, orientation = NULL)

image_shear(image, geometry = "10x10", color = "none")

image_distort(image, distortion = "perspective", coordinates, bestfit = FALSE)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{fuzz}{relative color distance (value between 0 and 100) to be considered similar
in the filling algorithm}

\item{geometry}{a \link{geometry} string specifying area (for cropping) or size (for resizing).}

\item{degrees}{value between 0 and 360 for how many degrees to rotate}

\item{filter}{string with \href{https://www.imagemagick.org/Magick++/Enumerations.html#FilterTypes}{filter}
type from: \link{filter_types}}

\item{gravity}{string with
\href{https://www.imagemagick.org/Magick++/Enumerations.html#GravityType}{gravity}
value from \link{gravity_types}.}

\item{repage}{resize the canvas to the cropped area}

\item{color}{a valid \href{https://www.imagemagick.org/Magick++/Color.html}{color string} such as
\code{"navyblue"} or \code{"#000080"}. Use \code{"none"} for transparency.}

\item{threshold}{straightens an image. A threshold of 40 works for most images.}

\item{pagesize}{geometry string with preferred size and location of an image canvas}

\item{density}{geometry string with vertical and horizontal resolution in pixels of
the image. Specifies an image density when decoding a Postscript or PDF.}

\item{orientation}{string to set image orientation one of the \link{orientation_types}.
If \code{NULL} it applies auto-orientation which tries to infer the correct orientation
from the Exif data.}

\item{distortion}{string to set image orientation one of the \link{distort_types}.}

\item{coordinates}{numeric vector (typically of length 12) with distortion coordinates}

\item{bestfit}{if set to \code{TRUE} the size of the output image can be different from input}
}
\description{
Basic transformations like rotate, resize, crop and flip. The \link{geometry} syntax
is used to specify sizes and areas.
}
\details{
For details see \href{https://www.imagemagick.org/Magick++/STL.html}{Magick++ STL}
documentation. Short descriptions:
\itemize{
\item \link{image_trim} removes edges that are the background color from the image.
\item \link{image_chop} removes vertical or horizontal subregion of image.
\item \link{image_crop} cuts out a subregion of original image
\item \link{image_rotate} rotates and increases size of canvas to fit rotated image.
\item \link{image_deskew} auto rotate to correct skewed images
\item \link{image_resize} resizes using custom \href{https://www.imagemagick.org/Magick++/Enumerations.html#FilterTypes}{filterType}
\item \link{image_scale} and \link{image_sample} resize using simple ratio and pixel sampling algorithm.
\item \link{image_flip} and \link{image_flop} invert image vertically and horizontally
}

The most powerful resize function is \link{image_resize} which allows for setting
a custom resize filter. Output of \link{image_scale} is similar to \code{image_resize(img, filter = "point")}.

For resize operations it holds that if no \code{geometry} is specified, all frames
are rescaled to match the top frame.
}
\examples{
logo <- image_read("logo:")
logo <- image_scale(logo, "400")
image_trim(logo)
image_chop(logo, "100x20")
image_rotate(logo, 45)
# Small image
rose <- image_convert(image_read("rose:"), "png")

# Resize to 400 width or height:
image_resize(rose, "400x")
image_resize(rose, "x400")

# Resize keeping ratio
image_resize(rose, "400x400")

# Resize, force size losing ratio
image_resize(rose, "400x400!")

# Different filters
image_resize(rose, "400x", filter = "Triangle")
image_resize(rose, "400x", filter = "Point")
# simple pixel resize
image_scale(rose, "400x")
image_sample(rose, "400x")
image_crop(logo, "400x400+200+200")
image_extent(rose, '200x200', color = 'pink')
image_flip(logo)
image_flop(logo)
skewed <- image_rotate(logo, 5)
deskewed <- image_deskew(skewed)
attr(deskewed, 'angle')
if(magick_config()$version > "6.8.6")
  image_orient(logo)
image_shear(logo, "10x10")
building <- demo_image('building.jpg')
image_distort(building, 'perspective', c(7,40,4,30,4,124,4,123,85,122,100,123,85,2,100,30))
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoview.R
\name{autoviewer}
\alias{autoviewer}
\alias{autoviewer_enable}
\alias{autoviewer_disable}
\title{RStudio Graphics AutoViewer}
\usage{
autoviewer_enable()

autoviewer_disable()
}
\description{
This enables a \link{addTaskCallback} that automatically updates the viewer after
the state of a magick graphics device has changed. This is enabled by default in
RStudio.
}
\examples{
# Only has effect in RStudio (or other GUI with a viewer):
autoviewer_enable()

img <- magick::image_graph()
plot(1)
abline(0, 1, col = "blue", lwd = 2, lty = "solid")
abline(0.1, 1, col = "red", lwd = 3, lty = "dotted")

autoviewer_disable()
abline(0.2, 1, col = "green", lwd = 4, lty = "twodash")
abline(0.3, 1, col = "black", lwd = 5, lty = "dotdash")

autoviewer_enable()
abline(0.4, 1, col = "purple", lwd = 6, lty = "dashed")
abline(0.5, 1, col = "yellow", lwd = 7, lty = "longdash")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EBImage.R
\name{as_EBImage}
\alias{as_EBImage}
\title{Convert to EBImage}
\usage{
as_EBImage(image)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}
}
\description{
Convert a Magick image to \href{https://bioconductor.org/packages/release/bioc/html/EBImage.html}{EBImage}
class. Note that EBImage only supports multi-frame images in greyscale.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{geometry}
\alias{geometry}
\alias{geometry_point}
\alias{geometry_area}
\alias{geometry_size_pixels}
\alias{geometry_size_percent}
\title{Geometry Helpers}
\usage{
geometry_point(x, y)

geometry_area(width = NULL, height = NULL, x_off = 0, y_off = 0)

geometry_size_pixels(width = NULL, height = NULL, preserve_aspect = TRUE)

geometry_size_percent(width = 100, height = NULL)
}
\arguments{
\item{x}{left offset in pixels}

\item{y}{top offset in pixels}

\item{width}{in pixels}

\item{height}{in pixels}

\item{x_off}{offset in pixels on x axis}

\item{y_off}{offset in pixels on y axis}

\item{preserve_aspect}{if FALSE, resize to width and height exactly, loosing original
aspect ratio. Only one of \code{percent} and \code{preserve_aspect} may be \code{TRUE}.}
}
\description{
ImageMagick uses a handy geometry syntax to specify coordinates and shapes
for use in image transformations. You can either specify these manually as
strings or use the helper functions below.
}
\details{
See \href{http://www.imagemagick.org/Magick++/Geometry.html}{ImageMagick Manual}
for details about the syntax specification.
Examples of \code{geometry} strings:
\itemize{
\item \strong{\code{"500x300"}}       -- \emph{Resize image keeping aspect ratio, such that width does not exceed 500 and the height does not exceed 300.}
\item \strong{\code{"500x300!"}}      -- \emph{Resize image to 500 by 300, ignoring aspect ratio}
\item \strong{\code{"500x"}}          -- \emph{Resize width to 500 keep aspect ratio}
\item \strong{\code{"x300"}}          -- \emph{Resize height to 300 keep aspect ratio}
\item \strong{\code{"50\%x20\%"}} -- \emph{Resize width to 50 percent and height to 20 percent of original}
\item \strong{\code{"500x300+10+20"}} -- \emph{Crop image to 500 by 300 at position 10,20}
}
}
\examples{
# Specify a point
logo <- image_read("logo:")
image_annotate(logo, "Some text", location = geometry_point(100, 200), size = 24)

# Specify image area
image_crop(logo, geometry_area(300, 300), repage = FALSE)
image_crop(logo, geometry_area(300, 300, 100, 100), repage = FALSE)

# Specify image size
image_resize(logo, geometry_size_pixels(300))
image_resize(logo, geometry_size_pixels(height = 300))
image_resize(logo, geometry_size_pixels(300, 300, preserve_aspect = FALSE))

# resize relative to current size
image_resize(logo, geometry_size_percent(50))
image_resize(logo, geometry_size_percent(50, 20))

}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defines.R
\name{defines}
\alias{defines}
\alias{image_set_defines}
\title{Set encoder defines}
\usage{
image_set_defines(image, defines)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{defines}{a named character vector with extra options to control reading.
These are the \verb{-define key\{=value\}} settings in the \href{http://www.imagemagick.org/script/command-line-options.php#define}{command line tool}.
Use an empty string for value-less defines, and NA to unset a define.}
}
\description{
So called 'defines' are properties that are passed along to external
filters and libraries. Usually defines are used in \link{image_read} or
\link{image_write} to control the image encoder/decoder, but you can also
set these manually on the image object.
}
\details{
The defines values must be a character string, where the names contain
the defines keys. Each name must be of the format "enc:key" where the
first part is the encoder or filter to which the key is passed. For
example \code{"png:...."} defines can control the encoding and decoding of
png images.

The \link{image_set_defines} function does not make a copy of the image, so
the defined values remain in the image object until they are overwritten
or unset.
}
\examples{
# Write an image
x <- image_read("https://jeroen.github.io/images/frink.png")
image_write(x, "frink.png")

# Pass some properties to PNG encoder
defines <- c("png:compression-filter" = "1", "png:compression-level" = "0")
image_set_defines(x, defines)
image_write(x, "frink-uncompressed.png")

# Unset properties
defines[1:2] = NA
image_set_defines(x, defines)
image_write(x, "frink-final.png")

# Compare size and cleanup
file.info(c("frink.png", "frink-uncompressed.png", "frink-final.png"))
unlink(c("frink.png", "frink-uncompressed.png", "frink-final.png"))
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paint.R
\name{painting}
\alias{painting}
\alias{image_fill}
\alias{image_annotate}
\title{Image Painting}
\usage{
image_fill(image, color, point = "+1+1", fuzz = 0, refcolor = NULL)

image_annotate(
  image,
  text,
  gravity = "northwest",
  location = "+0+0",
  degrees = 0,
  size = 10,
  font = "",
  style = "normal",
  weight = 400,
  kerning = 0,
  decoration = NULL,
  color = NULL,
  strokecolor = NULL,
  strokewidth = NULL,
  boxcolor = NULL
)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{color}{a valid \href{https://www.imagemagick.org/Magick++/Color.html}{color string} such as
\code{"navyblue"} or \code{"#000080"}. Use \code{"none"} for transparency.}

\item{point}{a geometry_point string indicating the starting point of the flood-fill}

\item{fuzz}{relative color distance (value between 0 and 100) to be considered similar
in the filling algorithm}

\item{refcolor}{if set, \code{fuzz} color distance will be measured against this color,
not the color of the starting \code{point}. Any color (within \code{fuzz} color distance of
the given \code{refcolor}), connected to starting point will be replaced with the \code{color}.
If the pixel at the starting point does not itself match the given \code{refcolor}
(according to \code{fuzz}) then no action will be taken.}

\item{text}{character vector of length equal to 'image' or length 1}

\item{gravity}{string with
\href{https://www.imagemagick.org/Magick++/Enumerations.html#GravityType}{gravity}
value from \link{gravity_types}.}

\item{location}{geometry string with location relative to \code{gravity}}

\item{degrees}{rotates text around center point}

\item{size}{font-size in pixels}

\item{font}{string with font family such as \code{"sans"}, \code{"mono"}, \code{"serif"},
\code{"Times"}, \code{"Helvetica"}, \code{"Trebuchet"}, \code{"Georgia"}, \code{"Palatino"} or \code{"Comic Sans"}.}

\item{style}{value of \link{style_types} for example \code{"italic"}}

\item{weight}{thickness of the font, 400 is normal and 700 is bold.}

\item{kerning}{increases or decreases whitespace between letters}

\item{decoration}{value of \link{decoration_types} for example \code{"underline"}}

\item{strokecolor}{a \href{https://www.imagemagick.org/Magick++/Color.html}{color string}
adds a stroke (border around the text)}

\item{strokewidth}{set the strokewidth of the border around the text}

\item{boxcolor}{a \href{https://www.imagemagick.org/Magick++/Color.html}{color string}
for background color that annotation text is rendered on.}
}
\description{
The \code{\link[=image_fill]{image_fill()}} function performs flood-fill by painting starting point and all
neighboring pixels of approximately the same color. Annotate prints some text on
the image.
}
\details{
Note that more sophisticated drawing mechanisms are available via the graphics
device using \link{image_draw}.

Setting a font, weight, style only works if your imagemagick is compiled
with fontconfig support.
}
\examples{
logo <- image_read("logo:")
logo <- image_background(logo, 'white')
image_fill(logo, "pink", point = "+450+400")
image_fill(logo, "pink", point = "+450+400", fuzz = 25)
# Add some text to an image
image_annotate(logo, "This is a test")
image_annotate(logo, "CONFIDENTIAL", size = 50, color = "red", boxcolor = "pink",
 degrees = 30, location = "+100+100")

# Setting fonts requires fontconfig support (and that you have the font)
image_annotate(logo, "The quick brown fox", font = "monospace", size = 50)
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paint.R
\name{painting}
\alias{painting}
\alias{image_fill}
\alias{image_annotate}
\title{Image Painting}
\usage{
image_fill(image, color, point = "+1+1", fuzz = 0, refcolor = NULL)

image_annotate(
  image,
  text,
  gravity = "northwest",
  location = "+0+0",
  degrees = 0,
  size = 10,
  font = "",
  style = "normal",
  weight = 400,
  kerning = 0,
  decoration = NULL,
  color = NULL,
  strokecolor = NULL,
  boxcolor = NULL
)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{color}{a valid \href{https://www.imagemagick.org/Magick++/Color.html}{color string} such as
\code{"navyblue"} or \code{"#000080"}. Use \code{"none"} for transparency.}

\item{point}{a geometry_point string indicating the starting point of the flood-fill}

\item{fuzz}{relative color distance (value between 0 and 100) to be considered similar
in the filling algorithm}

\item{refcolor}{if set, \code{fuzz} color distance will be measured against this color,
not the color of the starting \code{point}. Any color (within \code{fuzz} color distance of
the given \code{refcolor}), connected to starting point will be replaced with the \code{color}.
If the pixel at the starting point does not itself match the given \code{refcolor}
(according to \code{fuzz}) then no action will be taken.}

\item{text}{character vector of length equal to 'image' or length 1}

\item{gravity}{string with
\href{https://www.imagemagick.org/Magick++/Enumerations.html#GravityType}{gravity}
value from \link{gravity_types}.}

\item{location}{geometry string with location relative to \code{gravity}}

\item{degrees}{rotates text around center point}

\item{size}{font-size in pixels}

\item{font}{string with font family such as \code{"sans"}, \code{"mono"}, \code{"serif"},
\code{"Times"}, \code{"Helvetica"}, \code{"Trebuchet"}, \code{"Georgia"}, \code{"Palatino"} or \code{"Comic Sans"}.}

\item{style}{value of \link{style_types} for example \code{"italic"}}

\item{weight}{thickness of the font, 400 is normal and 700 is bold.}

\item{kerning}{increases or decreases whitespace between letters}

\item{decoration}{value of \link{decoration_types} for example \code{"underline"}}

\item{strokecolor}{a \href{https://www.imagemagick.org/Magick++/Color.html}{color string}
adds a stroke (border around the text)}

\item{boxcolor}{a \href{https://www.imagemagick.org/Magick++/Color.html}{color string}
for background color that annotation text is rendered on.}
}
\description{
The \code{\link[=image_fill]{image_fill()}} function performs flood-fill by painting starting point and all
neighboring pixels of approximately the same color. Annotate prints some text on
the image.
}
\details{
Note that more sophisticated drawing mechanisms are available via the graphics
device using \link{image_draw}.

Setting a font, weight, style only works if your imagemagick is compiled
with fontconfig support.
}
\examples{
logo <- image_read("logo:")
logo <- image_background(logo, 'white')
image_fill(logo, "pink", point = "+450+400")
image_fill(logo, "pink", point = "+450+400", fuzz = 25)
# Add some text to an image
image_annotate(logo, "This is a test")
image_annotate(logo, "CONFIDENTIAL", size = 50, color = "red", boxcolor = "pink",
 degrees = 30, location = "+100+100")

# Setting fonts requires fontconfig support (and that you have the font)
image_annotate(logo, "The quick brown fox", font = "monospace", size = 50)
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wizard.R
\docType{data}
\name{wizard}
\alias{wizard}
\alias{logo}
\alias{rose}
\alias{granite}
\title{Example Images}
\format{
An object of class \code{magick-image} of length 1.
}
\usage{
logo
}
\description{
Example images included with ImageMagick:
}
\details{
\itemize{
\item \code{logo}: ImageMagick Logo, 640x480
\item \code{wizard}: ImageMagick Wizard, 480x640
\item \code{rose} : Picture of a rose, 70x46
\item \code{granite} : Granite texture pattern, 128x128
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit.R
\name{editing}
\alias{editing}
\alias{image_read}
\alias{image_read_svg}
\alias{image_read_pdf}
\alias{image_read_video}
\alias{image_write}
\alias{image_convert}
\alias{image_data}
\alias{image_raster}
\alias{image_display}
\alias{image_browse}
\alias{image_strip}
\alias{image_blank}
\alias{image_destroy}
\alias{image_join}
\alias{image_attributes}
\alias{image_get_artifact}
\alias{demo_image}
\title{Image Editing}
\usage{
image_read(
  path,
  density = NULL,
  depth = NULL,
  strip = FALSE,
  coalesce = TRUE,
  defines = NULL
)

image_read_svg(path, width = NULL, height = NULL)

image_read_pdf(path, pages = NULL, density = 300, password = "")

image_read_video(path, fps = 1, format = "png")

image_write(
  image,
  path = NULL,
  format = NULL,
  quality = NULL,
  depth = NULL,
  density = NULL,
  comment = NULL,
  flatten = FALSE,
  defines = NULL,
  compression = NULL
)

image_convert(
  image,
  format = NULL,
  type = NULL,
  colorspace = NULL,
  depth = NULL,
  antialias = NULL,
  matte = NULL,
  interlace = NULL
)

image_data(image, channels = NULL, frame = 1)

image_raster(image, frame = 1, tidy = TRUE)

image_display(image, animate = TRUE)

image_browse(image, browser = getOption("browser"))

image_strip(image)

image_blank(width, height, color = "none", pseudo_image = "", defines = NULL)

image_destroy(image)

image_join(...)

image_attributes(image)

image_get_artifact(image, artifact = "")

demo_image(path)
}
\arguments{
\item{path}{a file, url, or raster object or bitmap array}

\item{density}{resolution to render pdf or svg}

\item{depth}{color depth (either 8 or 16)}

\item{strip}{drop image comments and metadata}

\item{coalesce}{automatically \code{\link[=image_coalesce]{image_coalesce()}} gif images}

\item{defines}{a named character vector with extra options to control reading.
These are the \verb{-define key\{=value\}} settings in the \href{http://www.imagemagick.org/script/command-line-options.php#define}{command line tool}.
Use an empty string for value-less defines, and NA to unset a define.}

\item{width}{in pixels}

\item{height}{in pixels}

\item{pages}{integer vector with page numbers. Defaults to all pages.}

\item{password}{user \link[pdftools:pdf_render_page]{password} to open protected pdf files}

\item{fps}{how many images to capture per second of video. Set to
\code{NULL} to get all frames from the input video.}

\item{format}{output format such as \code{"png"}, \code{"jpeg"}, \code{"gif"}, \code{"rgb"} or \code{"rgba"}.}

\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{quality}{number between 0 and 100 for jpeg quality. Defaults to 75.}

\item{comment}{text string added to the image metadata for supported formats}

\item{flatten}{should image be flattened before writing? This also replaces
transparency with background color.}

\item{compression}{a string with compression type from \link{compress_types}}

\item{type}{string with \href{https://www.imagemagick.org/Magick++/Enumerations.html#ImageType}{imagetype}
value from \link{image_types} for example \code{grayscale} to convert into black/white}

\item{colorspace}{string with a \href{https://www.imagemagick.org/Magick++/Enumerations.html#ColorspaceType}{\code{colorspace}}
from \link{colorspace_types} for example \code{"gray"}, \code{"rgb"} or \code{"cmyk"}}

\item{antialias}{enable anti-aliasing for text and strokes}

\item{matte}{set to \code{TRUE} or \code{FALSE} to enable or disable transparency}

\item{interlace}{string with \href{https://www.imagemagick.org/Magick++/Enumerations.html#InterlaceType}{interlace}}

\item{channels}{string with image channel(s) for example \code{"rgb"}, \code{"rgba"},
\code{"cmyk"},\code{"gray"}, or \code{"ycbcr"}. Default is either \code{"gray"}, \code{"rgb"} or \code{"rgba"}
depending on the image}

\item{frame}{integer setting which frame to extract from the image}

\item{tidy}{converts raster data to long form for use with \link[ggplot2:geom_tile]{geom_raster}.
If \code{FALSE} output is the same as \code{as.raster()}.}

\item{animate}{support animations in the X11 display}

\item{browser}{argument passed to \link[utils:browseURL]{browseURL}}

\item{color}{a valid \href{https://www.imagemagick.org/Magick++/Color.html}{color string} such as
\code{"navyblue"} or \code{"#000080"}. Use \code{"none"} for transparency.}

\item{pseudo_image}{string with \href{http://www.imagemagick.org/script/formats.php#pseudo}{pseudo image}
specification for example \code{"radial-gradient:purple-yellow"}}

\item{...}{several images or lists of images to be combined}

\item{artifact}{string with name of the artifact to extract, see the
\link{image_deskew} for an example.}
}
\description{
Read, write and join or combine images. All image functions are vectorized, meaning
they operate either on a single frame or a series of frames (e.g. a collage, video,
or animation). Besides paths and URLs, \code{\link[=image_read]{image_read()}} supports commonly used bitmap
and raster object types.
}
\details{
All standard base vector methods such as \link{[}, \link{[[}, \code{\link[=c]{c()}}, \code{\link[=as.list]{as.list()}},
\code{\link[=as.raster]{as.raster()}}, \code{\link[=rev]{rev()}}, \code{\link[=length]{length()}}, and \code{\link[=print]{print()}} can be used to work with magick
image objects. Use the standard \code{img[i]} syntax to extract a subset of the frames
from an image. The \code{img[[i]]} method is an alias for \code{\link[=image_data]{image_data()}} which extracts
a single frame as a raw bitmap matrix with pixel values.

For reading svg or pdf it is recommended to use \code{image_read_svg()} and \code{image_read_pdf()}
if the \link[rsvg:rsvg]{rsvg} and \link[pdftools:pdf_render_page]{pdftools} R packages are available.
These functions provide more rendering options (including rendering of literal svg) and
better quality than built-in svg/pdf rendering delegates from imagemagick itself.

X11 is required for \code{image_display()} which is only works on some platforms. A more
portable method is \code{image_browse()} which opens the image in a browser. RStudio has
an embedded viewer that does this automatically which is quite nice.

Image objects are automatically released by the garbage collector when they are no longer
reachable. Because the GC only runs once in a while, you can also call \code{image_destroy()}
explicitly to release the memory immediately. This is usually only needed if you create
a lot of images in a short period of time, and you might run out of memory.
}
\examples{
# Download image from the web
frink <- image_read("https://jeroen.github.io/images/frink.png")
worldcup_frink <- image_fill(frink, "orange", "+100+200", 20)
image_write(worldcup_frink, "output.png")

# extract raw bitmap array
bitmap <- frink[[1]]

# replace pixels with #FF69B4 ('hot pink') and convert back to image
bitmap[,50:100, 50:100] <- as.raw(c(0xff, 0x69, 0xb4, 0xff))
image_read(bitmap)

# Plot to graphics device via legacy raster format
raster <- as.raster(frink)
par(ask=FALSE)
plot(raster)

# Read bitmap arrays from from other image packages
curl::curl_download("https://jeroen.github.io/images/example.webp", "example.webp")
if(require(webp)) image_read(webp::read_webp("example.webp"))
unlink(c("example.webp", "output.png"))
if(require(rsvg)){
tiger <- image_read_svg("http://jeroen.github.io/images/tiger.svg")
svgtxt <- '<?xml version="1.0" encoding="UTF-8"?>
<svg width="400" height="400" viewBox="0 0 400 400" fill="none">
 <circle fill="steelblue" cx="200" cy="200" r="100" />
 <circle fill="yellow" cx="200" cy="200" r="90" />
</svg>'
circles <- image_read_svg(svgtxt)
}
if(require(pdftools))
image_read_pdf(file.path(R.home('doc'), 'NEWS.pdf'), pages = 1, density = 100)
# create a solid canvas
image_blank(600, 400, "green")
image_blank(600, 400, pseudo_image = "radial-gradient:purple-yellow")
image_blank(200, 200, pseudo_image = "gradient:#3498db-#db3a34",
  defines = c('gradient:direction' = 'east'))
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edit.R
\name{editing}
\alias{editing}
\alias{image_read}
\alias{image_read_svg}
\alias{image_read_pdf}
\alias{image_read_video}
\alias{image_write}
\alias{image_convert}
\alias{image_data}
\alias{image_raster}
\alias{image_display}
\alias{image_browse}
\alias{image_strip}
\alias{image_blank}
\alias{image_destroy}
\alias{image_join}
\alias{image_attributes}
\alias{image_get_artifact}
\alias{demo_image}
\title{Image Editing}
\usage{
image_read(
  path,
  density = NULL,
  depth = NULL,
  strip = FALSE,
  coalesce = TRUE,
  defines = NULL
)

image_read_svg(path, width = NULL, height = NULL)

image_read_pdf(path, pages = NULL, density = 300, password = "")

image_read_video(path, fps = 1, format = "png")

image_write(
  image,
  path = NULL,
  format = NULL,
  quality = NULL,
  depth = NULL,
  density = NULL,
  comment = NULL,
  flatten = FALSE,
  defines = NULL,
  compression = NULL
)

image_convert(
  image,
  format = NULL,
  type = NULL,
  colorspace = NULL,
  depth = NULL,
  antialias = NULL,
  matte = NULL,
  interlace = NULL
)

image_data(image, channels = NULL, frame = 1)

image_raster(image, frame = 1, tidy = TRUE)

image_display(image, animate = TRUE)

image_browse(image, browser = getOption("browser"))

image_strip(image)

image_blank(width, height, color = "none", pseudo_image = "", defines = NULL)

image_destroy(image)

image_join(...)

image_attributes(image)

image_get_artifact(image, artifact = "")

demo_image(path)
}
\arguments{
\item{path}{a file, url, or raster object or bitmap array}

\item{density}{resolution to render pdf or svg}

\item{depth}{color depth (either 8 or 16)}

\item{strip}{drop image comments and metadata}

\item{coalesce}{automatically \code{\link[=image_coalesce]{image_coalesce()}} gif images}

\item{defines}{a named character vector with extra options to control reading.
These are the \verb{-define key\{=value\}} settings in the \href{http://www.imagemagick.org/script/command-line-options.php#define}{command line tool}.
Use an empty string for value-less defines, and NA to unset a define.}

\item{width}{in pixels}

\item{height}{in pixels}

\item{pages}{integer vector with page numbers. Defaults to all pages.}

\item{password}{user \link[pdftools:pdf_render_page]{password} to open protected pdf files}

\item{fps}{how many images to capture per second of video. Set to
\code{NULL} to get all frames from the input video.}

\item{format}{output format such as \code{"png"}, \code{"jpeg"}, \code{"gif"}, \code{"rgb"} or \code{"rgba"}.}

\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{quality}{number between 0 and 100 for jpeg quality. Defaults to 75.}

\item{comment}{text string added to the image metadata for supported formats}

\item{flatten}{should image be flattened before writing? This also replaces
transparency with background color.}

\item{compression}{a string with compression type from \link{compress_types}}

\item{type}{string with \href{https://www.imagemagick.org/Magick++/Enumerations.html#ImageType}{imagetype}
value from \link{image_types} for example \code{grayscale} to convert into black/white}

\item{colorspace}{string with a \href{https://www.imagemagick.org/Magick++/Enumerations.html#ColorspaceType}{\code{colorspace}}
from \link{colorspace_types} for example \code{"gray"}, \code{"rgb"} or \code{"cmyk"}}

\item{antialias}{enable anti-aliasing for text and strokes}

\item{matte}{set to \code{TRUE} or \code{FALSE} to enable or disable transparency}

\item{interlace}{string with \href{https://www.imagemagick.org/Magick++/Enumerations.html#InterlaceType}{interlace}}

\item{channels}{string with image channel(s) for example \code{"rgb"}, \code{"rgba"},
\code{"cmyk"},\code{"gray"}, or \code{"ycbcr"}. Default is either \code{"gray"}, \code{"rgb"} or \code{"rgba"}
depending on the image}

\item{frame}{integer setting which frame to extract from the image}

\item{tidy}{converts raster data to long form for use with \link[ggplot2:geom_tile]{geom_raster}.
If \code{FALSE} output is the same as \code{as.raster()}.}

\item{animate}{support animations in the X11 display}

\item{browser}{argument passed to \link[utils:browseURL]{browseURL}}

\item{color}{a valid \href{https://www.imagemagick.org/Magick++/Color.html}{color string} such as
\code{"navyblue"} or \code{"#000080"}. Use \code{"none"} for transparency.}

\item{pseudo_image}{string with \href{http://www.imagemagick.org/script/formats.php#pseudo}{pseudo image}
specification for example \code{"radial-gradient:purple-yellow"}}

\item{...}{several images or lists of images to be combined}

\item{artifact}{string with name of the artifact to extract, see the
\link{image_deskew} for an example.}
}
\description{
Read, write and join or combine images. All image functions are vectorized, meaning
they operate either on a single frame or a series of frames (e.g. a collage, video,
or animation). Besides paths and URLs, \code{\link[=image_read]{image_read()}} supports commonly used bitmap
and raster object types.
}
\details{
All standard base vector methods such as \link{[}, \link{[[}, \code{\link[=c]{c()}}, \code{\link[=as.list]{as.list()}},
\code{\link[=as.raster]{as.raster()}}, \code{\link[=rev]{rev()}}, \code{\link[=length]{length()}}, and \code{\link[=print]{print()}} can be used to work with magick
image objects. Use the standard \code{img[i]} syntax to extract a subset of the frames
from an image. The \code{img[[i]]} method is an alias for \code{\link[=image_data]{image_data()}} which extracts
a single frame as a raw bitmap matrix with pixel values.

For reading svg or pdf it is recommended to use \code{image_read_svg()} and \code{image_read_pdf()}
if the \link[rsvg:rsvg]{rsvg} and \link[pdftools:pdf_render_page]{pdftools} R packages are available.
These functions provide more rendering options (including rendering of literal svg) and
better quality than built-in svg/pdf rendering delegates from imagemagick itself.

X11 is required for \code{image_display()} which is only works on some platforms. A more
portable method is \code{image_browse()} which opens the image in a browser. RStudio has
an embedded viewer that does this automatically which is quite nice.

Image objects are automatically released by the garbage collector when they are no longer
reachable. Because the GC only runs once in a while, you can also call \code{image_destroy()}
explicitly to release the memory immediately. This is usually only needed if you create
a lot of images in a short period of time, and you might run out of memory.
}
\examples{
# Download image from the web
frink <- image_read("https://jeroen.github.io/images/frink.png")
worldcup_frink <- image_fill(frink, "orange", "+100+200", 20)
image_write(worldcup_frink, "output.png")

# extract raw bitmap array
bitmap <- frink[[1]]

# replace pixels with #FF69B4 ('hot pink') and convert back to image
bitmap[,50:100, 50:100] <- as.raw(c(0xff, 0x69, 0xb4, 0xff))
image_read(bitmap)

# Plot to graphics device via legacy raster format
raster <- as.raster(frink)
par(ask=FALSE)
plot(raster)

# Read bitmap arrays from other image packages
download.file("https://jeroen.github.io/images/example.webp", "example.webp", mode = 'wb')
if(require(webp)) image_read(webp::read_webp("example.webp"))
unlink(c("example.webp", "output.png"))
if(require(rsvg)){
tiger <- image_read_svg("http://jeroen.github.io/images/tiger.svg")
svgtxt <- '<?xml version="1.0" encoding="UTF-8"?>
<svg width="400" height="400" viewBox="0 0 400 400" fill="none">
 <circle fill="steelblue" cx="200" cy="200" r="100" />
 <circle fill="yellow" cx="200" cy="200" r="90" />
</svg>'
circles <- image_read_svg(svgtxt)
}
if(require(pdftools))
image_read_pdf(file.path(R.home('doc'), 'NEWS.pdf'), pages = 1, density = 100)
# create a solid canvas
image_blank(600, 400, "green")
image_blank(600, 400, pseudo_image = "radial-gradient:purple-yellow")
image_blank(200, 200, pseudo_image = "gradient:#3498db-#db3a34",
  defines = c('gradient:direction' = 'east'))
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edges.R
\name{edges}
\alias{edges}
\alias{image_edge}
\alias{image_canny}
\alias{image_hough_draw}
\alias{image_hough_txt}
\title{Edge / Line Detection}
\usage{
image_edge(image, radius = 1)

image_canny(image, geometry = "0x1+10\%+30\%")

image_hough_draw(
  image,
  geometry = NULL,
  color = "red",
  bg = "transparent",
  size = 3,
  overlay = FALSE
)

image_hough_txt(image, geometry = NULL, format = c("mvg", "svg"))
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{radius}{edge size in pixels}

\item{geometry}{geometry string, see details.}

\item{color}{a valid \href{https://www.imagemagick.org/Magick++/Color.html}{color string} such as
\code{"navyblue"} or \code{"#000080"}. Use \code{"none"} for transparency.}

\item{bg}{background color}

\item{size}{size in points to draw the line}

\item{overlay}{composite the drawing atop the input image. Only for \code{bg = 'transparent'}.}

\item{format}{output format of the text, either \code{svg} or \code{mvg}}
}
\description{
Best results are obtained by finding edges with \code{\link[=image_canny]{image_canny()}} and
then performing Hough-line detection on the edge image.
}
\details{
For Hough-line detection, the geometry format is \verb{\{W\}x\{H\}+\{threshold\}}
defining the size and threshold of the filter used to find 'peaks' in
the intermediate search image. For canny edge detection the format is
\verb{\{radius\}x\{sigma\}+\{lower\%\}+\{upper\%\}}. More details and examples are
available at the \href{https://legacy.imagemagick.org/Usage/transform/}{imagemagick website}.
}
\examples{
if(magick_config()$version > "6.8.9"){
shape <- demo_image("shape_rectangle.gif")
rectangle <- image_canny(shape)
rectangle \%>\% image_hough_draw('5x5+20')
rectangle \%>\% image_hough_txt(format = 'svg') \%>\% cat()
}
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color.R
\name{color}
\alias{color}
\alias{image_modulate}
\alias{image_quantize}
\alias{image_map}
\alias{image_ordered_dither}
\alias{image_channel}
\alias{image_separate}
\alias{image_combine}
\alias{image_transparent}
\alias{image_background}
\alias{image_colorize}
\alias{image_contrast}
\alias{image_normalize}
\alias{image_enhance}
\alias{image_equalize}
\alias{image_median}
\title{Image Color}
\usage{
image_modulate(image, brightness = 100, saturation = 100, hue = 100)

image_quantize(
  image,
  max = 256,
  colorspace = "rgb",
  dither = TRUE,
  treedepth = NULL
)

image_map(image, map, dither = FALSE)

image_ordered_dither(image, threshold_map)

image_channel(image, channel = "lightness")

image_separate(image, channel = "default")

image_combine(image, colorspace = "sRGB", channel = "default")

image_transparent(image, color, fuzz = 0)

image_background(image, color, flatten = TRUE)

image_colorize(image, opacity, color)

image_contrast(image, sharpen = 1)

image_normalize(image)

image_enhance(image)

image_equalize(image)

image_median(image, radius = 1)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{brightness}{modulation of brightness as percentage of the current value (100 for no change)}

\item{saturation}{modulation of saturation as percentage of the current value (100 for no change)}

\item{hue}{modulation of hue is an absolute rotation of -180 degrees to +180 degrees from the
current position corresponding to an argument range of 0 to 200 (100 for no change)}

\item{max}{preferred number of colors in the image. The actual number of colors in the image may
be less than your request, but never more.}

\item{colorspace}{string with a \href{https://www.imagemagick.org/Magick++/Enumerations.html#ColorspaceType}{\code{colorspace}}
from \link{colorspace_types} for example \code{"gray"}, \code{"rgb"} or \code{"cmyk"}}

\item{dither}{a boolean (defaults to \code{TRUE}) specifying whether to apply Floyd/Steinberg error
diffusion to the image: averages intensities of several neighboring pixels}

\item{treedepth}{depth of the quantization color classification tree. Values of 0 or 1 allow
selection of the optimal tree depth for the color reduction algorithm. Values between 2 and 8
may be used to manually adjust the tree depth.}

\item{map}{reference image to map colors from}

\item{threshold_map}{A string giving the dithering pattern to use. See
\href{https://legacy.imagemagick.org/Usage/option_link.cgi?ordered-dither}{the ImageMagick documentation}
for possible values}

\item{channel}{a string with a
\href{https://www.imagemagick.org/Magick++/Enumerations.html#ChannelType}{channel} from
\link{channel_types} for example \code{"alpha"} or \code{"hue"} or \code{"cyan"}}

\item{color}{a valid \href{https://www.imagemagick.org/Magick++/Color.html}{color string} such as
\code{"navyblue"} or \code{"#000080"}. Use \code{"none"} for transparency.}

\item{fuzz}{relative color distance (value between 0 and 100) to be considered similar
in the filling algorithm}

\item{flatten}{should image be flattened before writing? This also replaces
transparency with background color.}

\item{opacity}{percentage of opacity used for coloring}

\item{sharpen}{enhance intensity differences in image}

\item{radius}{replace each pixel with the median color in a circular neighborhood}
}
\description{
Functions to adjust contrast, brightness, colors of the image. Details below.
}
\details{
For details see \href{https://www.imagemagick.org/Magick++/STL.html}{Magick++ STL}
documentation. Short descriptions:
\itemize{
\item \link{image_modulate} adjusts brightness, saturation and hue of image relative to current.
\item \link{image_quantize} reduces number of unique colors in the image.
\item \link{image_ordered_dither} reduces number of unique colors using a dithering threshold map.
\item \link{image_map} replaces colors of image with the closest color from a reference image.
\item \link{image_channel} extracts a single channel from an image and returns as grayscale.
\item \link{image_transparent} sets pixels approximately matching given color to transparent.
\item \link{image_background} sets background color. When image is flattened, transparent pixels get background color.
\item \link{image_colorize} overlays a solid color frame using specified opacity.
\item \link{image_contrast} enhances intensity differences in image
\item \link{image_normalize} increases contrast by normalizing the pixel values to span the full range of colors
\item \link{image_enhance} tries to minimize noise
\item \link{image_equalize} equalizes using histogram equalization
\item \link{image_median} replaces each pixel with the median color in a circular neighborhood
}

Note that
colors are also determined by image properties
\href{https://www.imagemagick.org/Magick++/Enumerations.html#ImageType}{imagetype} and
\href{https://www.imagemagick.org/Magick++/Enumerations.html#ColorspaceType}{colorspace}
which can be modified via \code{\link[=image_convert]{image_convert()}}.
}
\examples{
# manually adjust colors
logo <- image_read("logo:")
image_modulate(logo, brightness = 200)
image_modulate(logo, saturation = 150)
image_modulate(logo, hue = 200)

# Reduce image to 10 different colors using various spaces
image_quantize(logo, max = 10, colorspace = 'gray')
image_quantize(logo, max = 10, colorspace = 'rgb')
image_quantize(logo, max = 10, colorspace = 'cmyk')

image_ordered_dither(logo, 'o8x8')
# Change background color
translogo <- image_transparent(logo, 'white')
image_background(translogo, "pink", flatten = TRUE)

# Compare to flood-fill method:
image_fill(logo, "pink", fuzz = 20)

# Other color tweaks
image_colorize(logo, 50, "red")
image_contrast(logo)
image_normalize(logo)
image_enhance(logo)
image_equalize(logo)
image_median(logo)

# Alternate way to convert into black-white
image_convert(logo, type = 'grayscale')
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/device.R
\name{device}
\alias{device}
\alias{image_graph}
\alias{image_device}
\alias{image_draw}
\alias{image_capture}
\title{Magick Graphics Device}
\usage{
image_graph(
  width = 800,
  height = 600,
  bg = "white",
  pointsize = 12,
  res = 72,
  clip = TRUE,
  antialias = TRUE
)

image_draw(image, pointsize = 12, res = 72, antialias = TRUE, ...)

image_capture()
}
\arguments{
\item{width}{in pixels}

\item{height}{in pixels}

\item{bg}{background color}

\item{pointsize}{size of fonts}

\item{res}{resolution in pixels}

\item{clip}{enable clipping in the device. Because clipping can slow things down
a lot, you can disable it if you don't need it.}

\item{antialias}{TRUE/FALSE: enables anti-aliasing for text and strokes}

\item{image}{an existing image on which to start drawing}

\item{...}{additional device parameters passed to \link{plot.window} such as
\code{xlim}, \code{ylim}, or \code{mar}.}
}
\description{
Graphics device that produces a Magick image. Can either be used like a regular
device for making plots, or alternatively via \code{image_draw} to open a device
which draws onto an existing image using pixel coordinates. The latter is vectorized,
i.e. drawing operations are applied to each frame in the image.
}
\details{
The device is a relatively recent feature of the package. It should support all
operations but there might still be small inaccuracies. Also it is a bit slower than
some of the other devices, in particular for rendering text and clipping. Hopefully
this can be optimized in the next version.

By default \code{image_draw} sets all margins to 0 and uses graphics coordinates to
match image size in pixels (width x height) where \code{(0,0)} is the top left corner.
Note that this means the y axis increases from top to bottom which is the opposite
of typical graphics coordinates.  You can override all this by passing custom
\code{xlim}, \code{ylim} or \code{mar} values to \code{image_draw}.

The \code{image_capture} function returns the current device as an image. This only
works if the current device is a magick device or supports \link{dev.capture}.
}
\examples{
# Regular image
frink <- image_read("https://jeroen.github.io/images/frink.png")

# Produce image using graphics device
fig <- image_graph(res = 96)
ggplot2::qplot(mpg, wt, data = mtcars, colour = cyl)
dev.off()

# Combine
out <- image_composite(fig, frink, offset = "+70+30")
print(out)

# Or paint over an existing image
img <- image_draw(frink)
rect(20, 20, 200, 100, border = "red", lty = "dashed", lwd = 5)
abline(h = 300, col = 'blue', lwd = '10', lty = "dotted")
text(10, 250, "Hoiven-Glaven", family = "monospace", cex = 4, srt = 90)
palette(rainbow(11, end = 0.9))
symbols(rep(200, 11), seq(0, 400, 40), circles = runif(11, 5, 35),
  bg = 1:11, inches = FALSE, add = TRUE)
dev.off()
print(img)

\donttest{
# Vectorized example with custom coordinates
earth <- image_read("https://jeroen.github.io/images/earth.gif")
img <- image_draw(earth, xlim = c(0,1), ylim = c(0,1))
rect(.1, .1, .9, .9, border = "red", lty = "dashed", lwd = 5)
text(.5, .9, "Our planet", cex = 3, col = "white")
dev.off()
print(img)
}
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/video.R
\name{video}
\alias{video}
\alias{image_write_video}
\alias{image_write_gif}
\title{Write Video}
\usage{
image_write_video(image, path = NULL, framerate = 10, ...)

image_write_gif(image, path = NULL, delay = 1/10, ...)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{path}{filename of the output gif or video. This is also the return value.}

\item{framerate}{frames per second, passed to \link[av:encoding]{av_encode_video}}

\item{...}{additional parameters passed to \link[av:encoding]{av_encode_video} and
\link[gifski:gifski]{gifski}.}

\item{delay}{duration of each frame in seconds (inverse of framerate)}
}
\description{
High quality video / gif exporter based on external packages \link[gifski:gifski]{gifski}
and \link[av:encoding]{av}.
}
\details{
This requires an image with multiple frames. The GIF exporter accomplishes the same
thing as \link{image_animate} but much faster and with better quality.
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/index.R
\name{_index_}
\alias{_index_}
\alias{magick}
\alias{magick-package}
\alias{imagemagick}
\title{Magick Image Processing}
\description{
The \code{magick} package for graphics and image processing in R. Important resources:
\itemize{
\item \href{https://docs.ropensci.org/magick/articles/intro.html}{R introduction vignette}: getting started
\item \href{https://www.imagemagick.org/Magick++/Image++.html}{Magick++ API} and
\href{https://www.imagemagick.org/Magick++/STL.html}{Magick++ STL} detailed descriptions of methods and parameters
}
}
\details{
Documentation is split into the following pages:
\itemize{
\item \link{analysis} - metrics and calculations: \code{compare}, \code{fft}
\item \link{animation} - manipulate or combine multiple frames: \code{animate},
\code{morph}, \code{mosaic}, \code{montage}, \code{average}, \code{append}, \code{apply}
\item \link{attributes} - image properties: \code{comment}, \code{info}
\item \link{color} - contrast, brightness, colors: \code{modulate}, \code{quantize}, \code{map}, \code{transparent},
\code{background}, \code{colorize}, \code{contrast}, \code{normalize}, \code{enhance}, \code{equalize}, \code{median}
\item \link{composite} - advanced joining: \code{composite}, \code{border}, \code{frame}
\item \link{device} - creating graphics and drawing on images
\item \link{editing} - basic image IO: \code{read}, \code{write}, \code{convert}, \code{join}, \code{display}, \code{brose}
\item \link{effects} - fun effects: \code{despecle}, \code{reducenoise}, \code{noise}, \code{blur}, \code{charcoal},
\code{edge}, \code{oilpaint}, \code{emboss}, \code{implode}, \code{negate}
\item \link{geometry} - specify points, areas and sizes using geometry syntax
\item \link{ocr} - extract text from image using \link[tesseract:tesseract]{tesseract} package
\item \link{options} - list option types and values supported in your version of ImageMagick
\item \link{painting} - flood fill and annotating text
\item \link{transform} - shape operations: \code{trim}, \code{chop}, \code{rotate}, \code{resize}, \code{scale}, \code{sample}
\code{crop}, \code{flip}, \code{flop}, \code{deskew}, \code{page}
}
}
\seealso{
Other image: 
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/segmentation.R
\name{segmentation}
\alias{segmentation}
\alias{image_connect}
\alias{image_split}
\alias{image_fuzzycmeans}
\title{Image Segmentation}
\usage{
image_connect(image, connectivity = 4)

image_split(image, keep_color = TRUE)

image_fuzzycmeans(image, min_pixels = 1, smoothing = 1.5)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{connectivity}{number neighbor colors which are considered part of a unique object}

\item{keep_color}{if TRUE the output images retain the color of the input pixel.
If FALSE all matching pixels are set black to retain only the image mask.}

\item{min_pixels}{the minimum number of pixels contained in a hexahedra before it can be considered valid (expressed as a percentage)}

\item{smoothing}{the smoothing threshold which eliminates noise in the second derivative of the histogram (higher values gives smoother second derivative)}
}
\description{
Basic image segmentation like connected components labelling, blob extraction and fuzzy c-means
}
\details{
\itemize{
\item \link{image_connect} Connect adjacent pixels with the same pixel intensities to do blob extraction
\item \link{image_split} Splits the image according to pixel intensities
\item \link{image_fuzzycmeans} Fuzzy c-means segmentation of the histogram of color components
}

\link{image_connect} performs blob extraction by scanning the image, pixel-by-pixel from top-left
to bottom-right where regions of adjacent pixels which share the same set of intensity values
get combined.
}
\examples{
# Split an image by color
img <- image_quantize(logo, 4)
layers <- image_split(img)
layers

# This returns the original image
image_flatten(layers)

# From the IM website
objects <- image_convert(demo_image("objects.gif"), colorspace = "Gray")
objects

\donttest{
# Split image in blobs of connected pixel levels
if(magick_config()$version > "6.9.0"){
objects \%>\%
  image_connect(connectivity = 4) \%>\%
  image_split()

# Fuzzy c-means
image_fuzzycmeans(logo)

logo \%>\%
  image_convert(colorspace = "HCL") \%>\%
  image_fuzzycmeans(smoothing = 5)
}
}
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/threshold.R
\name{thresholding}
\alias{thresholding}
\alias{image_threshold}
\alias{image_level}
\alias{image_lat}
\title{Image thresholding}
\usage{
image_threshold(
  image,
  type = c("black", "white"),
  threshold = "50\%",
  channel = NULL
)

image_level(
  image,
  black_point = 0,
  white_point = 100,
  mid_point = 1,
  channel = NULL
)

image_lat(image, geometry = "10x10+5\%")
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{type}{type of thresholding, either one of lat, black or white (see details below)}

\item{threshold}{pixel intensity threshold percentage for black or white thresholding}

\item{channel}{a value of \code{\link[=channel_types]{channel_types()}} specifying which channel(s) to set}

\item{black_point}{value between 0 and 100, the darkest color in the image}

\item{white_point}{value between 0 and 100, the lightest color in the image}

\item{mid_point}{value between 0 and 10 used for gamma correction}

\item{geometry}{pixel window plus offset for LAT algorithm}
}
\description{
Thresholding an image can be used for simple and straightforward image segmentation.
The function \code{\link[=image_threshold]{image_threshold()}} allows to do black and white thresholding whereas
\code{\link[=image_lat]{image_lat()}} performs local adaptive thresholding.
}
\details{
\itemize{
\item \code{image_threshold(type = "black")}: Forces all pixels below the threshold into black while leaving all pixels
at or above the threshold unchanged
\item \code{image_threshold(type = "white")}: Forces all pixels above the threshold into white while leaving all pixels
at or below the threshold unchanged
\item \code{image_lat()}: Local Adaptive Thresholding. Looks in a box (width x height) around the
pixel neighborhood if the pixel value is bigger than the average minus an offset.
}
}
\examples{
test <- image_convert(logo, colorspace = "Gray")
image_threshold(test, type = "black", threshold = "50\%")
image_threshold(test, type = "white", threshold = "50\%")

# Turn image into BW
test \%>\%
  image_threshold(type = "white", threshold = "50\%") \%>\%
  image_threshold(type = "black", threshold = "50\%")

# adaptive thresholding
image_lat(test, geometry = '10x10+5\%')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/attributes.R
\name{attributes}
\alias{attributes}
\alias{image_comment}
\alias{image_info}
\title{Image Attributes}
\usage{
image_comment(image, comment = NULL)

image_info(image)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{comment}{string to set an image comment}
}
\description{
Attributes are properties of the image that might be present on some images
and might affect image manipulation methods.
}
\details{
Each attribute can be get and set with the same function. The \code{\link[=image_info]{image_info()}}
function returns a data frame with some commonly used attributes.
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{options}
\alias{options}
\alias{magick_options}
\alias{option_types}
\alias{filter_types}
\alias{metric_types}
\alias{dispose_types}
\alias{compose_types}
\alias{colorspace_types}
\alias{channel_types}
\alias{image_types}
\alias{kernel_types}
\alias{noise_types}
\alias{gravity_types}
\alias{orientation_types}
\alias{morphology_types}
\alias{style_types}
\alias{decoration_types}
\alias{compress_types}
\alias{distort_types}
\title{Magick Options}
\usage{
magick_options()

option_types()

filter_types()

metric_types()

dispose_types()

compose_types()

colorspace_types()

channel_types()

image_types()

kernel_types()

noise_types()

gravity_types()

orientation_types()

morphology_types()

style_types()

decoration_types()

compress_types()

distort_types()
}
\description{
List option types and values supported in your version of ImageMagick. For
descriptions see
\href{https://www.imagemagick.org/Magick++/Enumerations.html}{ImageMagick Enumerations}.
}
\references{
ImageMagick Manual: \href{https://www.imagemagick.org/Magick++/Enumerations.html}{Enumerations}
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{morphology}},
\code{\link{ocr}},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/morphology.R
\name{morphology}
\alias{morphology}
\alias{image_morphology}
\alias{image_convolve}
\title{Morphology}
\usage{
image_morphology(
  image,
  method = "convolve",
  kernel = "Gaussian",
  iterations = 1,
  opts = list()
)

image_convolve(
  image,
  kernel = "Gaussian",
  iterations = 1,
  scaling = NULL,
  bias = NULL
)
}
\arguments{
\item{image}{magick image object returned by \code{\link[=image_read]{image_read()}} or \code{\link[=image_graph]{image_graph()}}}

\item{method}{a string with a valid method from \code{\link[=morphology_types]{morphology_types()}}}

\item{kernel}{either a square matrix or a string. The string can either be a
parameterized \link[=kernel_types]{kerneltype} such as: \code{"DoG:0,0,2"} or \code{"Diamond"}
or it can contain a custom matrix (see examples)}

\item{iterations}{number of iterations}

\item{opts}{a named list or character vector with custom attributes}

\item{scaling}{string with kernel scaling. The special flag \code{"!"} automatically scales to full
dynamic range, for example: \code{"50\%!"}}

\item{bias}{output bias string, for example \code{"50\%"}}
}
\description{
Apply a morphology method. This is a very flexible function which
can be used to apply any morphology method with custom parameters.
See \href{https://legacy.imagemagick.org/Usage/morphology/}{imagemagick website}
for examples.
}
\examples{
#example from IM website:
if(magick_config()$version > "6.8.8"){
pixel <- image_blank(1, 1, 'white') \%>\% image_border('black', '5x5')

# See the effect of Dilate method
pixel \%>\% image_scale('800\%')
pixel \%>\% image_morphology('Dilate', "Diamond") \%>\% image_scale('800\%')

# These produce the same output:
pixel \%>\% image_morphology('Dilate', "Diamond", iter = 3) \%>\% image_scale('800\%')
pixel \%>\% image_morphology('Dilate', "Diamond:3") \%>\% image_scale('800\%')

# Plus example
pixel \%>\% image_morphology('Dilate', "Plus", iterations = 2) \%>\% image_scale('800\%')

# Rose examples
rose \%>\% image_morphology('ErodeI', 'Octagon', iter = 3)
rose \%>\% image_morphology('DilateI', 'Octagon', iter = 3)
rose \%>\% image_morphology('OpenI', 'Octagon', iter = 3)
rose \%>\% image_morphology('CloseI', 'Octagon', iter = 3)

# Edge detection
man <- demo_image('man.gif')
man \%>\% image_morphology('EdgeIn', 'Octagon')
man \%>\% image_morphology('EdgeOut', 'Octagon')
man \%>\% image_morphology('Edge', 'Octagon')

# Octagonal Convex Hull
 man \%>\%
   image_morphology('Close', 'Diamond') \%>\%
   image_morphology('Thicken', 'ConvexHull', iterations = 1)

# Thinning down to a Skeleton
man \%>\% image_morphology('Thinning', 'Skeleton', iterations = 1)

# Specify custom kernel matrix usingn a string:
img <- demo_image("test_mag.gif")
i <- image_convolve(img, kernel = '4x5:
       0 -1  0  0
      -1 +1 -1  0
      -1 +1 -1  0
      -1 +1 +1 -1
       0 -1 -1  0 ', bias = "50\%")
}
}
\seealso{
Other image: 
\code{\link{_index_}},
\code{\link{analysis}},
\code{\link{animation}},
\code{\link{attributes}()},
\code{\link{color}},
\code{\link{composite}},
\code{\link{defines}},
\code{\link{device}},
\code{\link{edges}},
\code{\link{editing}},
\code{\link{effects}()},
\code{\link{fx}},
\code{\link{geometry}},
\code{\link{ocr}},
\code{\link{options}()},
\code{\link{painting}},
\code{\link{segmentation}},
\code{\link{transform}()},
\code{\link{video}}
}
\concept{image}
